@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2010  Nexedi SA
+# Copyright (C) 2006-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,21 +12,16 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from ZODB import BaseStorage, ConflictResolution, POSException
 from zope.interface import implements
 import ZODB.interfaces
 
-import neo.lib
 from functools import wraps
-from neo.lib import setupLog
-from neo.lib.util import add64
-from neo.lib.protocol import ZERO_TID
-from neo.client.app import Application
-from neo.client.exception import NEOStorageNotFoundError
-from neo.client.exception import NEOStorageDoesNotExistError
+from neo.lib import logging
+from .app import Application
+from .exception import NEOStorageNotFoundError, NEOStorageDoesNotExistError
 
 def check_read_only(func):
     def wrapped(self, *args, **kw):
@@ -35,33 +30,10 @@ def check_read_only(func):
         return func(self, *args, **kw)
     return wraps(func)(wrapped)
 
-def old_history_api(func):
-    try:
-        if ZODB.interfaces.IStorage['history'].positional[1] != 'version':
-            return func # ZODB >= 3.9
-    except KeyError: # ZODB < 3.8
-        pass
-    def history(self, oid, version=None, *args, **kw):
-        if version is None:
-            return func(self, oid, *args, **kw)
-        raise ValueError('Versions are not supported')
-    return wraps(func)(history)
-
 class Storage(BaseStorage.BaseStorage,
               ConflictResolution.ConflictResolvingStorage):
     """Wrapper class for neoclient."""
 
-    # Stores the highest TID visible for current transaction.
-    # First call sets this snapshot by asking master node most recent
-    # committed TID.
-    # As a (positive) side-effect, this forces us to handle all pending
-    # invalidations, so we get a very recent view of the database (which is
-    # good when multiple databases are used in the same program with some
-    # amount of referential integrity).
-    # Should remain None when not bound to a connection,
-    # so that it always read the last revision.
-    _snapshot_tid = None
-
     implements(*filter(None, (
         ZODB.interfaces.IStorage,
         # "restore" missing for the moment, but "store" implements this
@@ -72,35 +44,26 @@ class Storage(BaseStorage.BaseStorage,
         # - transaction isolation is not done
         # ZODB.interfaces.IStorageIteration,
         ZODB.interfaces.IStorageUndoable,
-        getattr(ZODB.interfaces, 'IExternalGC', None), # XXX ZODB < 3.9
-        getattr(ZODB.interfaces, 'ReadVerifyingStorage', None), # XXX ZODB 3.9
+        ZODB.interfaces.IExternalGC,
+        getattr(ZODB.interfaces, 'ReadVerifyingStorage', None), # BBB ZODB 3.9
     )))
 
     def __init__(self, master_nodes, name, read_only=False,
-            compress=None, logfile=None, verbose=False, _app=None, **kw):
+            compress=None, logfile=None, _app=None, **kw):
         """
         Do not pass those parameters (used internally):
         _app
-        _cache
         """
         if compress is None:
             compress = True
-        setupLog('CLIENT', filename=logfile, verbose=verbose)
+        if logfile:
+            logging.setup(logfile)
         BaseStorage.BaseStorage.__init__(self, 'NEOStorage(%s)' % (name, ))
         # Warning: _is_read_only is used in BaseStorage, do not rename it.
         self._is_read_only = read_only
         if _app is None:
-            _app = Application(master_nodes, name, compress=compress)
+            _app = Application(master_nodes, name, compress=compress, **kw)
         self.app = _app
-        # Used to clone self (see new_instance & IMVCCStorage definition).
-        self._init_args = (master_nodes, name)
-        self._init_kw = {
-            'read_only': read_only,
-            'compress': compress,
-            'logfile': logfile,
-            'verbose': verbose,
-            '_app': _app,
-        }
 
     @property
     def _cache(self):
@@ -119,7 +82,7 @@ class Storage(BaseStorage.BaseStorage,
         # it optional.
         assert version == '', 'Versions are not supported'
         try:
-            return self.app.load(oid, None, self._snapshot_tid)[:2]
+            return self.app.load(oid)[:2]
         except NEOStorageNotFoundError:
             raise POSException.POSKeyError(oid)
 
@@ -132,38 +95,27 @@ class Storage(BaseStorage.BaseStorage,
         """
         Note: never blocks in NEO.
         """
-        return self.app.tpc_begin(transaction=transaction, tid=tid,
-                status=status)
+        return self.app.tpc_begin(transaction, tid, status)
 
     @check_read_only
     def tpc_vote(self, transaction):
-        return self.app.tpc_vote(transaction=transaction,
-            tryToResolveConflict=self.tryToResolveConflict)
+        return self.app.tpc_vote(transaction, self.tryToResolveConflict)
 
     @check_read_only
     def tpc_abort(self, transaction):
-        return self.app.tpc_abort(transaction=transaction)
+        return self.app.tpc_abort(transaction)
 
     def tpc_finish(self, transaction, f=None):
-        tid = self.app.tpc_finish(transaction=transaction,
-            tryToResolveConflict=self.tryToResolveConflict, f=f)
-        # XXX: Note that when undoing changes, the following is useless because
-        #      a temporary Storage object is used to commit.
-        #      See also testZODB.NEOZODBTests.checkMultipleUndoInOneTransaction
-        if self._snapshot_tid:
-            self._snapshot_tid = add64(tid, 1)
-        return tid
+        return self.app.tpc_finish(transaction, self.tryToResolveConflict, f)
 
     @check_read_only
     def store(self, oid, serial, data, version, transaction):
         assert version == '', 'Versions are not supported'
-        return self.app.store(oid=oid, serial=serial,
-            data=data, version=version, transaction=transaction)
+        return self.app.store(oid, serial, data, version, transaction)
 
     @check_read_only
     def deleteObject(self, oid, serial, transaction):
-        self.app.store(oid=oid, serial=serial, data='', version=None,
-            transaction=transaction)
+        self.app.store(oid, serial, None, None, transaction)
 
     # mutliple revisions
     def loadSerial(self, oid, serial):
@@ -183,18 +135,14 @@ class Storage(BaseStorage.BaseStorage,
     def iterator(self, start=None, stop=None):
         # Iterator lives in its own transaction, so get a fresh snapshot.
         snapshot_tid = self.lastTransaction()
-        if stop is None:
+        if not stop or snapshot_tid < stop:
             stop = snapshot_tid
-        else:
-            stop = min(snapshot_tid, stop)
         return self.app.iterator(start, stop)
 
     # undo
     @check_read_only
     def undo(self, transaction_id, txn):
-        return self.app.undo(self._snapshot_tid, undone_tid=transaction_id,
-            txn=txn, tryToResolveConflict=self.tryToResolveConflict)
-
+        return self.app.undo(transaction_id, txn, self.tryToResolveConflict)
 
     @check_read_only
     def undoLog(self, first=0, last=-20, filter=None):
@@ -216,18 +164,17 @@ class Storage(BaseStorage.BaseStorage,
 
     def loadEx(self, oid, version):
         try:
-            data, serial, _ = self.app.load(oid, None, self._snapshot_tid)
+            data, serial, _ = self.app.load(oid)
         except NEOStorageNotFoundError:
             raise POSException.POSKeyError(oid)
         return data, serial, ''
 
     def __len__(self):
-        return self.app.getStorageSize()
+        return self.app.getObjectCount()
 
     def registerDB(self, db, limit=None):
         self.app.registerDB(db, limit)
 
-    @old_history_api
     def history(self, oid, *args, **kw):
         try:
             return self.app.history(oid, *args, **kw)
@@ -235,28 +182,25 @@ class Storage(BaseStorage.BaseStorage,
             raise POSException.POSKeyError(oid)
 
     def sync(self, force=True):
-        # Increment by one, as we will use this as an excluded upper
-        # bound (loadBefore).
-        self._snapshot_tid = add64(self.lastTransaction(), 1)
+        # XXX: sync() is part of IMVCCStorage and we don't want to be called
+        #      from afterCompletion() so it may not be a good place to ping the
+        #      master here. See also monkey-patch in __init__.py
+        self.app.lastTransaction()
 
     def copyTransactionsFrom(self, source, verbose=False):
         """ Zope compliant API """
         return self.app.importFrom(source, None, None,
                 self.tryToResolveConflict)
 
-    def importFrom(self, source, start=None, stop=None):
+    def importFrom(self, source, start=None, stop=None, preindex=None):
         """ Allow import only a part of the source storage """
         return self.app.importFrom(source, start, stop,
-                self.tryToResolveConflict)
-
-    def restore(self, oid, serial, data, version, prev_txn, transaction):
-        raise NotImplementedError
+                self.tryToResolveConflict, preindex)
 
     def pack(self, t, referencesf, gc=False):
         if gc:
-            neo.lib.logging.warning(
-                'Garbage Collection is not available in NEO, '
-                'please use an external tool. Packing without GC.')
+            logging.warning('Garbage Collection is not available in NEO,'
+                ' please use an external tool. Packing without GC.')
         self.app.pack(t)
 
     def lastSerial(self):
@@ -290,6 +234,3 @@ class Storage(BaseStorage.BaseStorage,
 
     def checkCurrentSerialInTransaction(self, oid, serial, transaction):
         self.app.checkCurrentSerialInTransaction(oid, serial, transaction)
-
-    def new_instance(self):
-        return Storage(*self._init_args, **self._init_kw)
