@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2010  Nexedi SA
+# Copyright (C) 2006-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,22 +12,21 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from cPickle import dumps, loads
 from zlib import compress as real_compress, decompress
 from neo.lib.locking import Empty
 from random import shuffle
+import heapq
 import time
-import os
 
 from ZODB.POSException import UndoError, StorageTransactionError, ConflictError
 from ZODB.POSException import ReadConflictError
 from ZODB.ConflictResolution import ResolvedSerial
 from persistent.TimeStamp import TimeStamp
 
-import neo.lib
+from neo.lib import logging
 from neo.lib.protocol import NodeTypes, Packets, \
     INVALID_PARTITION, ZERO_HASH, ZERO_TID
 from neo.lib.event import EventManager
@@ -36,19 +35,18 @@ from neo.lib.locking import Lock
 from neo.lib.connection import MTClientConnection, OnTimeout, ConnectionClosed
 from neo.lib.node import NodeManager
 from neo.lib.connector import getConnectorHandler
-from neo.client.exception import NEOStorageError, NEOStorageCreationUndoneError
-from neo.client.exception import NEOStorageNotFoundError
-from neo.lib.exception import NeoException
-from neo.client.handlers import storage, master
+from .exception import NEOStorageError, NEOStorageCreationUndoneError
+from .exception import NEOStorageNotFoundError
+from .handlers import storage, master
 from neo.lib.dispatcher import Dispatcher, ForgottenPacket
-from neo.client.poll import ThreadedPoll, psThreadedPoll
-from neo.client.iterator import Iterator
-from neo.client.cache import ClientCache
-from neo.client.pool import ConnectionPool
-from neo.lib.util import u64, parseMasterList
+from .poll import ThreadedPoll, psThreadedPoll
+from .iterator import Iterator
+from .cache import ClientCache
+from .pool import ConnectionPool
+from neo.lib.util import p64, u64, parseMasterList
 from neo.lib.profiling import profiler_decorator, PROFILING_ENABLED
 from neo.lib.debug import register as registerLiveDebugger
-from neo.client.container import ThreadContainer, TransactionContainer
+from .container import ThreadContainer, TransactionContainer
 
 if PROFILING_ENABLED:
     # Those functions require a "real" python function wrapper before they can
@@ -65,13 +63,14 @@ else:
     compress = real_compress
     makeChecksum = real_makeChecksum
 
-CHECKED_SERIAL = object()
+CHECKED_SERIAL = master.CHECKED_SERIAL
 
 
 class Application(object):
     """The client node application."""
 
-    def __init__(self, master_nodes, name, compress=True, **kw):
+    def __init__(self, master_nodes, name, compress=True,
+            dynamic_master_list=None):
         # Start polling thread
         self.em = EventManager()
         self.poll_thread = ThreadedPoll(self.em, name=name)
@@ -82,7 +81,7 @@ class Application(object):
         master_addresses, connector_name = parseMasterList(master_nodes)
         self.connector_handler = getConnectorHandler(connector_name)
         self.dispatcher = Dispatcher(self.poll_thread)
-        self.nm = NodeManager()
+        self.nm = NodeManager(dynamic_master_list)
         self.cp = ConnectionPool(self)
         self.pt = None
         self.master_conn = None
@@ -96,8 +95,10 @@ class Application(object):
         # no self-assigned UUID, primary master will supply us one
         self.uuid = None
         self._cache = ClientCache()
+        self._loading_oid = None
         self.new_oid_list = []
         self.last_oid = '\0' * 8
+        self.last_tid = None
         self.storage_event_handler = storage.StorageEventHandler(self)
         self.storage_bootstrap_handler = storage.StorageBootstrapHandler(self)
         self.storage_handler = storage.StorageAnswersHandler(self)
@@ -149,7 +150,7 @@ class Application(object):
             self.pt.log()
 
     @profiler_decorator
-    def _handlePacket(self, conn, packet, handler=None):
+    def _handlePacket(self, conn, packet, kw={}, handler=None):
         """
           conn
             The connection which received the packet (forwarded to handler).
@@ -174,7 +175,7 @@ class Application(object):
                 raise ValueError, 'Unknown node type: %r' % (node.__class__, )
         conn.lock()
         try:
-            handler.dispatch(conn, packet)
+            handler.dispatch(conn, packet, kw)
         finally:
             conn.unlock()
 
@@ -191,7 +192,7 @@ class Application(object):
         _handlePacket = self._handlePacket
         while pending(queue):
             try:
-                conn, packet = get(block)
+                conn, packet, kw = get(block)
             except Empty:
                 break
             if packet is None or isinstance(packet, ForgottenPacket):
@@ -199,7 +200,7 @@ class Application(object):
                 continue
             block = False
             try:
-                _handlePacket(conn, packet)
+                _handlePacket(conn, packet, kw)
             except ConnectionClosed:
                 pass
 
@@ -217,14 +218,14 @@ class Application(object):
             self.setHandlerData(None)
 
     @profiler_decorator
-    def _ask(self, conn, packet, handler=None):
+    def _ask(self, conn, packet, handler=None, **kw):
         self.setHandlerData(None)
         queue = self._getThreadQueue()
-        msg_id = conn.ask(packet, queue=queue)
+        msg_id = conn.ask(packet, queue=queue, **kw)
         get = queue.get
         _handlePacket = self._handlePacket
         while True:
-            qconn, qpacket = get(True)
+            qconn, qpacket, kw = get(True)
             is_forgotten = isinstance(qpacket, ForgottenPacket)
             if conn is qconn:
                 # check fake packet
@@ -234,22 +235,22 @@ class Application(object):
                     if is_forgotten:
                         raise ValueError, 'ForgottenPacket for an ' \
                             'explicitely expected packet.'
-                    _handlePacket(qconn, qpacket, handler=handler)
+                    _handlePacket(qconn, qpacket, kw, handler)
                     break
             if not is_forgotten and qpacket is not None:
-                _handlePacket(qconn, qpacket)
+                _handlePacket(qconn, qpacket, kw)
         return self.getHandlerData()
 
     @profiler_decorator
-    def _askStorage(self, conn, packet):
+    def _askStorage(self, conn, packet, **kw):
         """ Send a request to a storage node and process its answer """
-        return self._ask(conn, packet, handler=self.storage_handler)
+        return self._ask(conn, packet, handler=self.storage_handler, **kw)
 
     @profiler_decorator
-    def _askPrimary(self, packet):
+    def _askPrimary(self, packet, **kw):
         """ Send a request to the primary master and process its answer """
         return self._ask(self._getMasterConnection(), packet,
-            handler=self.primary_handler)
+            handler=self.primary_handler, **kw)
 
     @profiler_decorator
     def _getMasterConnection(self):
@@ -261,7 +262,6 @@ class Application(object):
             try:
                 self.new_oid_list = []
                 result = self._connectToPrimaryNode()
-                self.master_conn = result
             finally:
                 self._connecting_to_master_node_release()
         return result
@@ -278,10 +278,9 @@ class Application(object):
         """
             Lookup for the current primary master node
         """
-        neo.lib.logging.debug('connecting to primary master...')
+        logging.debug('connecting to primary master...')
         ready = False
         nm = self.nm
-        packet = Packets.AskPrimary()
         while not ready:
             # Get network connection to primary master
             index = 0
@@ -304,34 +303,31 @@ class Application(object):
                 # Connect to master
                 conn = MTClientConnection(self.em,
                         self.notifications_handler,
-                        addr=self.trying_master_node.getAddress(),
+                        node=self.trying_master_node,
                         connector=self.connector_handler(),
                         dispatcher=self.dispatcher)
                 # Query for primary master node
                 if conn.getConnector() is None:
                     # This happens if a connection could not be established.
-                    neo.lib.logging.error(
-                                    'Connection to master node %s failed',
+                    logging.error('Connection to master node %s failed',
                                   self.trying_master_node)
                     continue
                 try:
-                    self._ask(conn, packet,
+                    self._ask(conn, Packets.RequestIdentification(
+                            NodeTypes.CLIENT, self.uuid, None, self.name),
                         handler=self.primary_bootstrap_handler)
                 except ConnectionClosed:
                     continue
                 # If we reached the primary master node, mark as connected
                 connected = self.primary_master_node is not None and \
                         self.primary_master_node is self.trying_master_node
-            neo.lib.logging.info(
-                            'Connected to %s' % (self.primary_master_node, ))
+            logging.info('Connected to %s', self.primary_master_node)
             try:
                 ready = self.identifyToPrimaryNode(conn)
             except ConnectionClosed:
-                neo.lib.logging.error('Connection to %s lost',
-                    self.trying_master_node)
+                logging.error('Connection to %s lost', self.trying_master_node)
                 self.primary_master_node = None
-                continue
-        neo.lib.logging.info("Connected and ready")
+        logging.info("Connected and ready")
         return conn
 
     def identifyToPrimaryNode(self, conn):
@@ -340,20 +336,12 @@ class Application(object):
             Might raise ConnectionClosed so that the new primary can be
             looked-up again.
         """
-        neo.lib.logging.info('Initializing from master')
+        logging.info('Initializing from master')
         ask = self._ask
         handler = self.primary_bootstrap_handler
-        # Identify to primary master and request initial data
-        p = Packets.RequestIdentification(NodeTypes.CLIENT, self.uuid, None,
-            self.name)
-        while conn.getUUID() is None:
-            ask(conn, p, handler=handler)
-            if conn.getUUID() is None:
-                # Node identification was refused by master, it is considered
-                # as the primary as long as we are connected to it.
-                time.sleep(1)
         ask(conn, Packets.AskNodeInformation(), handler=handler)
         ask(conn, Packets.AskPartitionTable(), handler=handler)
+        self.lastTransaction()
         return self.pt.operational()
 
     def registerDB(self, db, limit):
@@ -380,8 +368,8 @@ class Application(object):
         finally:
             self._oid_lock_release()
 
-    def getStorageSize(self):
-        # return the last OID used, this is innacurate
+    def getObjectCount(self):
+        # return the last OID used, this is inaccurate
         return int(u64(self.last_oid))
 
     @profiler_decorator
@@ -421,17 +409,38 @@ class Application(object):
         # TODO:
         # - rename parameters (here? and in handlers & packet definitions)
 
+        acquire = self._cache_lock_acquire
+        release = self._cache_lock_release
+        # XXX: Is it possible this giant lock ?
+        #      See commit b77c946d67c9d7cc1e9ee9b15437568dee144aa4
+        #      for a way to invalidate cache properly when several loads
+        #      are done simultaneously.
         self._load_lock_acquire()
         try:
-            result = self._loadFromCache(oid, tid, before_tid)
-            if not result:
-                result = self._loadFromStorage(oid, tid, before_tid)
-                self._cache_lock_acquire()
-                try:
-                    self._cache.store(oid, *result)
-                finally:
-                    self._cache_lock_release()
-            return result
+            acquire()
+            try:
+                result = self._loadFromCache(oid, tid, before_tid)
+                if result:
+                    return result
+                self._loading_oid = oid
+            finally:
+                release()
+            # When not bound to a ZODB Connection, load() may be the
+            # first method called and last_tid may still be None.
+            # This happens, for example, when opening the DB.
+            if not (tid or before_tid) and self.last_tid:
+                # Do not get something more recent than the last invalidation
+                # we got from master.
+                before_tid = p64(u64(self.last_tid) + 1)
+            result = self._loadFromStorage(oid, tid, before_tid)
+            acquire()
+            try:
+                if not (self._loading_oid or result[2]):
+                    result = result[0], result[1], self._loading_invalidated
+                self._cache.store(oid, *result)
+                return result
+            finally:
+                release()
         finally:
             self._load_lock_release()
 
@@ -447,7 +456,7 @@ class Application(object):
 
             if data or checksum != ZERO_HASH:
                 if checksum != makeChecksum(data):
-                    neo.lib.logging.error('wrong checksum from %s for oid %s',
+                    logging.error('wrong checksum from %s for oid %s',
                               conn, dump(oid))
                     continue
                 if compression:
@@ -463,15 +472,11 @@ class Application(object):
         """
         Load from local cache, return None if not found.
         """
-        self._cache_lock_acquire()
-        try:
-            if at_tid:
-                result = self._cache.load(oid, at_tid + '*')
-                assert not result or result[1] == at_tid
-                return result
-            return self._cache.load(oid, before_tid)
-        finally:
-            self._cache_lock_release()
+        if at_tid:
+            result = self._cache.load(oid, at_tid + '*')
+            assert not result or result[1] == at_tid
+            return result
+        return self._cache.load(oid, before_tid)
 
     @profiler_decorator
     def tpc_begin(self, transaction, tid=None, status=' '):
@@ -496,8 +501,7 @@ class Application(object):
         txn_context = self._txn_container.get(transaction)
         if txn_context is None:
             raise StorageTransactionError(self, transaction)
-        neo.lib.logging.debug(
-                        'storing oid %s serial %s', dump(oid), dump(serial))
+        logging.debug('storing oid %s serial %s', dump(oid), dump(serial))
         self._store(txn_context, oid, serial, data)
         return None
 
@@ -539,7 +543,7 @@ class Application(object):
         add_involved_nodes = involved_nodes.add
         packet = Packets.AskStoreObject(oid, serial, compression,
             checksum, compressed_data, data_serial, ttid, unlock)
-        for node, conn in self.cp.iterateForObject(oid, writable=True):
+        for node, conn in self.cp.iterateForObject(oid):
             try:
                 conn.ask(packet, on_timeout=on_timeout, queue=queue)
                 add_involved_nodes(node)
@@ -580,7 +584,7 @@ class Application(object):
             if ZERO_TID in conflict_serial_set:
               if 1:
                 # XXX: disable deadlock avoidance code until it is fixed
-                neo.lib.logging.info('Deadlock avoidance on %r:%r',
+                logging.info('Deadlock avoidance on %r:%r',
                     dump(oid), dump(serial))
                 # 'data' parameter of ConflictError is only used to report the
                 # class of the object. It doesn't matter if 'data' is None
@@ -601,7 +605,7 @@ class Application(object):
                 # XXX: currently, brute-force is implemented: we send
                 # object data again.
                 # WARNING: not maintained code
-                neo.lib.logging.info('Deadlock avoidance triggered on %r:%r',
+                logging.info('Deadlock avoidance triggered on %r:%r',
                     dump(oid), dump(serial))
                 for store_oid, store_data in data_dict.iteritems():
                     store_serial = object_serial_dict[store_oid]
@@ -611,9 +615,8 @@ class Application(object):
                     else:
                         if store_data is None:
                             # Some undo
-                            neo.lib.logging.warning('Deadlock avoidance cannot'
-                                ' reliably work with undo, this must be '
-                                'implemented.')
+                            logging.warning('Deadlock avoidance cannot reliably'
+                                ' work with undo, this must be implemented.')
                             conflict_serial = ZERO_TID
                             break
                         self._store(txn_context, store_oid, store_serial,
@@ -637,7 +640,7 @@ class Application(object):
                 new_data = tryToResolveConflict(oid, conflict_serial,
                     serial, data)
                 if new_data is not None:
-                    neo.lib.logging.info('Conflict resolution succeed for ' \
+                    logging.info('Conflict resolution succeed for '
                         '%r:%r with %r', dump(oid), dump(serial),
                         dump(conflict_serial))
                     # Mark this conflict as resolved
@@ -649,7 +652,7 @@ class Application(object):
                     append(oid)
                     continue
                 else:
-                    neo.lib.logging.info('Conflict resolution failed for ' \
+                    logging.info('Conflict resolution failed for '
                         '%r:%r with %r', dump(oid), dump(serial),
                         dump(conflict_serial))
             raise ConflictError(oid=oid, serials=(txn_context['ttid'],
@@ -657,12 +660,11 @@ class Application(object):
         return result
 
     @profiler_decorator
-    def waitResponses(self, queue, handler_data):
+    def waitResponses(self, queue):
         """Wait for all requests to be answered (or their connection to be
         detected as closed)"""
         pending = self.dispatcher.pending
         _waitAnyMessage = self._waitAnyMessage
-        self.setHandlerData(handler_data)
         while pending(queue):
             _waitAnyMessage(queue)
 
@@ -672,7 +674,6 @@ class Application(object):
         append = result.append
         resolved_oid_set = set()
         update = resolved_oid_set.update
-        ttid = txn_context['ttid']
         _handleConflicts = self._handleConflicts
         queue = txn_context['queue']
         conflict_serial_dict = txn_context['conflict_serial_dict']
@@ -692,7 +693,7 @@ class Application(object):
         for oid, store_dict in \
                 txn_context['object_stored_counter_dict'].iteritems():
             if not store_dict:
-                neo.lib.logging.error('tpc_store failed')
+                logging.error('tpc_store failed')
                 raise NEOStorageError('tpc_store failed')
             elif oid in resolved_oid_set:
                 append((oid, ResolvedSerial))
@@ -715,8 +716,8 @@ class Application(object):
             str(transaction.description), dumps(transaction._extension),
             txn_context['cache_dict'])
         add_involved_nodes = txn_context['involved_nodes'].add
-        for node, conn in self.cp.iterateForObject(ttid, writable=True):
-            neo.lib.logging.debug("voting object %s on %s", dump(ttid),
+        for node, conn in self.cp.iterateForObject(ttid):
+            logging.debug("voting object %s on %s", dump(ttid),
                 dump(conn.getUUID()))
             try:
                 self._askStorage(conn, packet)
@@ -727,7 +728,7 @@ class Application(object):
 
         # check at least one storage node accepted
         if txn_stored_counter == 0:
-            neo.lib.logging.error('tpc_vote failed')
+            logging.error('tpc_vote failed')
             raise NEOStorageError('tpc_vote failed')
         # Check if master connection is still alive.
         # This is just here to lower the probability of detecting a problem
@@ -757,10 +758,8 @@ class Application(object):
             try:
                 conn.notify(p)
             except:
-                neo.lib.logging.error(
-                    'Exception in tpc_abort while notifying' \
-                    'storage node %r of abortion, ignoring.',
-                    conn, exc_info=1)
+                logging.exception('Exception in tpc_abort while notifying'
+                    'storage node %r of abortion, ignoring.', conn)
         self._getMasterConnection().notify(p)
         queue = txn_context['queue']
         # We don't need to flush queue, as it won't be reused by future
@@ -784,35 +783,14 @@ class Application(object):
             # Call finish on master
             cache_dict = txn_context['cache_dict']
             tid = self._askPrimary(Packets.AskFinishTransaction(
-                txn_context['ttid'], cache_dict))
-
-            # Call function given by ZODB
-            if f is not None:
-                f(tid)
-
-            # Update cache
-            self._cache_lock_acquire()
-            try:
-                cache = self._cache
-                for oid, data in cache_dict.iteritems():
-                    if data is CHECKED_SERIAL:
-                        # this is just a remain of
-                        # checkCurrentSerialInTransaction call, ignore (no data
-                        # was modified).
-                        continue
-                    # Update ex-latest value in cache
-                    cache.invalidate(oid, tid)
-                    if data is not None:
-                        # Store in cache with no next_tid
-                        cache.store(oid, data, tid, None)
-            finally:
-                self._cache_lock_release()
+                txn_context['ttid'], cache_dict),
+                cache_dict=cache_dict, callback=f)
             txn_container.delete(transaction)
             return tid
         finally:
             self._load_lock_release()
 
-    def undo(self, snapshot_tid, undone_tid, txn, tryToResolveConflict):
+    def undo(self, undone_tid, txn, tryToResolveConflict):
         txn_context = self._txn_container.get(txn)
         if txn_context is None:
             raise StorageTransactionError(self, undone_tid)
@@ -838,25 +816,25 @@ class Application(object):
         getConnForCell = self.cp.getConnForCell
         queue = self._getThreadQueue()
         ttid = txn_context['ttid']
+        undo_object_tid_dict = {}
+        snapshot_tid = p64(u64(self.last_tid) + 1)
         for partition, oid_list in partition_oid_dict.iteritems():
             cell_list = getCellList(partition, readable=True)
             # We do want to shuffle before getting one with the smallest
             # key, so that all cells with the same (smallest) key has
             # identical chance to be chosen.
             shuffle(cell_list)
-            # BBB: min(..., key=...) requires Python >= 2.5
-            cell_list.sort(key=getCellSortKey)
-            storage_conn = getConnForCell(cell_list[0])
+            storage_conn = getConnForCell(min(cell_list, key=getCellSortKey))
             storage_conn.ask(Packets.AskObjectUndoSerial(ttid,
-                snapshot_tid, undone_tid, oid_list), queue=queue)
+                snapshot_tid, undone_tid, oid_list),
+                queue=queue, undo_object_tid_dict=undo_object_tid_dict)
 
         # Wait for all AnswerObjectUndoSerial. We might get OidNotFoundError,
         # meaning that objects in transaction's oid_list do not exist any
         # longer. This is the symptom of a pack, so forbid undoing transaction
         # when it happens.
-        undo_object_tid_dict = {}
         try:
-            self.waitResponses(queue, undo_object_tid_dict)
+            self.waitResponses(queue)
         except NEOStorageNotFoundError:
             self.dispatcher.forget_queue(queue)
             raise UndoError('non-undoable transaction')
@@ -910,10 +888,6 @@ class Application(object):
             raise NEOStorageError('Transaction %r not found' % (tid, ))
         return (txn_info, txn_ext)
 
-    # XXX: The following 2 methods fail when they reconnect to a storage after
-    #      they already sent a request to a previous storage.
-    #      See also testStorageReconnectDuringXxx
-
     def undoLog(self, first, last, filter=None, block=0):
         # XXX: undoLog is broken
         if last < 0:
@@ -924,24 +898,21 @@ class Application(object):
         # Each storage node will return TIDs only for UP_TO_DATE state and
         # FEEDING state cells
         pt = self.getPartitionTable()
-        storage_node_list = pt.getNodeList()
-
         queue = self._getThreadQueue()
         packet = Packets.AskTIDs(first, last, INVALID_PARTITION)
-        for storage_node in storage_node_list:
+        tid_set = set()
+        for storage_node in pt.getNodeSet(True):
             conn = self.cp.getConnForNode(storage_node)
             if conn is None:
                 continue
-            conn.ask(packet, queue=queue)
+            conn.ask(packet, queue=queue, tid_set=tid_set)
 
         # Wait for answers from all storages.
-        tid_set = set()
-        self.waitResponses(queue, tid_set)
+        self.waitResponses(queue)
 
         # Reorder tids
         ordered_tids = sorted(tid_set, reverse=True)
-        neo.lib.logging.debug(
-                        "UndoLog tids %s", [dump(x) for x in ordered_tids])
+        logging.debug("UndoLog tids %s", map(dump, ordered_tids))
         # For each transaction, get info
         undo_info = []
         append = undo_info.append
@@ -962,30 +933,28 @@ class Application(object):
         return undo_info
 
     def transactionLog(self, start, stop, limit):
-        node_map = self.pt.getNodeMap()
-        node_list = node_map.keys()
-        node_list.sort(key=self.cp.getCellSortKey)
-        partition_set = set(range(self.pt.getPartitions()))
-        queue = self._getThreadQueue()
+        tid_list = []
         # request a tid list for each partition
-        for node in node_list:
-            conn = self.cp.getConnForNode(node)
-            request_set = set(node_map[node]) & partition_set
-            if conn is None or not request_set:
-                continue
-            partition_set -= set(request_set)
-            packet = Packets.AskTIDsFrom(start, stop, limit, request_set)
-            conn.ask(packet, queue=queue)
-            if not partition_set:
-                break
-        assert not partition_set
-        tid_set = set()
-        self.waitResponses(queue, tid_set)
+        for offset in xrange(self.pt.getPartitions()):
+            p = Packets.AskTIDsFrom(start, stop, limit, offset)
+            for node, conn in self.cp.iterateForObject(offset, readable=True):
+                try:
+                    r = self._askStorage(conn, p)
+                    break
+                except ConnectionClosed:
+                    pass
+            else:
+                raise NEOStorageError('transactionLog failed')
+            if r:
+                tid_list = list(heapq.merge(tid_list, r))
+                if len(tid_list) >= limit:
+                    del tid_list[limit:]
+                    stop = tid_list[-1]
         # request transactions informations
         txn_list = []
         append = txn_list.append
         tid = None
-        for tid in sorted(tid_set):
+        for tid in tid_list:
             (txn_info, txn_ext) = self._getTransactionInformation(tid)
             txn_info['ext'] = loads(txn_ext)
             append(txn_info)
@@ -1017,18 +986,20 @@ class Application(object):
             return result
 
     @profiler_decorator
-    def importFrom(self, source, start, stop, tryToResolveConflict):
-        serials = {}
+    def importFrom(self, source, start, stop, tryToResolveConflict,
+            preindex=None):
+        if preindex is None:
+            preindex = {}
         transaction_iter = source.iterator(start, stop)
         for transaction in transaction_iter:
             tid = transaction.tid
             self.tpc_begin(transaction, tid, transaction.status)
             for r in transaction:
                 oid = r.oid
-                pre = serials.get(oid, None)
+                pre = preindex.get(oid)
                 # TODO: bypass conflict resolution, locks...
                 self.store(oid, pre, r.data, r.version, transaction)
-                serials[oid] = tid
+                preindex[oid] = tid
             conflicted = self.tpc_vote(transaction, tryToResolveConflict)
             assert not conflicted, conflicted
             real_tid = self.tpc_finish(transaction, tryToResolveConflict)
@@ -1041,7 +1012,8 @@ class Application(object):
         return Iterator(self, start, stop)
 
     def lastTransaction(self):
-        return self._askPrimary(Packets.AskLastTransaction())
+        self._askPrimary(Packets.AskLastTransaction())
+        return self.last_tid
 
     def abortVersion(self, src, transaction):
         if self._txn_container.get(transaction) is None:
@@ -1062,7 +1034,7 @@ class Application(object):
         self.cp.flush()
         self.master_conn = None
         # Stop polling thread
-        neo.lib.logging.debug('Stopping %s', self.poll_thread)
+        logging.debug('Stopping %s', self.poll_thread)
         self.poll_thread.stop()
         psThreadedPoll()
     close = __del__
@@ -1102,7 +1074,7 @@ class Application(object):
         assert oid not in txn_context['cache_dict'], (oid, txn_context)
         txn_context['data_dict'].setdefault(oid, CHECKED_SERIAL)
         packet = Packets.AskCheckCurrentSerial(ttid, serial, oid)
-        for node, conn in self.cp.iterateForObject(oid, writable=True):
+        for node, conn in self.cp.iterateForObject(oid):
             try:
                 conn.ask(packet, queue=queue)
             except ConnectionClosed:
