@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2010  Nexedi SA
+# Copyright (C) 2006-2011  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,15 +12,14 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from ZODB.config import BaseConfig
 
 class NeoStorage(BaseConfig):
 
     def open(self):
-        from neo.client.Storage import Storage
+        from .Storage import Storage
         config = self.config
         return Storage(**dict((k, getattr(config, k))
                               for k in config.getSectionAttributes()))
