@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2010  Nexedi SA
+# Copyright (C) 2006-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,19 +12,18 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from ZODB.TimeStamp import TimeStamp
 from ZODB.POSException import ConflictError
 
-import neo.lib
-from neo.client.handlers import BaseHandler, AnswerBaseHandler
-from neo.lib.protocol import NodeTypes, ProtocolError, LockState, ZERO_TID
+from neo.lib import logging
+from neo.lib.protocol import LockState, ZERO_TID
 from neo.lib.util import dump
-from neo.client.exception import NEOStorageError, NEOStorageNotFoundError
-from neo.client.exception import NEOStorageDoesNotExistError
 from neo.lib.exception import NodeNotReady
+from . import BaseHandler, AnswerBaseHandler
+from ..exception import NEOStorageError, NEOStorageNotFoundError
+from ..exception import NEOStorageDoesNotExistError
 
 class StorageEventHandler(BaseHandler):
 
@@ -48,18 +47,12 @@ class StorageBootstrapHandler(AnswerBaseHandler):
     def notReady(self, conn, message):
         raise NodeNotReady(message)
 
-    def acceptIdentification(self, conn, node_type,
-           uuid, num_partitions, num_replicas, your_uuid):
-        # this must be a storage node
-        if node_type != NodeTypes.STORAGE:
-            conn.close()
-            return
-
-        node = self.app.nm.getByAddress(conn.getAddress())
-        assert node is not None, conn.getAddress()
-        conn.setUUID(uuid)
+    def _acceptIdentification(self, node,
+           uuid, num_partitions, num_replicas, your_uuid, primary,
+           master_list):
+        assert primary == self.app.primary_master_node.getAddress(), (
+            primary, self.app.primary_master_node)
         node.setUUID(uuid)
-        node.setConnection(conn)
 
 class StorageAnswersHandler(AnswerBaseHandler):
     """ Handle all messages related to ZODB operations """
@@ -78,8 +71,8 @@ class StorageAnswersHandler(AnswerBaseHandler):
             # we may process entirely a conflict with S1 (i.e. we received the
             # answer to the store of the resolved object on S1) before we
             # receive the conflict answer from the first store on S2.
-            neo.lib.logging.info('%r report a conflict for %r with %r', conn,
-                        dump(oid), dump(serial))
+            logging.info('%r report a conflict for %r with %r',
+                         conn, dump(oid), dump(serial))
             # If this conflict is not already resolved, mark it for
             # resolution.
             if serial not in txn_context[
@@ -122,10 +115,8 @@ class StorageAnswersHandler(AnswerBaseHandler):
         pass
 
     def answerTIDsFrom(self, conn, tid_list):
-        neo.lib.logging.debug('Get %d TIDs from %r', len(tid_list), conn)
-        tids_from = self.app.getHandlerData()
-        assert not tids_from.intersection(set(tid_list))
-        tids_from.update(tid_list)
+        logging.debug('Get %u TIDs from %r', len(tid_list), conn)
+        self.app.setHandlerData(tid_list)
 
     def answerTransactionInformation(self, conn, tid,
                                            user, desc, ext, packed, oid_list):
@@ -155,11 +146,12 @@ class StorageAnswersHandler(AnswerBaseHandler):
         # This can happen when requiring txn informations
         raise NEOStorageNotFoundError(message)
 
-    def answerTIDs(self, conn, tid_list):
-        self.app.getHandlerData().update(tid_list)
+    def answerTIDs(self, conn, tid_list, tid_set):
+        tid_set.update(tid_list)
 
-    def answerObjectUndoSerial(self, conn, object_tid_dict):
-        self.app.getHandlerData().update(object_tid_dict)
+    def answerObjectUndoSerial(self, conn, object_tid_dict,
+                               undo_object_tid_dict):
+        undo_object_tid_dict.update(object_tid_dict)
 
     def answerHasLock(self, conn, oid, status):
         store_msg_id = self.app.getHandlerData()['timeout_dict'].pop(oid)
@@ -186,8 +178,7 @@ class StorageAnswersHandler(AnswerBaseHandler):
         #      Anyway, it's not clear that HasLock requests are useful.
         #      Are store requests potentially long to process ? If not,
         #      we should simply raise a ConflictError on store timeout.
-        neo.lib.logging.info('Store of oid %s delayed (storage overload ?)',
-                             dump(oid))
+        logging.info('Store of oid %s delayed (storage overload ?)', dump(oid))
 
     def alreadyPendingError(self, conn, message):
         pass
