@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2010  Nexedi SA
+# Copyright (C) 2006-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,25 +12,20 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from functools import wraps
 from time import time
 
-import neo.lib
-from neo.lib.locking import RLock
-
-from neo.lib.protocol import PacketMalformedError, Packets, ParserState
-from neo.lib.connector import ConnectorException, ConnectorTryAgainException, \
+from . import attributeTracker, logging
+from .connector import ConnectorException, ConnectorTryAgainException, \
         ConnectorInProgressException, ConnectorConnectionRefusedException, \
         ConnectorConnectionClosedException
-from neo.lib.util import dump
-from neo.lib.logger import PACKET_LOGGER
-
-from neo.lib import attributeTracker
-from neo.lib.util import ReadBuffer
-from neo.lib.profiling import profiler_decorator
+from .locking import RLock
+from .profiling import profiler_decorator
+from .protocol import uuid_str, Errors, \
+        PacketMalformedError, Packets, ParserState
+from .util import ReadBuffer
 
 CRITICAL_TIMEOUT = 30
 
@@ -60,8 +55,8 @@ def lockCheckWrapper(func):
     def wrapper(self, *args, **kw):
         if not self._lock._is_owned():
             import traceback
-            neo.lib.logging.warning('%s called on %s instance without being ' \
-                'locked. Stack:\n%s', func.func_code.co_name,
+            logging.warning('%s called on %s instance without being locked.'
+                ' Stack:\n%s', func.func_code.co_name,
                 self.__class__.__name__, ''.join(traceback.format_stack()))
         # Call anyway
         return func(self, *args, **kw)
@@ -91,12 +86,30 @@ class HandlerSwitcher(object):
         self._is_handling = False
 
     def clear(self):
-        handler = self._pending[0][1]
-        self._pending = [[{}, handler]]
+        self.__init__(self.getLastHandler())
+        try:
+            del (self._next_timeout,
+                 self._next_timeout_msg_id,
+                 self._next_on_timeout)
+        except AttributeError:
+            pass
 
     def isPending(self):
         return bool(self._pending[0][0])
 
+    def cancelRequests(self, conn, message):
+        if self.isPending():
+            p = Errors.ProtocolError(message)
+            while True:
+                request_dict, handler = self._pending[0]
+                while request_dict:
+                    msg_id, request = request_dict.popitem()
+                    p.setId(msg_id)
+                    handler.packetReceived(conn, p, request[3])
+                if len(self._pending) == 1:
+                    break
+                del self._pending[0]
+
     def getHandler(self):
         return self._pending[0][1]
 
@@ -105,7 +118,7 @@ class HandlerSwitcher(object):
         return self._pending[-1][1]
 
     @profiler_decorator
-    def emit(self, request, timeout, on_timeout):
+    def emit(self, request, timeout, on_timeout, kw={}):
         # register the request in the current handler
         _pending = self._pending
         if self._is_handling:
@@ -113,7 +126,7 @@ class HandlerSwitcher(object):
             # be excpected for the current handler...
             (request_dict, _) = _pending[0]
         else:
-            # ...otherwise, queue for for the latest handler
+            # ...otherwise, queue for the latest handler
             assert len(_pending) == 1 or _pending[0][0]
             (request_dict, _) = _pending[-1]
         msg_id = request.getId()
@@ -125,7 +138,7 @@ class HandlerSwitcher(object):
             self._next_timeout = timeout
             self._next_timeout_msg_id = msg_id
             self._next_on_timeout = on_timeout
-        request_dict[msg_id] = (answer_class, timeout, on_timeout)
+        request_dict[msg_id] = answer_class, timeout, on_timeout, kw
 
     def getNextTimeout(self):
         return self._next_timeout
@@ -152,9 +165,9 @@ class HandlerSwitcher(object):
     @profiler_decorator
     def _handle(self, connection, packet):
         assert len(self._pending) == 1 or self._pending[0][0]
-        PACKET_LOGGER.dispatch(connection, packet, False)
+        logging.packet(connection, packet, False)
         if connection.isClosed() and packet.ignoreOnClosedConnection():
-            neo.lib.logging.debug('Ignoring packet %r on closed connection %r',
+            logging.debug('Ignoring packet %r on closed connection %r',
                 packet, connection)
             return
         msg_id = packet.getId()
@@ -164,12 +177,15 @@ class HandlerSwitcher(object):
             handler.packetReceived(connection, packet)
             return
         # checkout the expected answer class
-        (klass, timeout, _) = request_dict.pop(msg_id, (None, None, None))
+        try:
+            klass, _, _, kw = request_dict.pop(msg_id)
+        except KeyError:
+            klass = None
+            kw = {}
         if klass and isinstance(packet, klass) or packet.isError():
-            handler.packetReceived(connection, packet)
+            handler.packetReceived(connection, packet, kw)
         else:
-            neo.lib.logging.error(
-                            'Unexpected answer %r in %r', packet, connection)
+            logging.error('Unexpected answer %r in %r', packet, connection)
             if not connection.isClosed():
                 notification = Packets.Notify('Unexpected answer: %r' % packet)
                 connection.notify(notification)
@@ -178,8 +194,7 @@ class HandlerSwitcher(object):
         # apply a pending handler if no more answers are pending
         while len(self._pending) > 1 and not self._pending[0][0]:
             del self._pending[0]
-            neo.lib.logging.debug(
-                            'Apply handler %r on %r', self._pending[0][1],
+            logging.debug('Apply handler %r on %r', self._pending[0][1],
                     connection)
         if msg_id == self._next_timeout_msg_id:
             self._updateNextTimeout()
@@ -188,7 +203,7 @@ class HandlerSwitcher(object):
         # Find next timeout and its msg_id
         next_timeout = None
         for pending in self._pending:
-            for msg_id, (_, timeout, on_timeout) in pending[0].iteritems():
+            for msg_id, (_, timeout, on_timeout, _) in pending[0].iteritems():
                 if not next_timeout or timeout < next_timeout[0]:
                     next_timeout = timeout, msg_id, on_timeout
         self._next_timeout, self._next_timeout_msg_id, self._next_on_timeout = \
@@ -228,7 +243,6 @@ class BaseConnection(object):
     """
 
     KEEP_ALIVE = 60
-    _base_timeout = None
 
     def __init__(self, event_manager, handler, connector, addr=None):
         assert connector is not None, "Need a low-level connector"
@@ -237,32 +251,18 @@ class BaseConnection(object):
         self.addr = addr
         self._handlers = HandlerSwitcher(handler)
         event_manager.register(self)
+        event_manager.addReader(self)
 
-    def isPending(self):
-        return self._handlers.isPending()
+    # XXX: do not use getHandler
+    getHandler      = property(lambda self: self._handlers.getHandler)
+    getLastHandler  = property(lambda self: self._handlers.getLastHandler)
+    isPending       = property(lambda self: self._handlers.isPending)
 
-    def updateTimeout(self, t=None):
-        if not self._queue:
-            if t:
-                self._base_timeout = t
-            self._timeout = self._handlers.getNextTimeout() or self.KEEP_ALIVE
+    def cancelRequests(self, *args, **kw):
+        return self._handlers.cancelRequests(self, *args, **kw)
 
     def checkTimeout(self, t):
-        # first make sure we don't timeout on answers we already received
-        if self._base_timeout and not self._queue:
-            timeout = t - self._base_timeout
-            if self._timeout <= timeout:
-                handlers = self._handlers
-                if handlers.isPending():
-                    msg_id = handlers.timeout(self)
-                    if msg_id is None:
-                        self._base_timeout = t
-                    else:
-                        neo.lib.logging.info('timeout for #0x%08x with %r',
-                                             msg_id, self)
-                        self.close()
-                else:
-                    self.idle()
+        pass
 
     def lock(self):
         return 1
@@ -292,12 +292,13 @@ class BaseConnection(object):
             self.connector.shutdown()
             self.connector.close()
             self.connector = None
+            self.aborted = False
 
     def __repr__(self):
         address = self.addr and '%s:%d' % self.addr or '?'
         return '<%s(uuid=%s, address=%s, closed=%s, handler=%s) at %x>' % (
             self.__class__.__name__,
-            dump(self.getUUID()),
+            uuid_str(self.getUUID()),
             address,
             int(self.isClosed()),
             self.getHandler(),
@@ -306,14 +307,11 @@ class BaseConnection(object):
 
     __del__ = close
 
-    def getHandler(self):
-        return self._handlers.getHandler()
-
     def setHandler(self, handler):
         if self._handlers.setHandler(handler):
-            neo.lib.logging.debug('Set handler %r on %r', handler, self)
+            logging.debug('Set handler %r on %r', handler, self)
         else:
-            neo.lib.logging.debug('Delay handler %r on %r', handler, self)
+            logging.debug('Delay handler %r on %r', handler, self)
 
     def getEventManager(self):
         return self.em
@@ -322,7 +320,7 @@ class BaseConnection(object):
         return None
 
     def isClosed(self):
-        return self.connector is None
+        return self.connector is None or self.isAborted()
 
     def isAborted(self):
         return False
@@ -356,16 +354,15 @@ class ListeningConnection(BaseConnection):
     """A listen connection."""
 
     def __init__(self, event_manager, handler, addr, connector, **kw):
-        neo.lib.logging.debug('listening to %s:%d', *addr)
+        logging.debug('listening to %s:%d', *addr)
         BaseConnection.__init__(self, event_manager, handler,
                                 addr=addr, connector=connector)
         self.connector.makeListeningConnection(addr)
-        self.em.addReader(self)
 
     def readable(self):
         try:
             new_s, addr = self.connector.getNewConnection()
-            neo.lib.logging.debug('accepted a connection from %s:%d', *addr)
+            logging.debug('accepted a connection from %s:%d', *addr)
             handler = self.getHandler()
             new_conn = ServerConnection(self.getEventManager(), handler,
                 connector=new_s, addr=addr)
@@ -386,25 +383,56 @@ class ListeningConnection(BaseConnection):
 class Connection(BaseConnection):
     """A connection."""
 
+    # XXX: rename isPending, hasPendingMessages & pending methods
+
     connecting = False
+    client = False
+    server = False
+    peer_id = None
+    _base_timeout = None
 
     def __init__(self, event_manager, *args, **kw):
         BaseConnection.__init__(self, event_manager, *args, **kw)
         self.read_buf = ReadBuffer()
         self.write_buf = []
         self.cur_id = 0
-        self.peer_id = 0
         self.aborted = False
         self.uuid = None
         self._queue = []
         self._on_close = None
         self._parser_state = ParserState()
-        event_manager.addReader(self)
 
     def setOnClose(self, callback):
-        assert self._on_close is None
         self._on_close = callback
 
+    def isClient(self):
+        return self.client
+
+    def isServer(self):
+        return self.server
+
+    def asClient(self):
+        try:
+            del self.idle
+            assert self.client
+        except AttributeError:
+            self.client = True
+
+    def asServer(self):
+        self.server = True
+
+    def _closeClient(self):
+        if self.server:
+            del self.idle
+            self.client = False
+            self.notify(Packets.CloseClient())
+        else:
+            self.close()
+
+    def closeClient(self):
+        if self.connector is not None and self.client:
+            self.idle = self._closeClient
+
     def isAborted(self):
         return self.aborted
 
@@ -415,6 +443,7 @@ class Connection(BaseConnection):
         self.uuid = uuid
 
     def setPeerId(self, peer_id):
+        assert peer_id is not None
         self.peer_id = peer_id
 
     def getPeerId(self):
@@ -426,11 +455,37 @@ class Connection(BaseConnection):
         self.cur_id = (next_id + 1) & 0xffffffff
         return next_id
 
+    def updateTimeout(self, t=None):
+        if not self._queue:
+            if t:
+                self._base_timeout = t
+            self._timeout = self._handlers.getNextTimeout() or self.KEEP_ALIVE
+
+    def checkTimeout(self, t):
+        # first make sure we don't timeout on answers we already received
+        if self._base_timeout and not self._queue:
+            timeout = t - self._base_timeout
+            if self._timeout <= timeout:
+                handlers = self._handlers
+                if handlers.isPending():
+                    msg_id = handlers.timeout(self)
+                    if msg_id is None:
+                        self._base_timeout = t
+                    else:
+                        logging.info('timeout for #0x%08x with %r',
+                                     msg_id, self)
+                        self.close()
+                else:
+                    self.idle()
+
     def abort(self):
         """Abort dealing with this connection."""
-        neo.lib.logging.debug('aborting a connector for %r', self)
+        logging.debug('aborting a connector for %r', self)
         self.aborted = True
         assert self.write_buf
+        if self._on_close is not None:
+            self._on_close()
+            self._on_close = None
 
     def writable(self):
         """Called when self is writable."""
@@ -450,16 +505,15 @@ class Connection(BaseConnection):
 
     def analyse(self):
         """Analyse received data."""
-        while True:
-            # parse a packet
-            try:
+        try:
+            while True:
                 packet = Packets.parse(self.read_buf, self._parser_state)
                 if packet is None:
                     break
-            except PacketMalformedError, msg:
-                self.getHandler()._packetMalformed(self, msg)
-                return
-            self._queue.append(packet)
+                self._queue.append(packet)
+        except PacketMalformedError, e:
+            logging.error('malformed packet from %r: %s', self, e)
+            self._closure()
 
     def hasPendingMessages(self):
         """
@@ -489,7 +543,6 @@ class Connection(BaseConnection):
         # process the network events with the last registered handler to
         # solve issues where a node is lost with pending handlers and
         # create unexpected side effects.
-        neo.lib.logging.debug('closing a connector for %r', self)
         handler = self._handlers.getLastHandler()
         super(Connection, self).close()
         if self._on_close is not None:
@@ -497,14 +550,19 @@ class Connection(BaseConnection):
             self._on_close = None
         del self.write_buf[:]
         self.read_buf.clear()
-        self._handlers.clear()
-        if self.connecting:
-            handler.connectionFailed(self)
-        else:
-            handler.connectionClosed(self)
+        try:
+            if self.connecting:
+                handler.connectionFailed(self)
+                self.connecting = False
+            else:
+                handler.connectionClosed(self)
+        finally:
+            self._handlers.clear()
 
     def _closure(self):
         assert self.connector is not None, self.whoSetConnector()
+        while self._queue:
+            self._handlers.handle(self, self._queue.pop(0))
         self.close()
 
     @profiler_decorator
@@ -520,19 +578,16 @@ class Connection(BaseConnection):
         except ConnectorConnectionClosedException:
             # connection resetted by peer, according to the man, this error
             # should not occurs but it seems it's false
-            neo.lib.logging.debug(
-                            'Connection reset by peer: %r', self.connector)
+            logging.debug('Connection reset by peer: %r', self.connector)
             self._closure()
         except:
-            neo.lib.logging.debug(
-                            'Unknown connection error: %r', self.connector)
+            logging.debug('Unknown connection error: %r', self.connector)
             self._closure()
             # unhandled connector exception
             raise
         else:
             if not data:
-                neo.lib.logging.debug(
-                    'Connection %r closed in recv', self.connector)
+                logging.debug('Connection %r closed in recv', self.connector)
                 self._closure()
                 return
             self._base_timeout = time() # last known remote activity
@@ -550,19 +605,16 @@ class Connection(BaseConnection):
             pass
         except ConnectorConnectionClosedException:
             # connection resetted by peer
-            neo.lib.logging.debug(
-                            'Connection reset by peer: %r', self.connector)
+            logging.debug('Connection reset by peer: %r', self.connector)
             self._closure()
         except:
-            neo.lib.logging.debug(
-                            'Unknown connection error: %r', self.connector)
+            logging.debug('Unknown connection error: %r', self.connector)
             # unhandled connector exception
             self._closure()
             raise
         else:
             if not n:
-                neo.lib.logging.debug('Connection %r closed in send',
-                    self.connector)
+                logging.debug('Connection %r closed in send', self.connector)
                 self._closure()
                 return
             if n == len(msg):
@@ -583,7 +635,7 @@ class Connection(BaseConnection):
         if was_empty:
             # enable polling for writing.
             self.em.addWriter(self)
-        PACKET_LOGGER.dispatch(self, packet, True)
+        logging.packet(self, packet, True)
 
     @not_closed
     def notify(self, packet):
@@ -595,7 +647,7 @@ class Connection(BaseConnection):
 
     @profiler_decorator
     @not_closed
-    def ask(self, packet, timeout=CRITICAL_TIMEOUT, on_timeout=None):
+    def ask(self, packet, timeout=CRITICAL_TIMEOUT, on_timeout=None, **kw):
         """
         Send a packet with a new ID and register the expectation of an answer
         """
@@ -604,7 +656,7 @@ class Connection(BaseConnection):
         self._addPacket(packet)
         handlers = self._handlers
         t = not handlers.isPending() and time() or None
-        handlers.emit(packet, timeout, on_timeout)
+        handlers.emit(packet, timeout, on_timeout, kw)
         self.updateTimeout(t)
         return msg_id
 
@@ -625,9 +677,12 @@ class ClientConnection(Connection):
     """A connection from this node to a remote node."""
 
     connecting = True
+    client = True
 
-    def __init__(self, event_manager, handler, addr, connector):
+    def __init__(self, event_manager, handler, node, connector):
+        addr = node.getAddress()
         Connection.__init__(self, event_manager, handler, connector, addr)
+        node.setConnection(self)
         handler.connectionStarted(self)
         try:
             try:
@@ -635,9 +690,7 @@ class ClientConnection(Connection):
             except ConnectorInProgressException:
                 event_manager.addWriter(self)
             else:
-                self.connecting = False
-                self.updateTimeout(time())
-                self.getHandler().connectionCompleted(self)
+                self._connectionCompleted()
         except ConnectorConnectionRefusedException:
             self._closure()
         except ConnectorException:
@@ -647,22 +700,17 @@ class ClientConnection(Connection):
 
     def writable(self):
         """Called when self is writable."""
-        if self.connecting:
-            err = self.connector.getError()
-            if err:
-                self._closure()
-                return
-            else:
-                self.connecting = False
-                self.updateTimeout(time())
-                self.getHandler().connectionCompleted(self)
-                self.em.addReader(self)
+        if self.connector.getError():
+            self._closure()
         else:
-            Connection.writable(self)
-
-    def isClient(self):
-        return True
+            self._connectionCompleted()
+            self.writable()
 
+    def _connectionCompleted(self):
+        self.writable = super(ClientConnection, self).writable
+        self.connecting = False
+        self.updateTimeout(time())
+        self.getHandler().connectionCompleted(self)
 
 class ServerConnection(Connection):
     """A connection from a remote node to this node."""
@@ -674,47 +722,31 @@ class ServerConnection(Connection):
     # ping the client. Otherwise, it would do it about half of the time.
     KEEP_ALIVE = Connection.KEEP_ALIVE + 5
 
+    server = True
+
     def __init__(self, *args, **kw):
         Connection.__init__(self, *args, **kw)
         self.updateTimeout(time())
 
-    def isServer(self):
-        return True
-
 
 class MTClientConnection(ClientConnection):
     """A Multithread-safe version of ClientConnection."""
 
+    def __metaclass__(name, base, d):
+        for k in ('analyse', 'answer', 'checkTimeout',
+                  'process', 'readable', 'writable'):
+            d[k] = lockCheckWrapper(getattr(base[0], k).im_func)
+        return type(name, base, d)
+
     def __init__(self, *args, **kwargs):
         # _lock is only here for lock debugging purposes. Do not use.
         self._lock = lock = RLock()
-        self.acquire = lock.acquire
-        self.release = lock.release
+        self.lock = lock.acquire
+        self.unlock = lock.release
         self.dispatcher = kwargs.pop('dispatcher')
         self.dispatcher.needPollThread()
-        self.lock()
-        try:
+        with lock:
             super(MTClientConnection, self).__init__(*args, **kwargs)
-        finally:
-            self.unlock()
-
-    def lock(self, blocking = 1):
-        return self.acquire(blocking = blocking)
-
-    def unlock(self):
-        self.release()
-
-    @lockCheckWrapper
-    def writable(self, *args, **kw):
-        return super(MTClientConnection, self).writable(*args, **kw)
-
-    @lockCheckWrapper
-    def readable(self, *args, **kw):
-        return super(MTClientConnection, self).readable(*args, **kw)
-
-    @lockCheckWrapper
-    def analyse(self, *args, **kw):
-        return super(MTClientConnection, self).analyse(*args, **kw)
 
     def notify(self, *args, **kw):
         self.lock()
@@ -725,7 +757,7 @@ class MTClientConnection(ClientConnection):
 
     @profiler_decorator
     def ask(self, packet, timeout=CRITICAL_TIMEOUT, on_timeout=None,
-            queue=None):
+            queue=None, **kw):
         self.lock()
         try:
             if self.isClosed():
@@ -744,28 +776,15 @@ class MTClientConnection(ClientConnection):
             self._addPacket(packet)
             handlers = self._handlers
             t = not handlers.isPending() and time() or None
-            handlers.emit(packet, timeout, on_timeout)
+            handlers.emit(packet, timeout, on_timeout, kw)
             self.updateTimeout(t)
             return msg_id
         finally:
             self.unlock()
 
-    @lockCheckWrapper
-    def answer(self, *args, **kw):
-        return super(MTClientConnection, self).answer(*args, **kw)
-
-    @lockCheckWrapper
-    def checkTimeout(self, *args, **kw):
-        return super(MTClientConnection, self).checkTimeout(*args, **kw)
-
     def close(self):
         self.lock()
         try:
             super(MTClientConnection, self).close()
         finally:
-            self.release()
-
-    @lockCheckWrapper
-    def process(self, *args, **kw):
-        return super(MTClientConnection, self).process(*args, **kw)
-
+            self.unlock()
