@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2010  Nexedi SA
+# Copyright (C) 2006-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,22 +12,22 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from time import time
-import neo.lib
-from neo.lib.epoll import Epoll
-from neo.lib.profiling import profiler_decorator
+from select import epoll, EPOLLIN, EPOLLOUT, EPOLLERR, EPOLLHUP
+from errno import EINTR, EAGAIN
+from . import logging
+from .profiling import profiler_decorator
 
 class EpollEventManager(object):
     """This class manages connections and events based on epoll(5)."""
 
     def __init__(self):
         self.connection_dict = {}
-        self.reader_set = set([])
-        self.writer_set = set([])
-        self.epoll = Epoll()
+        self.reader_set = set()
+        self.writer_set = set()
+        self.epoll = epoll()
         self._pending_processing = []
 
     def close(self):
@@ -37,7 +37,8 @@ class EpollEventManager(object):
 
     def getConnectionList(self):
         # XXX: use index
-        return [x for x in self.connection_dict.values() if not x.isAborted()]
+        return [x for x in self.connection_dict.itervalues()
+            if not x.isAborted()]
 
     def getClientList(self):
         # XXX: use index
@@ -77,9 +78,8 @@ class EpollEventManager(object):
         self.epoll.unregister(fd)
         del self.connection_dict[fd]
 
-    def _getPendingConnection(self):
-        if self._pending_processing:
-            return self._pending_processing.pop(0)
+    def isIdle(self):
+        return not (self._pending_processing or self.writer_set)
 
     def _addPendingConnection(self, conn):
         pending_processing = self._pending_processing
@@ -87,67 +87,78 @@ class EpollEventManager(object):
             pending_processing.append(conn)
 
     def poll(self, timeout=1):
-        to_process = self._getPendingConnection()
-        if to_process is None:
+        if not self._pending_processing:
             # Fetch messages from polled file descriptors
             self._poll(timeout=timeout)
-            # See if there is anything to process
-            to_process = self._getPendingConnection()
-        if to_process is not None:
-            to_process.lock()
+            if not self._pending_processing:
+                return
+        to_process = self._pending_processing.pop(0)
+        to_process.lock()
+        try:
             try:
-                try:
-                    # Process
-                    to_process.process()
-                finally:
-                    # ...and requeue if there are pending messages
-                    if to_process.hasPendingMessages():
-                        self._addPendingConnection(to_process)
+                to_process.process()
             finally:
-                to_process.unlock()
-            # Non-blocking call: as we handled a packet, we should just offer
-            # poll a chance to fetch & send already-available data, but it must
-            # not delay us.
-            self._poll(timeout=0)
+                # ...and requeue if there are pending messages
+                if to_process.hasPendingMessages():
+                    self._addPendingConnection(to_process)
+        finally:
+            to_process.unlock()
+        # Non-blocking call: as we handled a packet, we should just offer
+        # poll a chance to fetch & send already-available data, but it must
+        # not delay us.
+        self._poll(timeout=0)
 
     def _poll(self, timeout=1):
-        rlist, wlist, elist = self.epoll.poll(timeout)
-        for fd in frozenset(rlist):
-            conn = self.connection_dict[fd]
-            conn.lock()
-            try:
-                conn.readable()
-            finally:
-                conn.unlock()
-            if conn.hasPendingMessages():
-                self._addPendingConnection(conn)
-
-        for fd in frozenset(wlist):
-            # This can fail, if a connection is closed in readable().
-            try:
+        try:
+            event_list = self.epoll.poll(timeout)
+        except IOError, exc:
+            if exc.errno in (0, EAGAIN):
+                logging.info('epoll.poll triggered undocumented error %r',
+                    exc.errno)
+            elif exc.errno != EINTR:
+                raise
+            event_list = ()
+        wlist = []
+        elist = []
+        for fd, event in event_list:
+            if event & EPOLLIN:
                 conn = self.connection_dict[fd]
-            except KeyError:
-                pass
-            else:
-                conn.lock()
-                try:
-                    conn.writable()
-                finally:
-                    conn.unlock()
-
-        for fd in frozenset(elist):
-            # This can fail, if a connection is closed in previous calls to
-            # readable() or writable().
-            try:
-                conn = self.connection_dict[fd]
-            except KeyError:
-                pass
-            else:
                 conn.lock()
                 try:
                     conn.readable()
                 finally:
                     conn.unlock()
+                if conn.hasPendingMessages():
+                    self._addPendingConnection(conn)
+            if event & EPOLLOUT:
+                wlist.append(fd)
+            if event & (EPOLLERR | EPOLLHUP):
+                elist.append(fd)
+
+        for fd in wlist:
+            # This can fail, if a connection is closed in readable().
+            try:
+                conn = self.connection_dict[fd]
+            except KeyError:
+                continue
+            conn.lock()
+            try:
+                conn.writable()
+            finally:
+                conn.unlock()
+
+        for fd in elist:
+            # This can fail, if a connection is closed in previous calls to
+            # readable() or writable().
+            try:
+                conn = self.connection_dict[fd]
+            except KeyError:
+                continue
+            conn.lock()
+            try:
+                conn.readable()
+            finally:
+                conn.unlock()
             if conn.hasPendingMessages():
                 self._addPendingConnection(conn)
 
@@ -165,7 +176,8 @@ class EpollEventManager(object):
         fd = connector.getDescriptor()
         if fd not in self.reader_set:
             self.reader_set.add(fd)
-            self.epoll.modify(fd, 1, fd in self.writer_set)
+            self.epoll.modify(fd, EPOLLIN | (
+                fd in self.writer_set and EPOLLOUT))
 
     def removeReader(self, conn):
         connector = conn.getConnector()
@@ -173,7 +185,7 @@ class EpollEventManager(object):
         fd = connector.getDescriptor()
         if fd in self.reader_set:
             self.reader_set.remove(fd)
-            self.epoll.modify(fd, 0, fd in self.writer_set)
+            self.epoll.modify(fd, fd in self.writer_set and EPOLLOUT)
 
     @profiler_decorator
     def addWriter(self, conn):
@@ -182,7 +194,8 @@ class EpollEventManager(object):
         fd = connector.getDescriptor()
         if fd not in self.writer_set:
             self.writer_set.add(fd)
-            self.epoll.modify(fd, fd in self.reader_set, 1)
+            self.epoll.modify(fd, EPOLLOUT | (
+                fd in self.reader_set and EPOLLIN))
 
     def removeWriter(self, conn):
         connector = conn.getConnector()
@@ -190,16 +203,16 @@ class EpollEventManager(object):
         fd = connector.getDescriptor()
         if fd in self.writer_set:
             self.writer_set.remove(fd)
-            self.epoll.modify(fd, fd in self.reader_set, 0)
+            self.epoll.modify(fd, fd in self.reader_set and EPOLLIN)
 
     def log(self):
-        neo.lib.logging.info('Event Manager:')
-        neo.lib.logging.info('  Readers: %r', [x for x in self.reader_set])
-        neo.lib.logging.info('  Writers: %r', [x for x in self.writer_set])
-        neo.lib.logging.info('  Connections:')
+        logging.info('Event Manager:')
+        logging.info('  Readers: %r', list(self.reader_set))
+        logging.info('  Writers: %r', list(self.writer_set))
+        logging.info('  Connections:')
         pending_set = set(self._pending_processing)
         for fd, conn in self.connection_dict.items():
-            neo.lib.logging.info('    %r: %r (pending=%r)', fd, conn,
+            logging.info('    %r: %r (pending=%r)', fd, conn,
                 conn in pending_set)
 
 
