@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2010  Nexedi SA
+# Copyright (C) 2006-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,13 +12,12 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import neo.lib
-from neo.lib.protocol import NodeStates, ErrorCodes, Packets, Errors
-from neo.lib.protocol import PacketMalformedError, UnexpectedPacketError, \
-        BrokenNodeDisallowedError, NotReadyError, ProtocolError
+from . import logging
+from .protocol import (
+    NodeStates, Packets, Errors, BrokenNodeDisallowedError,
+    NotReadyError, PacketMalformedError, ProtocolError, UnexpectedPacketError)
 
 
 class EventHandler(object):
@@ -38,79 +37,82 @@ class EventHandler(object):
         else:
             message = 'unexpected packet: %s in %s' % (message,
                     self.__class__.__name__)
-        neo.lib.logging.error(message)
+        logging.error(message)
         conn.answer(Errors.ProtocolError(message))
         conn.abort()
         # self.peerBroken(conn)
 
-    def dispatch(self, conn, packet):
+    def dispatch(self, conn, packet, kw={}):
         """This is a helper method to handle various packet types."""
         try:
+            conn.setPeerId(packet.getId())
             try:
                 method = getattr(self, packet.handler_method_name)
             except AttributeError:
                 raise UnexpectedPacketError('no handler found')
             args = packet.decode() or ()
-            conn.setPeerId(packet.getId())
-            method(conn, *args)
+            method(conn, *args, **kw)
         except UnexpectedPacketError, e:
-            self.__unexpectedPacket(conn, packet, *e.args)
-        except PacketMalformedError:
-            neo.lib.logging.error('malformed packet from %r', conn)
-            conn.notify(Packets.Notify('Malformed packet: %r' % (packet, )))
-            conn.abort()
+            if not conn.isClosed():
+                self.__unexpectedPacket(conn, packet, *e.args)
+        except PacketMalformedError, e:
+            logging.error('malformed packet from %r: %s', conn, e)
+            conn.close()
             # self.peerBroken(conn)
         except BrokenNodeDisallowedError:
-            conn.answer(Errors.BrokenNode('go away'))
-            conn.abort()
+            if not conn.isClosed():
+                conn.answer(Errors.BrokenNode('go away'))
+                conn.abort()
         except NotReadyError, message:
-            if not message.args:
-                message = 'Retry Later'
-            message = str(message)
-            conn.answer(Errors.NotReady(message))
-            conn.abort()
+            if not conn.isClosed():
+                if not message.args:
+                    message = 'Retry Later'
+                message = str(message)
+                conn.answer(Errors.NotReady(message))
+                conn.abort()
         except ProtocolError, message:
-            message = str(message)
-            conn.answer(Errors.ProtocolError(message))
-            conn.abort()
+            if not conn.isClosed():
+                message = str(message)
+                conn.answer(Errors.ProtocolError(message))
+                conn.abort()
 
     def checkClusterName(self, name):
         # raise an exception if the given name mismatch the current cluster name
         if self.app.name != name:
-            neo.lib.logging.error('reject an alien cluster')
+            logging.error('reject an alien cluster')
             raise ProtocolError('invalid cluster name')
 
 
     # Network level handlers
 
-    def packetReceived(self, conn, packet):
+    def packetReceived(self, *args):
         """Called when a packet is received."""
-        self.dispatch(conn, packet)
+        self.dispatch(*args)
 
     def connectionStarted(self, conn):
         """Called when a connection is started."""
-        neo.lib.logging.debug('connection started for %r', conn)
+        logging.debug('connection started for %r', conn)
 
     def connectionCompleted(self, conn):
         """Called when a connection is completed."""
-        neo.lib.logging.debug('connection completed for %r (from %s:%u)',
-                              conn, *conn.getConnector().getAddress())
+        logging.debug('connection completed for %r (from %s:%u)',
+                      conn, *conn.getConnector().getAddress())
 
     def connectionFailed(self, conn):
         """Called when a connection failed."""
-        neo.lib.logging.debug('connection failed for %r', conn)
+        logging.debug('connection failed for %r', conn)
 
     def connectionAccepted(self, conn):
         """Called when a connection is accepted."""
 
     def connectionClosed(self, conn):
         """Called when a connection is closed by the peer."""
-        neo.lib.logging.debug('connection closed for %r', conn)
+        logging.debug('connection closed for %r', conn)
         self.connectionLost(conn, NodeStates.TEMPORARILY_DOWN)
 
     #def peerBroken(self, conn):
     #    """Called when a peer is broken."""
-    #    neo.lib.logging.error('%r is broken', conn)
+    #    logging.error('%r is broken', conn)
     #    # NodeStates.BROKEN
 
     def connectionLost(self, conn, new_state):
@@ -121,9 +123,26 @@ class EventHandler(object):
 
     # Packet handlers.
 
+    def acceptIdentification(self, conn, node_type, *args):
+        try:
+            acceptIdentification = self._acceptIdentification
+        except AttributeError:
+            raise UnexpectedPacketError('no handler found')
+        if conn.isClosed():
+            # acceptIdentification received on a closed (probably aborted,
+            # actually) connection. Reject any further packet as unexpected.
+            conn.setHandler(EventHandler(self.app))
+            return
+        node = self.app.nm.getByAddress(conn.getAddress())
+        assert node.getConnection() is conn, (node.getConnection(), conn)
+        if node.getType() == node_type:
+            node.setIdentified()
+            acceptIdentification(node, *args)
+            return
+        conn.close()
+
     def ping(self, conn):
-        if not conn.isAborted():
-            conn.answer(Packets.Pong())
+        conn.answer(Packets.Pong())
 
     def pong(self, conn):
         # Ignore PONG packets. The only purpose of ping/pong packets is
@@ -131,7 +150,12 @@ class EventHandler(object):
         pass
 
     def notify(self, conn, message):
-        neo.lib.logging.info('notification from %r: %s', conn, message)
+        logging.warning('notification from %r: %s', conn, message)
+
+    def closeClient(self, conn):
+        conn.server = False
+        if not conn.client:
+            conn.close()
 
     # Error packet handlers.
 
@@ -143,16 +167,16 @@ class EventHandler(object):
 
     def protocolError(self, conn, message):
         # the connection should have been closed by the remote peer
-        neo.lib.logging.error('protocol error: %s' % (message,))
+        logging.error('protocol error: %s', message)
 
     def timeoutError(self, conn, message):
-        neo.lib.logging.error('timeout error: %s' % (message,))
+        logging.error('timeout error: %s', message)
 
     def brokenNodeDisallowedError(self, conn, message):
         raise RuntimeError, 'broken node disallowed error: %s' % (message,)
 
     def alreadyPendingError(self, conn, message):
-        neo.lib.logging.error('already pending error: %s' % (message, ))
+        logging.error('already pending error: %s', message)
 
     def ack(self, conn, message):
-        neo.lib.logging.debug("no error message : %s" % (message))
+        logging.debug("no error message: %s", message)
