@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2010  Nexedi SA
+# Copyright (C) 2006-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,16 +12,15 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from time import time
+from os.path import exists, getsize
+import json
 
-import neo.lib
-from neo.lib.util import dump
-from neo.lib.protocol import NodeTypes, NodeStates
+from . import attributeTracker, logging
+from .protocol import uuid_str, NodeTypes, NodeStates, ProtocolError
 
-from neo.lib import attributeTracker
 
 class Node(object):
     """This class represents a node."""
@@ -35,6 +34,7 @@ class Node(object):
         self._uuid = uuid
         self._manager = manager
         self._last_state_change = time()
+        self._identified = False
         manager.add(self)
 
     def notify(self, packet):
@@ -58,10 +58,14 @@ class Node(object):
     def setState(self, new_state):
         if self._state == new_state:
             return
-        old_state = self._state
-        self._state = new_state
+        if new_state == NodeStates.DOWN:
+            self._manager.remove(self)
+            self._state = new_state
+        else:
+            old_state = self._state
+            self._state = new_state
+            self._manager._updateState(self, old_state)
         self._last_state_change = time()
-        self._manager._updateState(self, old_state)
 
     def setAddress(self, address):
         if self._address == address:
@@ -80,6 +84,8 @@ class Node(object):
         self._uuid = uuid
         self._manager._updateUUID(self, old_uuid)
         self._manager._updateIdentified(self)
+        if self._connection is not None:
+            self._connection.setUUID(uuid)
 
     def getUUID(self):
         return self._uuid
@@ -90,15 +96,45 @@ class Node(object):
         """
         assert self._connection is not None
         del self._connection
+        self._identified = False
         self._manager._updateIdentified(self)
 
-    def setConnection(self, connection):
+    def setConnection(self, connection, force=None):
         """
             Define the connection that is currently available to this node.
+            If there is already a connection set, 'force' must be given:
+            the new connection replaces the old one if it is true. In any case,
+            the node must be managed by the same handler for the client and
+            server parts.
         """
-        assert connection is not None
-        assert self._connection is None
-        self._connection = connection
+        assert connection.getUUID() in (None, self._uuid), connection
+        connection.setUUID(self._uuid)
+        conn = self._connection
+        if conn is None:
+            self._connection = connection
+            if connection.isServer():
+                self.setIdentified()
+        else:
+            assert force is not None, \
+                attributeTracker.whoSet(self, '_connection')
+            # The test on peer_id is there to protect against buggy nodes.
+            # XXX: handler comparison does not cover all cases: there may
+            # be a pending handler change, which won't be detected, or a future
+            # handler change which is not prevented. Complete implementation
+            # should allow different handlers for each connection direction,
+            # with in-packets client/server indicators to decide which handler
+            # (server-ish or client-ish) to use. There is currently no need for
+            # the full-fledged functionality, and it is simpler this way.
+            if not force or conn.getPeerId() is not None or \
+               type(conn.getHandler()) is not type(connection.getHandler()):
+                raise ProtocolError("already connected")
+            def on_closed():
+                self._connection = connection
+                assert connection.isServer()
+                self.setIdentified()
+            conn.setOnClose(on_closed)
+            conn.close()
+        assert not connection.isClosed(), connection
         connection.setOnClose(self.onConnectionClosed)
         self._manager._updateIdentified(self)
 
@@ -109,24 +145,30 @@ class Node(object):
         assert self._connection is not None
         return self._connection
 
-    def isConnected(self):
+    def isConnected(self, connecting=False):
         """
             Returns True is a connection is established with the node
         """
-        return self._connection is not None
+        return self._connection is not None and (connecting or
+            not self._connection.connecting)
+
+    def setIdentified(self):
+        assert self._connection is not None
+        self._identified = True
 
     def isIdentified(self):
         """
-            Returns True is the node is connected and identified
+            Returns True if identification packets have been exchanged
         """
-        return self._connection is not None and self._uuid is not None
+        return self._identified
 
     def __repr__(self):
-        return '<%s(uuid=%s, address=%s, state=%s) at %x>' % (
+        return '<%s(uuid=%s, address=%s, state=%s, connection=%r) at %x>' % (
             self.__class__.__name__,
-            dump(self._uuid),
+            uuid_str(self._uuid),
             self._address,
             self._state,
+            self._connection,
             id(self),
         )
 
@@ -247,37 +289,87 @@ NODE_CLASS_MAPPING = {
     AdminNode: NodeTypes.ADMIN,
 }
 
+class MasterDB(object):
+    """
+    Manages accesses to master's address database.
+    """
+    def __init__(self, path):
+        self._path = path
+        try_load = exists(path) and getsize(path)
+        if try_load:
+            db = open(path, 'r')
+            init_set = map(tuple, json.load(db))
+        else:
+            db = open(path, 'w+')
+            init_set = []
+        self._set = set(init_set)
+        db.close()
+
+    def _save(self):
+        try:
+            db = open(self._path, 'w')
+        except IOError:
+            logging.warning('failed opening master database at %r '
+                'for writing, update skipped', self._path)
+        else:
+            json.dump(list(self._set), db)
+            db.close()
+
+    def add(self, addr):
+        self._set.add(addr)
+        self._save()
+
+    def discard(self, addr):
+        self._set.discard(addr)
+        self._save()
+
+    def __iter__(self):
+        return iter(self._set)
+
 class NodeManager(object):
     """This class manages node status."""
+    _master_db = None
 
     # TODO: rework getXXXList() methods, filter first by node type
     # - getStorageList(identified=True, connected=True, )
     # - getList(...)
 
-    def __init__(self):
+    def __init__(self, master_db=None):
+        """
+        master_db (string)
+        Path to a file containing master nodes's addresses. Used to automate
+        master list updates. If not provided, no automation will happen.
+        """
         self._node_set = set()
         self._address_dict = {}
         self._uuid_dict = {}
         self._type_dict = {}
         self._state_dict = {}
         self._identified_dict = {}
+        if master_db is not None:
+            self._master_db = db = MasterDB(master_db)
+            for addr in db:
+                self.createMaster(address=addr)
 
     close = __init__
 
     def add(self, node):
         if node in self._node_set:
-            neo.lib.logging.warning('adding a known node %r, ignoring', node)
+            logging.warning('adding a known node %r, ignoring', node)
             return
+        assert not node.isDown(), node
         self._node_set.add(node)
         self._updateAddress(node, None)
         self._updateUUID(node, None)
         self.__updateSet(self._type_dict, None, node.__class__, node)
         self.__updateSet(self._state_dict, None, node.getState(), node)
         self._updateIdentified(node)
+        if node.isMaster() and self._master_db is not None:
+            self._master_db.add(node.getAddress())
 
     def remove(self, node):
         if node not in self._node_set:
-            neo.lib.logging.warning('removing unknown node %r, ignoring', node)
+            logging.warning('removing unknown node %r, ignoring', node)
             return
         self._node_set.remove(node)
         self.__drop(self._address_dict, node.getAddress())
@@ -287,6 +379,8 @@ class NodeManager(object):
         uuid = node.getUUID()
         if uuid in self._identified_dict:
             del self._identified_dict[uuid]
+        if node.isMaster() and self._master_db is not None:
+            self._master_db.discard(node.getAddress())
 
     def __drop(self, index_dict, key):
         try:
@@ -305,14 +399,19 @@ class NodeManager(object):
                 new_key)
             del index_dict[old_key]
         if new_key is not None:
+            assert index_dict.get(new_key, node) is node, 'Adding %r at %r ' \
+                'would overwrite %r' % (node, new_key, index_dict[new_key])
             index_dict[new_key] = node
 
     def _updateIdentified(self, node):
         uuid = node.getUUID()
-        if node.isIdentified():
-            self._identified_dict[uuid] = node
-        else:
-            self._identified_dict.pop(uuid, None)
+        if uuid:
+            # XXX: It's probably a bug to include connecting nodes but there's
+            #      no API yet to update manager when connection is established.
+            if node.isConnected(connecting=True):
+                self._identified_dict[uuid] = node
+            else:
+                self._identified_dict.pop(uuid, None)
 
     def _updateAddress(self, node, old_address):
         self.__update(self._address_dict, old_address, node.getAddress(), node)
@@ -332,10 +431,11 @@ class NodeManager(object):
             set_dict.setdefault(new_key, set()).add(node)
 
     def _updateState(self, node, old_state):
+        assert not node.isDown(), node
         self.__updateSet(self._state_dict, old_state, node.getState(), node)
 
     def getList(self, node_filter=None):
-        if filter is None:
+        if node_filter is None:
             return list(self._node_set)
         return filter(node_filter, self._node_set)
 
@@ -347,7 +447,7 @@ class NodeManager(object):
         if pool_set is not None:
             identified_nodes = self._identified_dict.items()
             return [v for k, v in identified_nodes if k in pool_set]
-        return list(self._identified_dict.values())
+        return self._identified_dict.values()
 
     def getConnectedList(self):
         """
@@ -399,8 +499,36 @@ class NodeManager(object):
     def hasUUID(self, uuid):
         return uuid in self._uuid_dict
 
-    def _createNode(self, klass, **kw):
-        return klass(self, **kw)
+    def _createNode(self, klass, address=None, uuid=None, **kw):
+        by_address = self.getByAddress(address)
+        by_uuid = self.getByUUID(uuid)
+        if by_address is None and by_uuid is None:
+            node = klass(self, address=address, uuid=uuid, **kw)
+        else:
+            if by_uuid is None or by_address is by_uuid:
+                node = by_address
+            elif by_address is None:
+                node = by_uuid
+            else:
+                raise ValueError('Got different nodes for uuid %s: %r and '
+                    'address %r: %r.' % (uuid_str(uuid), by_uuid, address,
+                    by_address))
+            if uuid is not None:
+                node_uuid = node.getUUID()
+                if node_uuid is None:
+                    node.setUUID(uuid)
+                elif node_uuid != uuid:
+                    raise ValueError('Expected uuid %s on node %r' % (
+                        uuid_str(uuid), node))
+            if address is not None:
+                node_address = node.getAddress()
+                if node_address is None:
+                    node.setAddress(address)
+                elif node_address != address:
+                    raise ValueError('Expected address %r on node %r' % (
+                        address, node))
+            assert node.__class__ is klass, (node.__class__, klass)
+        return node
 
     def createMaster(self, **kw):
         """ Create and register a new master """
@@ -445,15 +573,14 @@ class NodeManager(object):
             node_by_addr = self.getByAddress(addr)
             node = node_by_uuid or node_by_addr
 
-            log_args = (node_type, dump(uuid), addr, state)
+            log_args = node_type, uuid_str(uuid), addr, state
             if node is None:
                 if state == NodeStates.DOWN:
-                    neo.lib.logging.debug('NOT creating node %s %s %s %s',
-                        *log_args)
+                    logging.debug('NOT creating node %s %s %s %s', *log_args)
                 else:
                     node = self._createNode(klass, address=addr, uuid=uuid,
                             state=state)
-                    neo.lib.logging.debug('creating node %r', node)
+                    logging.debug('creating node %r', node)
             else:
                 assert isinstance(node, klass), 'node %r is not ' \
                     'of expected type: %r' % (node, klass)
@@ -462,15 +589,14 @@ class NodeManager(object):
                     'Discrepancy between node_by_uuid (%r) and ' \
                     'node_by_addr (%r)' % (node_by_uuid, node_by_addr)
                 if state == NodeStates.DOWN:
-                    neo.lib.logging.debug(
-                                    'droping node %r (%r), found with %s ' \
+                    logging.debug('droping node %r (%r), found with %s '
                         '%s %s %s', node, node.isConnected(), *log_args)
                     if node.isConnected():
                         # cut this connection, node removed by handler
                         node.getConnection().close()
                     self.remove(node)
                 else:
-                    neo.lib.logging.debug('updating node %r to %s %s %s %s',
+                    logging.debug('updating node %r to %s %s %s %s',
                         node, *log_args)
                     node.setUUID(uuid)
                     node.setAddress(addr)
@@ -478,12 +604,14 @@ class NodeManager(object):
         self.log()
 
     def log(self):
-        neo.lib.logging.info('Node manager : %d nodes' % len(self._node_set))
-        for node in sorted(list(self._node_set)):
-            uuid = dump(node.getUUID()) or '-' * 32
-            address = node.getAddress() or ''
-            if address:
-                address = '%s:%d' % address
-            neo.lib.logging.info(' * %32s | %8s | %22s | %s' % (
-                uuid, node.getType(), address, node.getState()))
-
+        logging.info('Node manager : %u nodes', len(self._node_set))
+        if self._node_set:
+            node_list = [(node, uuid_str(node.getUUID()))
+                         for node in sorted(self._node_set)]
+            max_len = max(len(x[1]) for x in node_list)
+            for node, uuid in node_list:
+                address = node.getAddress() or ''
+                if address:
+                    address = '%s:%d' % address
+                logging.info(' * %*s | %8s | %22s | %s',
+                    max_len, uuid, node.getType(), address, node.getState())
