@@ -1,5 +1,5 @@
 
-# Copyright (C) 2006-2010  Nexedi SA
+# Copyright (C) 2006-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,20 +12,19 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import socket
 import sys
 import traceback
-from socket import inet_ntoa, inet_aton
 from cStringIO import StringIO
 from struct import Struct
+try:
+    from .util import getAddressType
+except ImportError:
+    pass
 
-from neo.lib.util import Enum, getAddressType
-
-# The protocol version (major, minor).
-PROTOCOL_VERSION = (4, 1)
+PROTOCOL_VERSION = 1
 
 # Size restrictions.
 MIN_PACKET_SIZE = 10
@@ -39,53 +38,98 @@ assert PACKET_HEADER_FORMAT.size == 10, \
     (PACKET_HEADER_FORMAT.size, )
 RESPONSE_MASK = 0x8000
 
-class ErrorCodes(Enum):
-    ACK = Enum.Item(0)
-    NOT_READY = Enum.Item(1)
-    OID_NOT_FOUND = Enum.Item(2)
-    OID_DOES_NOT_EXIST = Enum.Item(6)
-    TID_NOT_FOUND = Enum.Item(3)
-    PROTOCOL_ERROR = Enum.Item(4)
-    BROKEN_NODE = Enum.Item(5)
-    ALREADY_PENDING = Enum.Item(7)
-ErrorCodes = ErrorCodes()
+class Enum(tuple):
 
-class ClusterStates(Enum):
-    RECOVERING = Enum.Item(1)
-    VERIFYING = Enum.Item(2)
-    RUNNING = Enum.Item(3)
-    STOPPING = Enum.Item(4)
-ClusterStates = ClusterStates()
+    class Item(int):
+        __slots__ = '_name', '_enum'
+        def __str__(self):
+            return self._name
+        def __repr__(self):
+            return "<EnumItem %s (%d)>" % (self._name, self)
+        def __eq__(self, other):
+            if type(other) is self.__class__:
+                assert other._enum is self._enum
+                return other is self
+            return other == int(self)
 
-class NodeTypes(Enum):
-    MASTER = Enum.Item(1)
-    STORAGE = Enum.Item(2)
-    CLIENT = Enum.Item(3)
-    ADMIN = Enum.Item(4)
-NodeTypes = NodeTypes()
+    def __new__(cls, func):
+        names = func.func_code.co_names
+        self = tuple.__new__(cls, map(cls.Item, xrange(len(names))))
+        self._name = func.__name__
+        for item, name in zip(self, names):
+            setattr(self, name, item)
+            item._name = name
+            item._enum = self
+        return self
 
-class NodeStates(Enum):
-    RUNNING = Enum.Item(1)
-    TEMPORARILY_DOWN = Enum.Item(2)
-    DOWN = Enum.Item(3)
-    BROKEN = Enum.Item(4)
-    HIDDEN = Enum.Item(5)
-    PENDING = Enum.Item(6)
-    UNKNOWN = Enum.Item(7)
-NodeStates = NodeStates()
+    def __repr__(self):
+        return "<Enum %s>" % self._name
 
-class CellStates(Enum):
-    UP_TO_DATE = Enum.Item(1)
-    OUT_OF_DATE = Enum.Item(2)
-    FEEDING = Enum.Item(3)
-    DISCARDED = Enum.Item(4)
-CellStates = CellStates()
 
-class LockState(Enum):
-    NOT_LOCKED = Enum.Item(1)
-    GRANTED = Enum.Item(2)
-    GRANTED_TO_OTHER = Enum.Item(3)
-LockState = LockState()
+@Enum
+def ErrorCodes():
+    ACK
+    NOT_READY
+    OID_NOT_FOUND
+    TID_NOT_FOUND
+    OID_DOES_NOT_EXIST
+    PROTOCOL_ERROR
+    BROKEN_NODE
+    ALREADY_PENDING
+    REPLICATION_ERROR
+    CHECKING_ERROR
+
+@Enum
+def ClusterStates():
+    RECOVERING
+    VERIFYING
+    RUNNING
+    STOPPING
+    STARTING_BACKUP
+    BACKINGUP
+    STOPPING_BACKUP
+
+@Enum
+def NodeTypes():
+    MASTER
+    STORAGE
+    CLIENT
+    ADMIN
+
+@Enum
+def NodeStates():
+    RUNNING
+    TEMPORARILY_DOWN
+    DOWN
+    BROKEN
+    HIDDEN
+    PENDING
+    UNKNOWN
+
+@Enum
+def CellStates():
+    # Normal state: cell is writable/readable, and it isn't planned to drop it.
+    UP_TO_DATE
+    # Write-only cell. Last transactions are missing because storage is/was down
+    # for a while, or because it is new for the partition. It usually becomes
+    # UP_TO_DATE when replication is done.
+    OUT_OF_DATE
+    # Same as UP_TO_DATE, except that it will be discarded as soon as another
+    # node finishes to replicate it. It means a partition is moved from 1 node
+    # to another.
+    FEEDING
+    # Not really a state: only used in network packets to tell storages to drop
+    # partitions.
+    DISCARDED
+    # A check revealed that data differs from other replicas. Cell is neither
+    # readable nor writable.
+    CORRUPTED
+
+@Enum
+def LockState():
+    NOT_LOCKED
+    GRANTED
+    GRANTED_TO_OTHER
 
 # used for logging
 node_state_prefix_dict = {
@@ -104,10 +148,11 @@ cell_state_prefix_dict = {
     CellStates.OUT_OF_DATE: 'O',
     CellStates.FEEDING: 'F',
     CellStates.DISCARDED: 'D',
+    CellStates.CORRUPTED: 'C',
 }
 
 # Other constants.
-INVALID_UUID = '\0' * 16
+INVALID_UUID = 0
 INVALID_TID = '\xff' * 8
 INVALID_OID = '\xff' * 8
 INVALID_PARTITION = 0xffffffff
@@ -117,13 +162,26 @@ ZERO_TID = '\0' * 8
 ZERO_OID = '\0' * 8
 OID_LEN = len(INVALID_OID)
 TID_LEN = len(INVALID_TID)
+MAX_TID = '\x7f' + '\xff' * 7 # SQLite does not accept numbers above 2^63-1
 
+# High-order byte:
+# 7 6 5 4 3 2 1 0
+# | | | | +-+-+-+-- reserved (0)
+# | +-+-+---------- node type
+# +---------------- temporary if negative
+# UUID namespaces are required to prevent conflicts when the master generate
+# new uuid before it knows uuid of existing storage nodes. So only the high
+# order bit is really important and the 31 other bits could be random.
+# Extra namespace information and non-randomness of 3 LOB help to read logs.
 UUID_NAMESPACES = {
-    NodeTypes.STORAGE: 'S',
-    NodeTypes.MASTER: 'M',
-    NodeTypes.CLIENT: 'C',
-    NodeTypes.ADMIN: 'A',
+    NodeTypes.STORAGE: 0x00,
+    NodeTypes.MASTER: -0x10,
+    NodeTypes.CLIENT: -0x20,
+    NodeTypes.ADMIN: -0x30,
 }
+uuid_str = (lambda ns: lambda uuid:
+    ns[uuid >> 24] + str(uuid & 0xffffff) if uuid else str(uuid)
+    )(dict((v, str(k)[0]) for k, v in UUID_NAMESPACES.iteritems()))
 
 class ProtocolError(Exception):
     """ Base class for protocol errors, close the connection """
@@ -511,21 +569,18 @@ class PPTID(PStructItem):
             value = None
         return value
 
-class PProtocol(PStructItem):
+class PProtocol(PNumber):
     """
         The protocol version definition
     """
-    def __init__(self, name):
-        PStructItem.__init__(self, name, '!LL')
-
     def _encode(self, writer, version):
-        writer(self.pack(*version))
+        writer(self.pack(version))
 
     def _decode(self, reader):
-        major, minor = self.unpack(reader(self.size))
-        if (major, minor) != PROTOCOL_VERSION:
+        version = self.unpack(reader(self.size))
+        if version != (PROTOCOL_VERSION,):
             raise ProtocolError('protocol version mismatch')
-        return (major, minor)
+        return version
 
 class PChecksum(PItem):
     """
@@ -538,21 +593,18 @@ class PChecksum(PItem):
     def _decode(self, reader):
         return reader(20)
 
-class PUUID(PItem):
+class PUUID(PStructItem):
     """
-        An UUID (node identifier)
+        An UUID (node identifier, 4-bytes signed integer)
     """
+    def __init__(self, name):
+        PStructItem.__init__(self, name, '!l')
+
     def _encode(self, writer, uuid):
-        if uuid is None:
-            uuid = INVALID_UUID
-        assert len(uuid) == 16, (len(uuid), uuid)
-        writer(uuid)
+        writer(self.pack(uuid or 0))
 
     def _decode(self, reader):
-        uuid = reader(16)
-        if uuid == INVALID_UUID:
-            uuid = None
-        return uuid
+        return self.unpack(reader(self.size))[0] or None
 
 class PTID(PItem):
     """
@@ -649,6 +701,11 @@ class Ping(Packet):
     """
     _answer = PFEmpty
 
+class CloseClient(Packet):
+    """
+    Tell peer it can close the connection if it has finished with us. Any -> Any
+    """
+
 class RequestIdentification(Packet):
     """
     Request a node identification. This must be the first packet for any
@@ -669,6 +726,13 @@ class RequestIdentification(Packet):
         PNumber('num_partitions'),
         PNumber('num_replicas'),
         PUUID('your_uuid'),
+        PAddress('primary'),
+        PList('known_master_list',
+            PStruct('master',
+                PAddress('address'),
+                PUUID('uuid'),
+            ),
+        ),
     )
 
     def __init__(self, *args, **kw):
@@ -683,19 +747,10 @@ class RequestIdentification(Packet):
 
 class PrimaryMaster(Packet):
     """
-    Ask a current primary master node. This must be the second message when
-    connecting to a master node. Any -> M.
-    Reply to Ask Primary Master. This message includes a list of known master
-    nodes to make sure that a peer has the same information. M -> Any.
+    Ask current primary master's uuid. CTL -> A.
     """
     _answer = PStruct('answer_primary',
         PUUID('primary_uuid'),
-        PList('known_master_list',
-            PStruct('master',
-                PAddress('address'),
-                PUUID('uuid'),
-            ),
-        ),
     )
 
 class AnnouncePrimary(Packet):
@@ -710,14 +765,14 @@ class ReelectPrimary(Packet):
 
 class LastIDs(Packet):
     """
-    Ask the last OID, the last TID and the last Partition Table ID that
-    a storage node stores. Used to recover information. PM -> S, S -> PM.
-    Reply to Ask Last IDs. S -> PM, PM -> S.
+    Ask the last OID, the last TID and the last Partition Table ID so that
+    a master recover. PM -> S, S -> PM.
     """
     _answer = PStruct('answer_last_ids',
         POID('last_oid'),
         PTID('last_tid'),
         PPTID('last_ptid'),
+        PTID('backup_tid'),
     )
 
 class PartitionTable(Packet):
@@ -750,26 +805,19 @@ class PartitionChanges(Packet):
             PStruct('cell',
                 PNumber('offset'),
                 PUUID('uuid'),
-                PFNodeState,
+                PFCellState,
             ),
         ),
     )
 
-class ReplicationDone(Packet):
-    """
-    Notify the master node that a partition has been successully replicated from
-    a storage to another.
-    S -> M
-    """
-    _fmt = PStruct('notify_replication_done',
-        PNumber('offset'),
-    )
-
 class StartOperation(Packet):
     """
     Tell a storage nodes to start an operation. Until a storage node receives
     this message, it must not serve client nodes. PM -> S.
     """
+    _fmt = PStruct('start_operation',
+        PBoolean('backup'),
+    )
 
 class StopOperation(Packet):
     """
@@ -960,7 +1008,7 @@ class GetObject(Packet):
     """
     Ask a stored object by its OID and a serial or a TID if given. If a serial
     is specified, the specified revision of an object will be returned. If
-    a TID is specified, an object right before the TID will be returned. S,C -> S.
+    a TID is specified, an object right before the TID will be returned. C -> S.
     Answer the requested object. S -> C.
     """
     _fmt = PStruct('ask_object',
@@ -998,16 +1046,14 @@ class TIDList(Packet):
 class TIDListFrom(Packet):
     """
     Ask for length TIDs starting at min_tid. The order of TIDs is ascending.
-    S -> S.
-    Answer the requested TIDs. S -> S
+    C -> S.
+    Answer the requested TIDs. S -> C
     """
     _fmt = PStruct('tid_list_from',
         PTID('min_tid'),
         PTID('max_tid'),
         PNumber('length'),
-        PList('partition_list',
-            PNumber('partition'),
-        ),
+        PNumber('partition'),
     )
 
     _answer = PStruct('answer_tids',
@@ -1049,27 +1095,6 @@ class ObjectHistory(Packet):
         PFHistoryList,
     )
 
-class ObjectHistoryFrom(Packet):
-    """
-    Ask history information for a given object. The order of serials is
-    ascending, and starts at (or above) min_serial for min_oid. S -> S.
-    Answer the requested serials. S -> S.
-    """
-    _fmt = PStruct('ask_object_history',
-        POID('min_oid'),
-        PTID('min_serial'),
-        PTID('max_serial'),
-        PNumber('length'),
-        PNumber('partition'),
-    )
-
-    _answer = PStruct('ask_finish_transaction',
-        PDict('object_dict',
-            POID('oid'),
-            PFTidList,
-        ),
-    )
-
 class PartitionList(Packet):
     """
     All the following messages are for neoctl to admin node
@@ -1107,7 +1132,6 @@ class SetNodeState(Packet):
     _fmt = PStruct('set_node_state',
         PUUID('uuid'),
         PFNodeState,
-        PBoolean('modify_partition_table'),
     )
 
     _answer = Error
@@ -1122,6 +1146,16 @@ class AddPendingNodes(Packet):
 
     _answer = Error
 
+class TweakPartitionTable(Packet):
+    """
+    Ask the primary to optimize the partition table. A -> PM.
+    """
+    _fmt = PStruct('tweak_partition_table',
+        PFUUIDList,
+    )
+
+    _answer = Error
+
 class NotifyNodeInformation(Packet):
     """
     Notify information about one or more nodes. PM -> Any.
@@ -1164,14 +1198,6 @@ class ClusterState(Packet):
         PEnum('state', ClusterStates),
     )
 
-class NotifyLastOID(Packet):
-    """
-    Notify last OID generated
-    """
-    _fmt = PStruct('notify_last_oid',
-        POID('last_oid'),
-    )
-
 class ObjectUndoSerial(Packet):
     """
     Ask storage the serial where object data is when undoing given transaction,
@@ -1261,6 +1287,35 @@ class Pack(Packet):
         PBoolean('status'),
     )
 
+class CheckReplicas(Packet):
+    """
+    ctl -> A
+    A -> M
+    """
+    _fmt = PStruct('check_replicas',
+        PDict('partition_dict',
+            PNumber('partition'),
+            PUUID('source'),
+        ),
+        PTID('min_tid'),
+        PTID('max_tid'),
+    )
+    _answer = Error
+
+class CheckPartition(Packet):
+    """
+    M -> S
+    """
+    _fmt = PStruct('check_partition',
+        PNumber('partition'),
+        PStruct('source',
+            PString('upstream_name'),
+            PAddress('address'),
+        ),
+        PTID('min_tid'),
+        PTID('max_tid'),
+    )
+
 class CheckTIDRange(Packet):
     """
     Ask some stats about a range of transactions.
@@ -1273,15 +1328,13 @@ class CheckTIDRange(Packet):
     S -> S
     """
     _fmt = PStruct('ask_check_tid_range',
+        PNumber('partition'),
+        PNumber('length'),
         PTID('min_tid'),
         PTID('max_tid'),
-        PNumber('length'),
-        PNumber('partition'),
     )
 
     _answer = PStruct('answer_check_tid_range',
-        PTID('min_tid'),
-        PNumber('length'),
         PNumber('count'),
         PChecksum('checksum'),
         PTID('max_tid'),
@@ -1299,22 +1352,30 @@ class CheckSerialRange(Packet):
     S -> S
     """
     _fmt = PStruct('ask_check_serial_range',
-        POID('min_oid'),
-        PTID('min_serial'),
-        PTID('max_tid'),
-        PNumber('length'),
         PNumber('partition'),
+        PNumber('length'),
+        PTID('min_tid'),
+        PTID('max_tid'),
+        POID('min_oid'),
     )
 
     _answer = PStruct('answer_check_serial_range',
-        POID('min_oid'),
-        PTID('min_serial'),
-        PNumber('length'),
         PNumber('count'),
+        PChecksum('tid_checksum'),
+        PTID('max_tid'),
         PChecksum('oid_checksum'),
         POID('max_oid'),
-        PChecksum('serial_checksum'),
-        PTID('max_serial'),
+    )
+
+class PartitionCorrupted(Packet):
+    """
+    S -> M
+    """
+    _fmt = PStruct('partition_corrupted',
+        PNumber('partition'),
+        PList('cell_list',
+            PUUID('uuid'),
+        ),
     )
 
 class LastTransaction(Packet):
@@ -1336,13 +1397,122 @@ class NotifyReady(Packet):
     """
     pass
 
+# replication
+
+class FetchTransactions(Packet):
+    """
+    S -> S
+    """
+    _fmt = PStruct('ask_transaction_list',
+        PNumber('partition'),
+        PNumber('length'),
+        PTID('min_tid'),
+        PTID('max_tid'),
+        PFTidList,           # already known transactions
+    )
+    _answer = PStruct('answer_transaction_list',
+        PTID('pack_tid'),
+        PTID('next_tid'),
+        PFTidList,           # transactions to delete
+    )
+
+class AddTransaction(Packet):
+    """
+    S -> S
+    """
+    _fmt = PStruct('add_transaction',
+        PTID('tid'),
+        PString('user'),
+        PString('description'),
+        PString('extension'),
+        PBoolean('packed'),
+        PTID('ttid'),
+        PFOidList,
+    )
+
+class FetchObjects(Packet):
+    """
+    S -> S
+    """
+    _fmt = PStruct('ask_object_list',
+        PNumber('partition'),
+        PNumber('length'),
+        PTID('min_tid'),
+        PTID('max_tid'),
+        POID('min_oid'),
+        PDict('object_dict', # already known objects
+            PTID('serial'),
+            PFOidList,
+        ),
+    )
+    _answer = PStruct('answer_object_list',
+        PTID('pack_tid'),
+        PTID('next_tid'),
+        POID('next_oid'),
+        PDict('object_dict', # objects to delete
+            PTID('serial'),
+            PFOidList,
+        ),
+    )
+
+class AddObject(Packet):
+    """
+    S -> S
+    """
+    _fmt = PStruct('add_object',
+        POID('oid'),
+        PTID('serial'),
+        PBoolean('compression'),
+        PChecksum('checksum'),
+        PString('data'),
+        PTID('data_serial'),
+    )
+
+class Replicate(Packet):
+    """
+    M -> S
+    """
+    _fmt = PStruct('replicate',
+        PTID('tid'),
+        PString('upstream_name'),
+        PDict('source_dict',
+            PNumber('partition'),
+            PAddress('address'),
+        )
+    )
+
+class ReplicationDone(Packet):
+    """
+    Notify the master node that a partition has been successully replicated from
+    a storage to another.
+    S -> M
+    """
+    _fmt = PStruct('notify_replication_done',
+        PNumber('offset'),
+        PTID('tid'),
+    )
+
+class Truncate(Packet):
+    """
+    XXX: Used for both make storage consistent and leave backup mode
+    M -> S
+    """
+    _fmt = PStruct('truncate',
+        PTID('tid'),
+    )
+
+
 StaticRegistry = {}
-def register(code, request, ignore_when_closed=None):
+def register(request, ignore_when_closed=None):
     """ Register a packet in the packet registry """
+    code = len(StaticRegistry)
+    if request is Error:
+        code |= RESPONSE_MASK
     # register the request
-    assert code not in StaticRegistry, "Duplicate request packet code"
-    request._code = code
     StaticRegistry[code] = request
+    if request is None:
+        return # None registered only to skip a code number (for compatibility)
+    request._code = code
     answer = request._answer
     if ignore_when_closed is None:
         # By default, on a closed connection:
@@ -1426,120 +1596,142 @@ class Packets(dict):
 
     # notifications
     Error = register(
-            0x8000, Error)
-    Ping, Pong = register(
-            0x0001, Ping)
-    Notify = register(
-            0x0002, Notify)
+                    Error)
     RequestIdentification, AcceptIdentification = register(
-            0x0003, RequestIdentification)
+                    RequestIdentification)
+    # Code of RequestIdentification packet must never change so that 2
+    # incompatible nodes can reject themselves gracefully (i.e. comparing
+    # protocol versions) instead of raising PacketMalformedError.
+    assert RequestIdentification._code == 1
+
+    Ping, Pong = register(
+                    Ping)
+    CloseClient  = register(
+                    CloseClient)
+    Notify = register(
+                    Notify)
     AskPrimary, AnswerPrimary = register(
-            0x0004, PrimaryMaster)
+                    PrimaryMaster)
     AnnouncePrimary = register(
-            0x0005, AnnouncePrimary)
+                    AnnouncePrimary)
     ReelectPrimary = register(
-            0x0006, ReelectPrimary)
+                    ReelectPrimary)
     NotifyNodeInformation = register(
-            0x0007, NotifyNodeInformation)
+                    NotifyNodeInformation)
     AskLastIDs, AnswerLastIDs = register(
-            0x0008, LastIDs)
+                    LastIDs)
     AskPartitionTable, AnswerPartitionTable = register(
-            0x0009, PartitionTable)
+                    PartitionTable)
     SendPartitionTable = register(
-            0x000A, NotifyPartitionTable)
+                    NotifyPartitionTable)
     NotifyPartitionChanges = register(
-            0x000B, PartitionChanges)
+                    PartitionChanges)
     StartOperation = register(
-            0x000C, StartOperation)
+                    StartOperation)
     StopOperation = register(
-            0x000D, StopOperation)
+                    StopOperation)
     AskUnfinishedTransactions, AnswerUnfinishedTransactions = register(
-            0x000E, UnfinishedTransactions)
+                    UnfinishedTransactions)
     AskObjectPresent, AnswerObjectPresent = register(
-            0x000F, ObjectPresent)
+                    ObjectPresent)
     DeleteTransaction = register(
-            0x0010, DeleteTransaction)
+                    DeleteTransaction)
     CommitTransaction = register(
-            0x0011, CommitTransaction)
+                    CommitTransaction)
     AskBeginTransaction, AnswerBeginTransaction = register(
-            0x0012, BeginTransaction)
+                    BeginTransaction)
     AskFinishTransaction, AnswerTransactionFinished = register(
-            0x0013, FinishTransaction, ignore_when_closed=False)
+                    FinishTransaction, ignore_when_closed=False)
     AskLockInformation, AnswerInformationLocked = register(
-            0x0014, LockInformation, ignore_when_closed=False)
+                    LockInformation, ignore_when_closed=False)
     InvalidateObjects = register(
-            0x0015, InvalidateObjects)
+                    InvalidateObjects)
     NotifyUnlockInformation = register(
-            0x0016, UnlockInformation)
+                    UnlockInformation)
     AskNewOIDs, AnswerNewOIDs = register(
-            0x0017, GenerateOIDs)
+                    GenerateOIDs)
     AskStoreObject, AnswerStoreObject = register(
-            0x0018, StoreObject)
+                    StoreObject)
     AbortTransaction = register(
-            0x0019, AbortTransaction)
+                    AbortTransaction)
     AskStoreTransaction, AnswerStoreTransaction = register(
-            0x001A, StoreTransaction)
+                    StoreTransaction)
     AskObject, AnswerObject = register(
-            0x001B, GetObject)
+                    GetObject)
     AskTIDs, AnswerTIDs = register(
-            0x001C, TIDList)
+                    TIDList)
     AskTransactionInformation, AnswerTransactionInformation = register(
-            0x001D, TransactionInformation)
+                    TransactionInformation)
     AskObjectHistory, AnswerObjectHistory = register(
-            0x001E, ObjectHistory)
+                    ObjectHistory)
     AskPartitionList, AnswerPartitionList = register(
-            0x001F, PartitionList)
+                    PartitionList)
     AskNodeList, AnswerNodeList = register(
-            0x0020, NodeList)
+                    NodeList)
     SetNodeState = register(
-            0x0021, SetNodeState, ignore_when_closed=False)
+                    SetNodeState, ignore_when_closed=False)
     AddPendingNodes = register(
-            0x0022, AddPendingNodes, ignore_when_closed=False)
+                    AddPendingNodes, ignore_when_closed=False)
+    TweakPartitionTable = register(
+                    TweakPartitionTable, ignore_when_closed=False)
     AskNodeInformation, AnswerNodeInformation = register(
-            0x0023, NodeInformation)
+                    NodeInformation)
     SetClusterState = register(
-            0x0024, SetClusterState, ignore_when_closed=False)
+                    SetClusterState, ignore_when_closed=False)
     NotifyClusterInformation = register(
-            0x0025, ClusterInformation)
+                    ClusterInformation)
     AskClusterState, AnswerClusterState = register(
-            0x0026, ClusterState)
-    NotifyLastOID = register(
-            0x0027, NotifyLastOID)
-    NotifyReplicationDone = register(
-            0x0028, ReplicationDone)
+                    ClusterState)
     AskObjectUndoSerial, AnswerObjectUndoSerial = register(
-            0x0029, ObjectUndoSerial)
+                    ObjectUndoSerial)
     AskHasLock, AnswerHasLock = register(
-            0x002A, HasLock)
+                    HasLock)
     AskTIDsFrom, AnswerTIDsFrom = register(
-            0x002B, TIDListFrom)
-    AskObjectHistoryFrom, AnswerObjectHistoryFrom = register(
-            0x002C, ObjectHistoryFrom)
-    # 2D
+                    TIDListFrom)
     AskPack, AnswerPack = register(
-            0x002E, Pack, ignore_when_closed=False)
+                    Pack, ignore_when_closed=False)
+    CheckReplicas = register(
+                    CheckReplicas)
+    CheckPartition = register(
+                    CheckPartition)
     AskCheckTIDRange, AnswerCheckTIDRange = register(
-            0x002F, CheckTIDRange)
+                    CheckTIDRange)
     AskCheckSerialRange, AnswerCheckSerialRange = register(
-            0x0030, CheckSerialRange)
+                    CheckSerialRange)
+    NotifyPartitionCorrupted = register(
+                    PartitionCorrupted)
     NotifyReady = register(
-            0x0031, NotifyReady)
+                    NotifyReady)
     AskLastTransaction, AnswerLastTransaction = register(
-            0x0032, LastTransaction)
+                    LastTransaction)
     AskCheckCurrentSerial, AnswerCheckCurrentSerial = register(
-            0x0033, CheckCurrentSerial)
+                    CheckCurrentSerial)
     NotifyTransactionFinished = register(
-            0x003E, NotifyTransactionFinished)
+                    NotifyTransactionFinished)
+    Replicate = register(
+                    Replicate)
+    NotifyReplicationDone = register(
+                    ReplicationDone)
+    AskFetchTransactions, AnswerFetchTransactions = register(
+                    FetchTransactions)
+    AskFetchObjects, AnswerFetchObjects = register(
+                    FetchObjects)
+    AddTransaction = register(
+                    AddTransaction)
+    AddObject = register(
+                    AddObject)
+    Truncate = register(
+                    Truncate)
 
 def Errors():
     registry_dict = {}
     handler_method_name_dict = {}
     def register_error(code):
         return lambda self, message='': Error(code, message)
-    for code, error in ErrorCodes.iteritems():
+    for error in ErrorCodes:
         name = ''.join(part.capitalize() for part in str(error).split('_'))
-        registry_dict[name] = register_error(error)
-        handler_method_name_dict[code] = name[0].lower() + name[1:]
+        registry_dict[name] = register_error(int(error))
+        handler_method_name_dict[int(error)] = name[0].lower() + name[1:]
     return type('ErrorRegistry', (dict,),
                 registry_dict)(handler_method_name_dict)
 
