@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2010  Nexedi SA
+# Copyright (C) 2006-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,21 +12,91 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
-import re
 import socket
+from binascii import a2b_hex, b2a_hex
+from datetime import timedelta, datetime
 from hashlib import sha1
 from Queue import deque
 from struct import pack, unpack
+from time import gmtime
 
 SOCKET_CONNECTORS_DICT = {
     socket.AF_INET : 'SocketConnectorIPv4',
     socket.AF_INET6: 'SocketConnectorIPv6',
 }
 
+TID_LOW_OVERFLOW = 2**32
+TID_LOW_MAX = TID_LOW_OVERFLOW - 1
+SECOND_PER_TID_LOW = 60.0 / TID_LOW_OVERFLOW
+TID_CHUNK_RULES = (
+    (-1900, 0),
+    (-1, 12),
+    (-1, 31),
+    (0, 24),
+    (0, 60),
+)
+
+def tidFromTime(tm):
+    gmt = gmtime(tm)
+    return packTID(
+        (gmt.tm_year, gmt.tm_mon, gmt.tm_mday, gmt.tm_hour, gmt.tm_min),
+        int((gmt.tm_sec + (tm - int(tm))) / SECOND_PER_TID_LOW))
+
+def packTID(higher, lower):
+    """
+    higher: a 5-tuple containing year, month, day, hour and minute
+    lower: seconds scaled to 60:2**32 into a 64 bits TID
+    """
+    assert len(higher) == len(TID_CHUNK_RULES), higher
+    packed_higher = 0
+    for value, (offset, multiplicator) in zip(higher, TID_CHUNK_RULES):
+        assert isinstance(value, (int, long)), value
+        value += offset
+        assert 0 <= value, (value, offset, multiplicator)
+        assert multiplicator == 0 or value < multiplicator, (value,
+            offset, multiplicator)
+        packed_higher *= multiplicator
+        packed_higher += value
+    # If the machine is configured in such way that gmtime() returns leap
+    # seconds (e.g. TZ=right/UTC), then the best we can do is to use
+    # TID_LOW_MAX, because TID format was not designed to support them.
+    # For more information about leap seconds on Unix, see:
+    #   http://en.wikipedia.org/wiki/Unix_time
+    #   http://www.madore.org/~david/computers/unix-leap-seconds.html
+    return pack('!LL', packed_higher, min(lower, TID_LOW_MAX))
+
+def unpackTID(ptid):
+    """
+    Unpack given 64 bits TID in to a 2-tuple containing:
+    - a 5-tuple containing year, month, day, hour and minute
+    - seconds scaled to 60:2**32
+    """
+    packed_higher, lower = unpack('!LL', ptid)
+    higher = []
+    append = higher.append
+    for offset, multiplicator in reversed(TID_CHUNK_RULES):
+        if multiplicator:
+            packed_higher, value = divmod(packed_higher, multiplicator)
+        else:
+            packed_higher, value = 0, packed_higher
+        append(value - offset)
+    higher.reverse()
+    return (tuple(higher), lower)
+
+def addTID(ptid, offset):
+    """
+    Offset given packed TID.
+    """
+    higher, lower = unpackTID(ptid)
+    high_offset, lower = divmod(lower + offset, TID_LOW_OVERFLOW)
+    if high_offset:
+        d = datetime(*higher) + timedelta(0, 60 * high_offset)
+        higher = (d.year, d.month, d.day, d.hour, d.minute)
+    return packTID(higher, lower)
+
 def u64(s):
     return unpack('!Q', s)[0]
 
@@ -39,26 +109,15 @@ def add64(packed, offset):
 
 def dump(s):
     """Dump a binary string in hex."""
-    if s is None:
-        return None
-    if isinstance(s, str):
-        ret = []
-        for c in s:
-            ret.append('%02x' % ord(c))
-        return ''.join(ret)
-    else:
+    if s is not None:
+        if isinstance(s, str):
+            return b2a_hex(s)
         return repr(s)
 
-
 def bin(s):
     """Inverse of dump method."""
-    if s is None:
-        return None
-    ret = []
-    while len(s):
-        ret.append(chr(int(s[:2], 16)))
-        s = s[2:]
-    return ''.join(ret)
+    if s is not None:
+        return a2b_hex(s)
 
 
 def makeChecksum(s):
@@ -66,22 +125,11 @@ def makeChecksum(s):
     return sha1(s).digest()
 
 
-def resolve(hostname):
-    """
-        Returns the first IP address that match with the given hostname
-    """
-    try:
-        # an IP resolves to itself
-        _, _, address_list = socket.gethostbyname_ex(hostname)
-    except socket.gaierror:
-        return None
-    return address_list[0]
-
 def getAddressType(address):
     "Return the type (IPv4 or IPv6) of an ip"
     (host, port) = address
 
-    for af_type in SOCKET_CONNECTORS_DICT.keys():
+    for af_type in SOCKET_CONNECTORS_DICT:
         try :
             socket.inet_pton(af_type, host)
         except:
@@ -97,92 +145,43 @@ def getConnectorFromAddress(address):
     return SOCKET_CONNECTORS_DICT[address_type]
 
 def parseNodeAddress(address, port_opt=None):
-    if ']' in address:
-       (ip, port) = address.split(']')
-       ip = ip.lstrip('[')
-       port = port.lstrip(':')
-       if port == '':
-           port = port_opt
-    elif ':' in address:
-        (ip, port) = address.split(':')
-        ip = resolve(ip)
+    if address[:1] == '[':
+        (host, port) = address[1:].split(']')
+        if port[:1] == ':':
+            port = port[1:]
+        else:
+            port = port_opt
+    elif address.count(':') == 1:
+        (host, port) = address.split(':')
     else:
-        ip = address
+        host = address
         port = port_opt
-
-    if port is None:
-        raise ValueError
-    return (ip, int(port))
+    # Resolve (maybe) and cast to cannonical form
+    # XXX: Always pick the first result. This might not be what is desired, and
+    # if so this function should either take a hint on the desired address type
+    # or return either raw host & port or getaddrinfo return value.
+    return socket.getaddrinfo(host, port, 0, socket.SOCK_STREAM)[0][4][:2]
 
 def parseMasterList(masters, except_node=None):
     assert masters, 'At least one master must be defined'
-    socket_connector = ''
     # load master node list
+    socket_connector = None
     master_node_list = []
-    # XXX: support '/' and ' ' as separator
-    masters = masters.replace('/', ' ')
     for node in masters.split(' '):
+        if not node:
+            continue
         address = parseNodeAddress(node)
 
         if (address != except_node):
             master_node_list.append(address)
 
         socket_connector_temp = getConnectorFromAddress(address)
-        if socket_connector == '':
+        if socket_connector is None:
             socket_connector = socket_connector_temp
-        elif socket_connector == socket_connector_temp:
-           pass
-        else:
-            return TypeError, (" Wrong connector type : you're trying to use ipv6 and ipv4 simultaneously")
-
-    return tuple(master_node_list), socket_connector
-
-class Enum(dict):
-    """
-    Simulate an enumeration, define them as follow :
-        class MyEnum(Enum):
-          ITEM1 = Enum.Item(0)
-          ITEM2 = Enum.Item(1)
-    Enum items must be written in full upper case
-    """
-
-    class Item(int):
-
-        _enum = None
-        _name = None
-
-        def __new__(cls, value):
-            instance = super(Enum.Item, cls).__new__(cls, value)
-            instance._enum = None
-            instance._name = None
-            return instance
-
-        def __str__(self):
-            return self._name
-
-        def __repr__(self):
-            return "<EnumItem %s (%d)>" % (self._name, self)
-
-        def __eq__(self, other):
-            if other is None:
-                return False
-            assert isinstance(other, (Enum.Item, int, float, long))
-            if isinstance(other, Enum):
-                assert self._enum == other._enum
-            return int(self) == int(other)
-
-    def __init__(self):
-        dict.__init__(self)
-        for name in dir(self):
-            if not re.match('^[A-Z_]*$', name):
-                continue
-            item = getattr(self, name)
-            item._name = name
-            item._enum = self
-            self[int(item)] = item
-
-    def getByName(self, name):
-        return getattr(self, name)
+        elif socket_connector != socket_connector_temp:
+            raise TypeError("Wrong connector type : you're trying to use "
+                "ipv6 and ipv4 simultaneously")
+    return master_node_list, socket_connector
 
 
 class ReadBuffer(object):
@@ -215,7 +214,6 @@ class ReadBuffer(object):
         pop_chunk = self.content.popleft
         append_data = chunk_list.append
         to_read = size
-        chunk_len = 0
         # select required chunks
         while to_read > 0:
             chunk_size, chunk_data = pop_chunk()
