@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2010  Nexedi SA
+# Copyright (C) 2006-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,42 +12,46 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import neo
-import os, sys
+import sys, weakref
 from time import time
 
-from neo.lib import protocol
-from neo.lib.protocol import UUID_NAMESPACES, ZERO_TID
+from neo.lib import logging
+from neo.lib.connector import getConnectorHandler
+from neo.lib.debug import register as registerLiveDebugger
+from neo.lib.protocol import uuid_str, UUID_NAMESPACES, ZERO_TID
 from neo.lib.protocol import ClusterStates, NodeStates, NodeTypes, Packets
 from neo.lib.node import NodeManager
 from neo.lib.event import EventManager
+from neo.lib.handler import EventHandler
 from neo.lib.connection import ListeningConnection, ClientConnection
 from neo.lib.exception import ElectionFailure, PrimaryFailure, OperationFailure
-from neo.master.handlers import election, identification, secondary
-from neo.master.handlers import storage, client, shutdown
-from neo.master.handlers import administration
-from neo.master.pt import PartitionTable
-from neo.master.transactions import TransactionManager
-from neo.master.verification import VerificationManager
-from neo.master.recovery import RecoveryManager
-from neo.lib.util import dump
-from neo.lib.connector import getConnectorHandler
 
-from neo.lib.debug import register as registerLiveDebugger
+class StateChangedException(Exception): pass
+
+from .backup_app import BackupApplication
+from .handlers import election, identification, secondary
+from .handlers import administration, client, storage
+from .pt import PartitionTable
+from .recovery import RecoveryManager
+from .transactions import TransactionManager
+from .verification import VerificationManager
+
 
 class Application(object):
     """The master node application."""
     packing = None
     # Latest completely commited TID
     last_transaction = ZERO_TID
+    backup_tid = None
+    backup_app = None
+    uuid = None
 
     def __init__(self, config):
         # Internal attributes.
         self.em = EventManager()
-        self.nm = NodeManager()
+        self.nm = NodeManager(config.getDynamicMasterList())
         self.tm = TransactionManager(self.onTransactionCommitted)
 
         self.name = config.getCluster()
@@ -56,10 +60,10 @@ class Application(object):
         self.storage_readiness = set()
         master_addresses, connector_name = config.getMasters()
         self.connector_handler = getConnectorHandler(connector_name)
-        for master_address in master_addresses :
+        for master_address in master_addresses:
             self.nm.createMaster(address=master_address)
 
-        neo.lib.logging.debug('IP address is %s, port is %d', *(self.server))
+        logging.debug('IP address is %s, port is %d', *self.server)
 
         # Partition table
         replicas, partitions = config.getReplicas(), config.getPartitions()
@@ -68,10 +72,10 @@ class Application(object):
         if partitions <= 0:
             raise RuntimeError, 'partitions must be more than zero'
         self.pt = PartitionTable(partitions, replicas)
-        neo.lib.logging.info('Configuration:')
-        neo.lib.logging.info('Partitions: %d', partitions)
-        neo.lib.logging.info('Replicas  : %d', replicas)
-        neo.lib.logging.info('Name      : %s', self.name)
+        logging.info('Configuration:')
+        logging.info('Partitions: %d', partitions)
+        logging.info('Replicas  : %d', replicas)
+        logging.info('Name      : %s', self.name)
 
         self.listening_conn = None
         self.primary = None
@@ -79,29 +83,46 @@ class Application(object):
         self.cluster_state = None
         self._startup_allowed = False
 
-        # Generate an UUID for self
         uuid = config.getUUID()
-        if uuid is None or uuid == '':
-            uuid = self.getNewUUID(NodeTypes.MASTER)
-        self.uuid = uuid
-        neo.lib.logging.info('UUID      : %s', dump(uuid))
+        if uuid:
+            self.uuid = uuid
 
         # election related data
         self.unconnected_master_node_set = set()
         self.negotiating_master_node_set = set()
+        self.master_address_dict = weakref.WeakKeyDictionary()
 
         self._current_manager = None
 
+        # backup
+        upstream_cluster = config.getUpstreamCluster()
+        if upstream_cluster:
+            if upstream_cluster == self.name:
+                raise ValueError("upstream cluster name must be"
+                                 " different from cluster name")
+            self.backup_app = BackupApplication(self, upstream_cluster,
+                                                *config.getUpstreamMasters())
+
+        self.administration_handler = administration.AdministrationHandler(
+            self)
+        self.secondary_master_handler = secondary.SecondaryMasterHandler(self)
+        self.client_service_handler = client.ClientServiceHandler(self)
+        self.storage_service_handler = storage.StorageServiceHandler(self)
+
         registerLiveDebugger(on_log=self.log)
 
     def close(self):
         self.listening_conn = None
+        if self.backup_app is not None:
+            self.backup_app.close()
         self.nm.close()
         self.em.close()
         del self.__dict__
 
     def log(self):
         self.em.log()
+        if self.backup_app is not None:
+            self.backup_app.log()
         self.nm.log()
         self.tm.log()
         if self.pt is not None:
@@ -110,26 +131,23 @@ class Application(object):
     def run(self):
         try:
             self._run()
-        except:
-            neo.lib.logging.info('\nPre-mortem informations:')
+        except Exception:
+            logging.exception('Pre-mortem data:')
             self.log()
             raise
 
     def _run(self):
         """Make sure that the status is sane and start a loop."""
-        bootstrap = True
-
         # Make a listening port.
         self.listening_conn = ListeningConnection(self.em, None,
             addr=self.server, connector=self.connector_handler())
 
         # Start a normal operation.
-        while True:
+        while self.cluster_state != ClusterStates.STOPPING:
             # (Re)elect a new primary master.
             self.primary = not self.nm.getMasterList()
             if not self.primary:
-                self.electPrimary(bootstrap=bootstrap)
-                bootstrap = False
+                self.electPrimary()
             try:
                 if self.primary:
                     self.playPrimaryRole()
@@ -142,18 +160,21 @@ class Application(object):
                     conn.close()
 
 
-    def electPrimary(self, bootstrap = True):
+    def electPrimary(self):
         """Elect a primary master node.
 
         The difficulty is that a master node must accept connections from
         others while attempting to connect to other master nodes at the
         same time. Note that storage nodes and client nodes may connect
         to self as well as master nodes."""
-        neo.lib.logging.info('begin the election of a primary master')
+        logging.info('begin the election of a primary master')
 
+        client_handler = election.ClientElectionHandler(self)
         self.unconnected_master_node_set.clear()
         self.negotiating_master_node_set.clear()
+        self.master_address_dict.clear()
         self.listening_conn.setHandler(election.ServerElectionHandler(self))
+        getByAddress = self.nm.getByAddress
 
         while True:
 
@@ -166,103 +187,44 @@ class Application(object):
             self.primary = None
             self.primary_master_node = None
             try:
-                self._doElection(bootstrap)
+                while (self.unconnected_master_node_set or
+                        self.negotiating_master_node_set):
+                    for addr in self.unconnected_master_node_set:
+                        ClientConnection(self.em, client_handler,
+                            # XXX: Ugly, but the whole election code will be
+                            # replaced soon
+                            node=getByAddress(addr),
+                            connector=self.connector_handler())
+                        self.negotiating_master_node_set.add(addr)
+                    self.unconnected_master_node_set.clear()
+                    self.em.poll(1)
             except ElectionFailure, m:
                 # something goes wrong, clean then restart
-                self._electionFailed(m)
-                bootstrap = False
+                logging.error('election failed: %s', m)
+
+                # Ask all connected nodes to reelect a single primary master.
+                for conn in self.em.getClientList():
+                    conn.notify(Packets.ReelectPrimary())
+                    conn.abort()
+
+                # Wait until the connections are closed.
+                self.primary = None
+                self.primary_master_node = None
+                t = time() + 10
+                while self.em.getClientList() and time() < t:
+                    try:
+                        self.em.poll(1)
+                    except ElectionFailure:
+                        pass
+
+                # Close all connections.
+                for conn in self.em.getClientList() + self.em.getServerList():
+                    conn.close()
             else:
                 # election succeed, stop the process
                 self.primary = self.primary is None
                 break
 
-
-    def _doElection(self, bootstrap):
-        """
-            Start the election process:
-                - Try to connect to any known master node
-                - Wait at most for the timeout defined by bootstrap parameter
-            When done, the current process is defined either as primary or
-            secondary master node
-        """
-        # Wait at most 20 seconds at bootstrap. Otherwise, wait at most
-        # 10 seconds to avoid stopping the whole cluster for a long time.
-        # Note that even if not all master are up in the first 20 seconds
-        # this is not an issue because the first up will timeout and take
-        # the primary role.
-        if bootstrap:
-            expiration = 20
-        else:
-            expiration = 10
-        client_handler = election.ClientElectionHandler(self)
-        t = 0
-        while True:
-            current_time = time()
-            if current_time >= t:
-                t = current_time + 1
-                for node in self.nm.getMasterList():
-                    if not node.isRunning() and node.getLastStateChange() + \
-                            expiration < current_time:
-                        neo.lib.logging.info('%s is down' % (node, ))
-                        node.setDown()
-                        self.unconnected_master_node_set.discard(
-                                node.getAddress())
-
-                # Try to connect to master nodes.
-                for addr in self.unconnected_master_node_set.difference(
-                              x.getAddress() for x in self.em.getClientList()):
-                    ClientConnection(self.em, client_handler, addr=addr,
-                                     connector=self.connector_handler())
-            self.em.poll(1)
-
-            if not (self.unconnected_master_node_set or
-                    self.negotiating_master_node_set):
-                break
-
-    def _announcePrimary(self):
-        """
-            Broadcast the announce that I'm the primary
-        """
-        # I am the primary.
-        neo.lib.logging.debug('I am the primary, sending an announcement')
-        for conn in self.em.getClientList():
-            conn.notify(Packets.AnnouncePrimary())
-            conn.abort()
-        t = time()
-        while self.em.getClientList():
-            self.em.poll(1)
-            if t + 10 < time():
-                for conn in self.em.getClientList():
-                    conn.close()
-                break
-
-
-    def _electionFailed(self, m):
-        """
-            Ask other masters to reelect a primary after an election failure.
-        """
-        neo.lib.logging.error('election failed: %s', (m, ))
-
-        # Ask all connected nodes to reelect a single primary master.
-        for conn in self.em.getClientList():
-            conn.notify(Packets.ReelectPrimary())
-            conn.abort()
-
-        # Wait until the connections are closed.
-        self.primary = None
-        self.primary_master_node = None
-        t = time() + 10
-        while self.em.getClientList() and time() < t:
-            try:
-                self.em.poll(1)
-            except ElectionFailure:
-                pass
-
-        # Close all connections.
-        for conn in self.em.getClientList() + self.em.getServerList():
-            conn.close()
-
-
     def broadcastNodesInformation(self, node_list):
         """
           Broadcast changes for a set a nodes
@@ -284,37 +246,21 @@ class Application(object):
 
         # send at most one non-empty notification packet per node
         for node in self.nm.getIdentifiedList():
-            node_list = node_dict.get(node.getType(), [])
+            node_list = node_dict.get(node.getType())
             if node_list and node.isRunning():
                 node.notify(Packets.NotifyNodeInformation(node_list))
 
-    def broadcastPartitionChanges(self, cell_list, selector=None):
+    def broadcastPartitionChanges(self, cell_list):
         """Broadcast a Notify Partition Changes packet."""
-        neo.lib.logging.debug('broadcastPartitionChanges')
-        if not cell_list:
-            return
-        if not selector:
-            selector = lambda n: n.isClient() or n.isStorage() or n.isAdmin()
-        self.pt.log()
-        ptid = self.pt.setNextID()
-        packet = Packets.NotifyPartitionChanges(ptid, cell_list)
-        for node in self.nm.getIdentifiedList():
-            if not node.isRunning():
-                continue
-            if selector(node):
-                node.notify(packet)
-
-    def outdateAndBroadcastPartition(self):
-        " Outdate cell of non-working nodes and broadcast changes """
-        self.broadcastPartitionChanges(self.pt.outdate())
-
-    def broadcastLastOID(self):
-        oid = self.tm.getLastOID()
-        neo.lib.logging.debug(
-                        'Broadcast last OID to storages : %s' % dump(oid))
-        packet = Packets.NotifyLastOID(oid)
-        for node in self.nm.getStorageList(only_identified=True):
-            node.notify(packet)
+        logging.debug('broadcastPartitionChanges')
+        if cell_list:
+            self.pt.log()
+            ptid = self.pt.setNextID()
+            packet = Packets.NotifyPartitionChanges(ptid, cell_list)
+            for node in self.nm.getIdentifiedList():
+                # TODO: notify masters
+                if node.isRunning() and not node.isMaster():
+                    node.notify(packet)
 
     def provideService(self):
         """
@@ -322,96 +268,124 @@ class Application(object):
         and stop the service only if a catastrophy happens or the user commits
         a shutdown.
         """
-        neo.lib.logging.info('provide service')
-        em = self.em
+        logging.info('provide service')
+        poll = self.em.poll
         self.tm.reset()
 
         self.changeClusterState(ClusterStates.RUNNING)
 
         # Now everything is passive.
-        while True:
-            try:
-                em.poll(1)
-            except OperationFailure:
-                # If not operational, send Stop Operation packets to storage
-                # nodes and client nodes. Abort connections to client nodes.
-                neo.lib.logging.critical('No longer operational')
+        try:
+            while True:
+                poll(1)
+        except StateChangedException, e:
+            if e.args[0] != ClusterStates.STARTING_BACKUP:
+                raise
+            self.backup_tid = tid = self.getLastTransaction()
+            self.pt.setBackupTidDict(dict((node.getUUID(), tid)
+                for node in self.nm.getStorageList(only_identified=True)))
+
+    def playPrimaryRole(self):
+        logging.info('play the primary role with %r', self.listening_conn)
+        self.master_address_dict.clear()
+        em = self.em
+        packet = Packets.AnnouncePrimary()
+        for conn in em.getConnectionList():
+            if conn.isListening():
+                conn.setHandler(identification.IdentificationHandler(self))
+            else:
+                conn.notify(packet)
+                # Primary master should rather establish connections to all
+                # secondaries, rather than the other way around. This requires
+                # a bit more work when a new master joins a cluster but makes
+                # it easier to resolve UUID conflicts with minimal cluster
+                # impact, and ensure primary master unicity (primary masters
+                # become noisy, in that they actively try to maintain
+                # connections to all other master nodes, so duplicate
+                # primaries will eventually get in touch with each other and
+                # resolve the situation with a duel).
+                # TODO: only abort client connections, don't close server
+                # connections as we want to have them in the end. Secondary
+                # masters will reconnect nevertheless, but it's dirty.
+                # Currently, it's not trivial to preserve connected nodes,
+                # because of poor node status tracking during election.
+                conn.abort()
+
+        # If I know any storage node, make sure that they are not in the
+        # running state, because they are not connected at this stage.
+        for node in self.nm.getStorageList():
+            if node.isRunning():
+                node.setTemporarilyDown()
+
+        if self.uuid is None:
+            self.uuid = self.getNewUUID(None, self.server, NodeTypes.MASTER)
+            logging.info('My UUID: ' + uuid_str(self.uuid))
+        else:
+            in_conflict = self.nm.getByUUID(self.uuid)
+            if in_conflict is not None:
+                logging.warning('UUID conflict at election exit with %r',
+                    in_conflict)
+                in_conflict.setUUID(None)
+
+        # recover the cluster status at startup
+        try:
+            self.runManager(RecoveryManager)
+            while True:
+                self.runManager(VerificationManager)
+                try:
+                    if self.backup_tid:
+                        if self.backup_app is None:
+                            raise RuntimeError("No upstream cluster to backup"
+                                               " defined in configuration")
+                        self.backup_app.provideService()
+                        # Reset connection with storages (and go through a
+                        # recovery phase) when leaving backup mode in order
+                        # to get correct last oid/tid.
+                        self.runManager(RecoveryManager)
+                        continue
+                    self.provideService()
+                except OperationFailure:
+                    logging.critical('No longer operational')
                 for node in self.nm.getIdentifiedList():
                     if node.isStorage() or node.isClient():
                         node.notify(Packets.StopOperation())
                         if node.isClient():
                             node.getConnection().abort()
-
-                # Then, go back, and restart.
-                return
-
-    def playPrimaryRole(self):
-        neo.lib.logging.info(
-                        'play the primary role with %r', self.listening_conn)
-
-        # i'm the primary, send the announcement
-        self._announcePrimary()
-        # all incoming connections identify through this handler
-        self.listening_conn.setHandler(
-                identification.IdentificationHandler(self))
-
-        em = self.em
-        nm = self.nm
-
-        # Close all remaining connections to other masters,
-        # for the same reason as in playSecondaryRole.
-        for conn in em.getConnectionList():
-            conn_uuid = conn.getUUID()
-            if conn_uuid is not None:
-                node = nm.getByUUID(conn_uuid)
-                assert node is not None
-                assert node.isMaster() and not conn.isClient()
-                assert node._connection is None and node.isUnknown()
-                # this may trigger 'unexpected answer' warnings on remote side
-                conn.close()
-
-        # If I know any storage node, make sure that they are not in the
-        # running state, because they are not connected at this stage.
-        for node in nm.getStorageList():
-            if node.isRunning():
-                node.setTemporarilyDown()
-
-        # recover the cluster status at startup
-        self.runManager(RecoveryManager)
-        while True:
-            self.runManager(VerificationManager)
-            self.provideService()
+        except StateChangedException, e:
+            assert e.args[0] == ClusterStates.STOPPING
+            self.shutdown()
 
     def playSecondaryRole(self):
         """
         I play a secondary role, thus only wait for a primary master to fail.
         """
-        neo.lib.logging.info('play the secondary role with %r',
-            self.listening_conn)
+        logging.info('play the secondary role with %r', self.listening_conn)
 
         # Wait for an announcement. If this is too long, probably
         # the primary master is down.
-        t = time()
+        t = time() + 10
         while self.primary_master_node is None:
             self.em.poll(1)
-            if t + 10 < time():
+            if t < time():
                 # election timeout
                 raise ElectionFailure("Election timeout")
+        self.master_address_dict.clear()
 
         # Restart completely. Non-optimized
         # but lower level code needs to be stabilized first.
-        addr = self.primary_master_node.getAddress()
         for conn in self.em.getConnectionList():
-            conn.close()
+            if not conn.isListening():
+                conn.close()
 
         # Reconnect to primary master node.
         primary_handler = secondary.PrimaryHandler(self)
-        ClientConnection(self.em, primary_handler, addr=addr,
+        ClientConnection(self.em, primary_handler,
+            node=self.primary_master_node,
             connector=self.connector_handler())
 
         # and another for the future incoming connections
-        handler = identification.IdentificationHandler(self)
-        self.listening_conn.setHandler(handler)
+        self.listening_conn.setHandler(
+            identification.SecondaryIdentificationHandler(self))
 
         while True:
             self.em.poll(1)
@@ -429,136 +403,98 @@ class Application(object):
             return
 
         # select the storage handler
-        client_handler = client.ClientServiceHandler(self)
-        if state == ClusterStates.RUNNING:
-            storage_handler = storage.StorageServiceHandler(self)
+        client_handler = self.client_service_handler
+        if state in (ClusterStates.RUNNING, ClusterStates.STARTING_BACKUP,
+                     ClusterStates.BACKINGUP, ClusterStates.STOPPING_BACKUP):
+            storage_handler = self.storage_service_handler
         elif self._current_manager is not None:
             storage_handler = self._current_manager.getHandler()
+        elif state == ClusterStates.STOPPING:
+            storage_handler = None
         else:
             raise RuntimeError('Unexpected cluster state')
 
         # change handlers
         notification_packet = Packets.NotifyClusterInformation(state)
         for node in self.nm.getIdentifiedList():
-            if node.isMaster():
-                continue
             conn = node.getConnection()
-            if node.isClient() and conn.isAborted():
-                continue
-            node.notify(notification_packet)
+            conn.notify(notification_packet)
             if node.isClient():
                 if state != ClusterStates.RUNNING:
-                    conn.close()
+                    conn.abort()
                     continue
                 handler = client_handler
-            elif node.isStorage():
+            elif node.isStorage() and storage_handler:
                 handler = storage_handler
             else:
                 continue # keep handler
-            conn.setHandler(handler)
-            handler.connectionCompleted(conn)
+            if type(handler) is not type(conn.getLastHandler()):
+                conn.setHandler(handler)
+                handler.connectionCompleted(conn)
         self.cluster_state = state
 
-    def getNewUUID(self, node_type):
-        # build an UUID
-        uuid = os.urandom(15)
-        while uuid == protocol.INVALID_UUID[1:]:
-            uuid = os.urandom(15)
-        # look for the prefix
-        prefix = UUID_NAMESPACES.get(node_type, None)
-        if prefix is None:
-            raise RuntimeError, 'No UUID namespace found for this node type'
-        return prefix + uuid
-
-    def isValidUUID(self, uuid, addr):
-        node = self.nm.getByUUID(uuid)
-        if node is not None and node.getAddress() is not None \
-                and node.getAddress() != addr:
-            return False
-        return uuid != self.uuid and uuid is not None
+    def getNewUUID(self, uuid, address, node_type):
+        getByUUID = self.nm.getByUUID
+        if None != uuid != self.uuid:
+            node = getByUUID(uuid)
+            if node is None or node.getAddress() == address:
+                return uuid
+        hob = UUID_NAMESPACES[node_type]
+        for uuid in xrange((hob << 24) + 1, hob + 0x10 << 24):
+            if uuid != self.uuid and getByUUID(uuid) is None:
+                return uuid
+        raise RuntimeError
 
     def getClusterState(self):
         return self.cluster_state
 
     def shutdown(self):
         """Close all connections and exit"""
-        # XXX: This behaviour is probably broken, as it applies the same
-        #   handler to all connection types. It must be carefuly reviewed and
-        #   corrected.
-        # change handler
-        handler = shutdown.ShutdownHandler(self)
-        for node in self.nm.getIdentifiedList():
-            node.getConnection().setHandler(handler)
+        self.changeClusterState(ClusterStates.STOPPING)
+        self.listening_conn.close()
+        for conn in self.em.getConnectionList():
+            node = self.nm.getByUUID(conn.getUUID())
+            if node is None or not node.isIdentified():
+                conn.close()
+        # No need to change handlers in order to reject RequestIdentification
+        # & AskBeginTransaction packets because they won't be any:
+        # the only remaining connected peers are identified non-clients
+        # and we don't accept new connections anymore.
+        try:
+            # wait for all transaction to be finished
+            while self.tm.hasPending():
+                self.em.poll(1)
+        except OperationFailure:
+            logging.critical('No longer operational')
 
-        # wait for all transaction to be finished
-        while self.tm.hasPending():
+        logging.info("asking remaining nodes to shutdown")
+        handler = EventHandler(self)
+        for node in self.nm.getConnectedList():
+            conn = node.getConnection()
+            if node.isStorage():
+                conn.setHandler(handler)
+                conn.notify(Packets.NotifyNodeInformation(((
+                  node.getType(), node.getAddress(), node.getUUID(),
+                  NodeStates.TEMPORARILY_DOWN),)))
+                conn.abort()
+            elif conn.pending():
+                conn.abort()
+            else:
+                conn.close()
+
+        while self.em.connection_dict:
             self.em.poll(1)
 
-        if self.cluster_state != ClusterStates.RUNNING:
-            neo.lib.logging.info("asking all nodes to shutdown")
-            # This code sends packets but never polls, so they never reach
-            # network.
-            for node in self.nm.getIdentifiedList():
-                notification = Packets.NotifyNodeInformation([node.asTuple()])
-                if node.isClient():
-                    node.notify(notification)
-                elif node.isStorage() or node.isMaster():
-                    node.notify(notification)
-
         # then shutdown
         sys.exit()
 
-    def identifyStorageNode(self, uuid, node):
-        state = NodeStates.RUNNING
-        handler = None
-        if self.cluster_state == ClusterStates.RUNNING:
-            if uuid is None or node is None:
-                # same as for verification
-                state = NodeStates.PENDING
-            handler = storage.StorageServiceHandler(self)
-        elif self.cluster_state == ClusterStates.STOPPING:
-            raise protocol.NotReadyError
+    def identifyStorageNode(self, known):
+        if known:
+            state = NodeStates.RUNNING
         else:
-            raise RuntimeError('unhandled cluster state: %s' %
-                    (self.cluster_state, ))
-        return (uuid, state, handler)
-
-    def identifyNode(self, node_type, uuid, node):
-
-        state = NodeStates.RUNNING
-
-        if node_type == NodeTypes.ADMIN:
-            # always accept admin nodes
-            node_ctor = self.nm.createAdmin
-            handler = administration.AdministrationHandler(self)
-            neo.lib.logging.info('Accept an admin %s' % (dump(uuid), ))
-        elif node_type == NodeTypes.MASTER:
-            if node is None:
-                # unknown master, rejected
-                raise protocol.ProtocolError('Reject an unknown master node')
-            # always put other master in waiting state
-            node_ctor = self.nm.createMaster
-            handler = secondary.SecondaryMasterHandler(self)
-            neo.lib.logging.info('Accept a master %s' % (dump(uuid), ))
-        elif node_type == NodeTypes.CLIENT:
-            # refuse any client before running
-            if self.cluster_state != ClusterStates.RUNNING:
-                neo.lib.logging.info('Reject a connection from a client')
-                raise protocol.NotReadyError
-            node_ctor = self.nm.createClient
-            handler = client.ClientServiceHandler(self)
-            neo.lib.logging.info('Accept a client %s' % (dump(uuid), ))
-        elif node_type == NodeTypes.STORAGE:
-            node_ctor = self.nm.createStorage
-            manager = self._current_manager
-            if manager is None:
-                manager = self
-            (uuid, state, handler) = manager.identifyStorageNode(uuid, node)
-            neo.lib.logging.info('Accept a storage %s (%s)' %
-                            (dump(uuid), state))
-        else:
-            handler = identification.IdentificationHandler(self)
-        return (uuid, node, state, handler, node_ctor)
+            # same as for verification
+            state = NodeStates.PENDING
+        return state, self.storage_service_handler
 
     def onTransactionCommitted(self, txn):
         # I have received all the lock answers now:
@@ -591,14 +527,13 @@ class Application(object):
 
         # remove transaction from manager
         self.tm.remove(transaction_node.getUUID(), ttid)
+        assert self.last_transaction < tid, (self.last_transaction, tid)
         self.setLastTransaction(tid)
 
     def getLastTransaction(self):
         return self.last_transaction
 
     def setLastTransaction(self, tid):
-        ltid = self.last_transaction
-        assert tid >= ltid, (tid, ltid)
         self.last_transaction = tid
 
     def setStorageNotReady(self, uuid):
