@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2010  Nexedi SA
+# Copyright (C) 2006-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,77 +12,13 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from time import time, gmtime
+from time import time
 from struct import pack, unpack
-from neo.lib.protocol import ZERO_TID
-from datetime import timedelta, datetime
-from neo.lib.util import dump, u64, p64
-import neo.lib
-
-TID_LOW_OVERFLOW = 2**32
-TID_LOW_MAX = TID_LOW_OVERFLOW - 1
-SECOND_PER_TID_LOW = 60.0 / TID_LOW_OVERFLOW
-TID_CHUNK_RULES = (
-    (-1900, 0),
-    (-1, 12),
-    (-1, 31),
-    (0, 24),
-    (0, 60),
-)
-
-def packTID(utid):
-    """
-    Pack given 2-tuple containing:
-    - a 5-tuple containing year, month, day, hour and minute
-    - seconds scaled to 60:2**32
-    into a 64 bits TID.
-    """
-    higher, lower = utid
-    assert len(higher) == len(TID_CHUNK_RULES), higher
-    packed_higher = 0
-    for value, (offset, multiplicator) in zip(higher, TID_CHUNK_RULES):
-        assert isinstance(value, (int, long)), value
-        value += offset
-        assert 0 <= value, (value, offset, multiplicator)
-        assert multiplicator == 0 or value < multiplicator, (value,
-            offset, multiplicator)
-        packed_higher *= multiplicator
-        packed_higher += value
-    assert isinstance(lower, (int, long)), lower
-    assert 0 <= lower < TID_LOW_OVERFLOW, hex(lower)
-    return pack('!LL', packed_higher, lower)
-
-def unpackTID(ptid):
-    """
-    Unpack given 64 bits TID in to a 2-tuple containing:
-    - a 5-tuple containing year, month, day, hour and minute
-    - seconds scaled to 60:2**32
-    """
-    packed_higher, lower = unpack('!LL', ptid)
-    higher = []
-    append = higher.append
-    for offset, multiplicator in reversed(TID_CHUNK_RULES):
-        if multiplicator:
-            packed_higher, value = divmod(packed_higher, multiplicator)
-        else:
-            packed_higher, value = 0, packed_higher
-        append(value - offset)
-    higher.reverse()
-    return (tuple(higher), lower)
-
-def addTID(ptid, offset):
-    """
-    Offset given packed TID.
-    """
-    higher, lower = unpackTID(ptid)
-    high_offset, lower = divmod(lower + offset, TID_LOW_OVERFLOW)
-    if high_offset:
-        d = datetime(*higher) + timedelta(0, 60 * high_offset)
-        higher = (d.year, d.month, d.day, d.hour, d.minute)
-    return packTID((higher, lower))
+from neo.lib import logging
+from neo.lib.protocol import uuid_str, ZERO_TID
+from neo.lib.util import dump, u64, addTID, tidFromTime
 
 class DelayedError(Exception):
     pass
@@ -114,8 +50,8 @@ class Transaction(object):
                 self.__class__.__name__,
                 self._node,
                 dump(self._tid),
-                [dump(x) for x in self._oid_list or ()],
-                [dump(x) for x in self._uuid_set or ()],
+                map(dump, self._oid_list or ()),
+                map(uuid_str, self._uuid_set or ()),
                 time() - self._birth,
                 id(self),
         )
@@ -220,7 +156,6 @@ class TransactionManager(object):
         Manage current transactions
     """
     _last_tid = ZERO_TID
-    _next_ttid = 0
 
     def __init__(self, on_commit):
         # ttid -> transaction
@@ -254,42 +189,24 @@ class TransactionManager(object):
         self._last_oid = oid_list[-1]
         return oid_list
 
-    def updateLastOID(self, oid_list):
-        """
-            Updates the last oid with the max of those supplied if greater than
-            the current known, returns True if changed
-        """
-        max_oid = oid_list and max(oid_list) or None # oid_list might be empty
-        if max_oid > self._last_oid:
-            self._last_oid = max_oid
-            return True
-        return False
-
     def setLastOID(self, oid):
-        self._last_oid = oid
+        self._last_oid = max(oid, self._last_oid)
 
     def getLastOID(self):
         return self._last_oid
 
-    def _nextTID(self, ttid, divisor):
+    def _nextTID(self, ttid=None, divisor=None):
         """
         Compute the next TID based on the current time and check collisions.
-        Also, adjust it so that
+        Also, if ttid is not None, divisor is mandatory adjust it so that
             tid % divisor == ttid % divisor
         while preserving
             min_tid < tid
+        If ttid is None, divisor is ignored.
         When constraints allow, prefer decreasing generated TID, to avoid
         fast-forwarding to future dates.
         """
-        assert isinstance(ttid, basestring), repr(ttid)
-        assert isinstance(divisor, (int, long)), repr(divisor)
-        tm = time()
-        gmt = gmtime(tm)
-        tid = packTID((
-            (gmt.tm_year, gmt.tm_mon, gmt.tm_mday, gmt.tm_hour,
-                gmt.tm_min),
-            int((gmt.tm_sec % 60 + (tm - int(tm))) / SECOND_PER_TID_LOW)
-        ))
+        tid = tidFromTime(time())
         min_tid = self._last_tid
         if tid <= min_tid:
             tid  = addTID(min_tid, 1)
@@ -297,21 +214,24 @@ class TransactionManager(object):
             try_decrease = False
         else:
             try_decrease = True
-        ref_remainder = u64(ttid) % divisor
-        remainder = u64(tid) % divisor
-        if ref_remainder != remainder:
-            if try_decrease:
-                new_tid = addTID(tid, ref_remainder - divisor - remainder)
-                assert u64(new_tid) % divisor == ref_remainder, (dump(new_tid),
-                    ref_remainder)
-                if new_tid <= min_tid:
-                    new_tid = addTID(new_tid, divisor)
-            else:
-                if ref_remainder > remainder:
-                    ref_remainder += divisor
-                new_tid = addTID(tid, ref_remainder - remainder)
-            assert min_tid < new_tid, (dump(min_tid), dump(tid), dump(new_tid))
-            tid = new_tid
+        if ttid is not None:
+            assert isinstance(ttid, basestring), repr(ttid)
+            assert isinstance(divisor, (int, long)), repr(divisor)
+            ref_remainder = u64(ttid) % divisor
+            remainder = u64(tid) % divisor
+            if ref_remainder != remainder:
+                if try_decrease:
+                    new_tid = addTID(tid, ref_remainder - divisor - remainder)
+                    assert u64(new_tid) % divisor == ref_remainder, (dump(new_tid),
+                        ref_remainder)
+                    if new_tid <= min_tid:
+                        new_tid = addTID(new_tid, divisor)
+                else:
+                    if ref_remainder > remainder:
+                        ref_remainder += divisor
+                    new_tid = addTID(tid, ref_remainder - remainder)
+                assert min_tid < new_tid, (dump(min_tid), dump(tid), dump(new_tid))
+                tid = new_tid
         self._last_tid = tid
         return self._last_tid
 
@@ -327,14 +247,6 @@ class TransactionManager(object):
         """
         self._last_tid = max(self._last_tid, tid)
 
-    def getTTID(self):
-        """
-            Generate a temporary TID, to be used only during a single node's
-            2PC.
-        """
-        self._next_ttid += 1
-        return p64(self._next_ttid)
-
     def reset(self):
         """
             Discard all manager content
@@ -357,7 +269,7 @@ class TransactionManager(object):
         # will be finished
         for txn in self._ttid_dict.itervalues():
             txn.registerForNotification(uuid)
-        return set(self._ttid_dict)
+        return self._ttid_dict.keys()
 
     def begin(self, node, tid=None):
         """
@@ -365,7 +277,7 @@ class TransactionManager(object):
         """
         if tid is None:
             # No TID requested, generate a temporary one
-            ttid = self.getTTID()
+            ttid = self._nextTID()
         else:
             # Use of specific TID requested, queue it immediately and update
             # last TID.
@@ -375,7 +287,7 @@ class TransactionManager(object):
         txn = Transaction(node, ttid)
         self._ttid_dict[ttid] = txn
         self._node_dict.setdefault(node, {})[ttid] = txn
-        neo.lib.logging.debug('Begin %s', txn)
+        logging.debug('Begin %s', txn)
         return ttid
 
     def prepare(self, ttid, divisor, oid_list, uuid_list, msg_id):
@@ -391,16 +303,19 @@ class TransactionManager(object):
         else:
             tid = self._nextTID(ttid, divisor)
             self._queue.append((node.getUUID(), ttid))
-        neo.lib.logging.debug('Finish TXN %s for %s (was %s)',
-                        dump(tid), node, dump(ttid))
+        logging.debug('Finish TXN %s for %s (was %s)',
+                      dump(tid), node, dump(ttid))
         txn.prepare(tid, oid_list, uuid_list, msg_id)
+        # check if greater and foreign OID was stored
+        if oid_list:
+            self.setLastOID(max(oid_list))
         return tid
 
     def remove(self, uuid, ttid):
         """
             Remove a transaction, commited or aborted
         """
-        neo.lib.logging.debug('Remove TXN %s', dump(ttid))
+        logging.debug('Remove TXN %s', dump(ttid))
         try:
             # only in case of an import:
             self._queue.remove((uuid, ttid))
@@ -421,7 +336,7 @@ class TransactionManager(object):
             If transaction is completely locked, calls function given at
             instanciation time.
         """
-        neo.lib.logging.debug('Lock TXN %s for %s', dump(ttid), dump(uuid))
+        logging.debug('Lock TXN %s for %s', dump(ttid), uuid_str(uuid))
         assert ttid in self._ttid_dict, "Transaction not started"
         txn = self._ttid_dict[ttid]
         if txn.lock(uuid) and self._queue[0][1] == ttid:
@@ -462,7 +377,7 @@ class TransactionManager(object):
         """
             Abort pending transactions initiated by a node
         """
-        neo.lib.logging.debug('Abort TXN for %s', node)
+        logging.debug('Abort TXN for %s', node)
         uuid = node.getUUID()
         # XXX: this loop is usefull only during an import
         for nuuid, ntid in list(self._queue):
@@ -478,7 +393,7 @@ class TransactionManager(object):
             del self._node_dict[node]
 
     def log(self):
-        neo.lib.logging.info('Transactions:')
+        logging.info('Transactions:')
         for txn in self._ttid_dict.itervalues():
-            neo.lib.logging.info('  %r', txn)
+            logging.info('  %r', txn)
 
