@@ -1,6 +1,8 @@
+#!/usr/bin/env python
+#
 # neoadmin - run an administrator node of NEO
 #
-# Copyright (C) 2009  Nexedi SA
+# Copyright (C) 2009-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -13,20 +15,17 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import sys
 from optparse import OptionParser
-from neo.lib import setupLog
+from neo.lib import logging
 from neo.lib.util import parseNodeAddress
 
 parser = OptionParser()
-parser.add_option('-v', '--verbose', action = 'store_true',
-                  help = 'print verbose messages')
 parser.add_option('-a', '--address', help = 'specify the address (ip:port) ' \
     'of an admin node', default = '127.0.0.1:9999')
 parser.add_option('--handler', help = 'specify the connection handler')
+parser.add_option('-l', '--logfile', help = 'specify a logging file')
 
 def main(args=None):
     (options, args) = parser.parse_args(args=args)
@@ -35,7 +34,7 @@ def main(args=None):
     else:
         address = ('127.0.0.1', 9999)
 
-    setupLog('NEOCTL', options.verbose)
+    logging.setup(options.logfile)
     from neo.neoctl.app import Application
 
     print Application(address).execute(args)
