@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2010  Nexedi SA
+# Copyright (C) 2006-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,37 +12,28 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+LOG_QUERIES = False
 
 from neo.lib.exception import DatabaseFailure
-from neo.storage.database.manager import DatabaseManager
+from .manager import DatabaseManager
+from .sqlite import SQLiteDatabaseManager
 
-DATABASE_MANAGER_DICT = {}
+DATABASE_MANAGER_DICT = {'SQLite': SQLiteDatabaseManager}
 
 try:
-    from neo.storage.database.mysqldb import MySQLDatabaseManager
+    from .mysqldb import MySQLDatabaseManager
 except ImportError:
     pass
 else:
     DATABASE_MANAGER_DICT['MySQL'] = MySQLDatabaseManager
 
-try:
-    from neo.storage.database.btree import BTreeDatabaseManager
-except ImportError:
-    pass
-else:
-    # XXX: warning: name might change in the future.
-    DATABASE_MANAGER_DICT['BTree'] = BTreeDatabaseManager
-
-if not DATABASE_MANAGER_DICT:
-    raise ImportError('No database back-end available.')
-
-def buildDatabaseManager(name, config):
+def buildDatabaseManager(name, args=(), kw={}):
     if name is None:
         name = DATABASE_MANAGER_DICT.keys()[0]
     adapter_klass = DATABASE_MANAGER_DICT.get(name, None)
     if adapter_klass is None:
         raise DatabaseFailure('Cannot find a database adapter <%s>' % name)
-    return adapter_klass(config)
+    return adapter_klass(*args, **kw)
 
