@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2010  Nexedi SA
+# Copyright (C) 2006-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,12 +12,10 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import neo.lib
-from neo.lib import util
-from neo.lib.exception import DatabaseFailure
+from neo.lib import logging, util
+from neo.lib.protocol import ZERO_TID
 
 class CreationUndone(Exception):
     pass
@@ -25,41 +23,16 @@ class CreationUndone(Exception):
 class DatabaseManager(object):
     """This class only describes an interface for database managers."""
 
-
-
-    def __init__(self):
+    def __init__(self, database, wait=0):
         """
             Initialize the object.
         """
-        self._under_transaction = False
+        self._wait = wait
+        self._parse(database)
 
-    def isUnderTransaction(self):
-        return self._under_transaction
-
-    def begin(self):
-        """
-            Begin a transaction
-        """
-        if self._under_transaction:
-            raise DatabaseFailure('A transaction has already begun')
-        self._begin()
-        self._under_transaction = True
-
-    def commit(self):
-        """
-            Commit the current transaction
-        """
-        if not self._under_transaction:
-            raise DatabaseFailure('The transaction has not begun')
-        self._commit()
-        self._under_transaction = False
-
-    def rollback(self):
-        """
-            Rollback the current transaction
-        """
-        self._rollback()
-        self._under_transaction = False
+    def _parse(self, database):
+        """Called during instanciation, to process database parameter."""
+        pass
 
     def setup(self, reset = 0):
         """Set up a database
@@ -68,21 +41,15 @@ class DatabaseManager(object):
         _uncommitted_data is a dict containing refcounts to data of
         write-locked objects, except in case of undo, where the refcount is
         increased later, when the object is read-locked.
-        Keys are checksums and values are number of references.
+        Keys are data ids and values are number of references.
 
         If reset is true, existing data must be discarded and
         self._uncommitted_data must be an empty dict.
         """
         raise NotImplementedError
 
-    def _begin(self):
-        raise NotImplementedError
-
-    def _commit(self):
-        raise NotImplementedError
-
-    def _rollback(self):
-        raise NotImplementedError
+    def commit(self):
+        pass
 
     def _getPartition(self, oid_or_tid):
         return oid_or_tid % self.getNumPartitions()
@@ -97,31 +64,25 @@ class DatabaseManager(object):
         """
             Set a configuration value
         """
-        if self._under_transaction:
-            self._setConfiguration(key, value)
-        else:
-            self.begin()
-            try:
-                self._setConfiguration(key, value)
-            except:
-                self.rollback()
-                raise
-            self.commit()
+        self._setConfiguration(key, value)
+        self.commit()
 
     def _setConfiguration(self, key, value):
         raise NotImplementedError
 
     def getUUID(self):
         """
-            Load an UUID from a database.
+            Load a NID from a database.
         """
-        return util.bin(self.getConfiguration('uuid'))
+        nid = self.getConfiguration('nid')
+        if nid is not None:
+            return int(nid)
 
-    def setUUID(self, uuid):
+    def setUUID(self, nid):
         """
-            Store an UUID into a database.
+            Store a NID into a database.
         """
-        self.setConfiguration('uuid', util.dump(uuid))
+        self.setConfiguration('nid', str(nid))
 
     def getNumPartitions(self):
         """
@@ -167,7 +128,9 @@ class DatabaseManager(object):
         """
             Load a Partition Table ID from a database.
         """
-        return long(self.getConfiguration('ptid'))
+        ptid = self.getConfiguration('ptid')
+        if ptid is not None:
+            return long(ptid)
 
     def setPTID(self, ptid):
         """
@@ -178,30 +141,33 @@ class DatabaseManager(object):
             ptid = str(ptid)
         self.setConfiguration('ptid', ptid)
 
-    def getLastOID(self):
-        """
-            Returns the last OID used
-        """
-        return util.bin(self.getConfiguration('loid'))
+    def getBackupTID(self):
+        return util.bin(self.getConfiguration('backup_tid'))
 
-    def setLastOID(self, loid):
-        """
-            Set the last OID used
-        """
-        self.setConfiguration('loid', util.dump(loid))
+    def setBackupTID(self, backup_tid):
+        tid = util.dump(backup_tid)
+        logging.debug('backup_tid = %s', tid)
+        return self.setConfiguration('backup_tid', tid)
 
     def getPartitionTable(self):
-        """Return a whole partition table as a tuple of rows. Each row
-        is again a tuple of an offset (row ID), an UUID of a storage
+        """Return a whole partition table as a sequence of rows. Each row
+        is again a tuple of an offset (row ID), the NID of a storage
         node, and a cell state."""
         raise NotImplementedError
 
-    def getLastTID(self, all = True):
-        """Return the last TID in a database. If all is true,
-        unfinished transactions must be taken account into. If there
-        is no TID in the database, return None."""
+    def _getLastIDs(self, all=True):
         raise NotImplementedError
 
+    def getLastIDs(self, all=True):
+        trans, obj, oid = self._getLastIDs()
+        if trans:
+            tid = max(trans.itervalues())
+            if obj:
+                tid = max(tid, max(obj.itervalues()))
+        else:
+            tid = max(obj.itervalues()) if obj else None
+        return tid, trans, obj, oid
+
     def getUnfinishedTIDList(self):
         """Return a list of unfinished transaction's IDs."""
         raise NotImplementedError
@@ -272,7 +238,7 @@ class DatabaseManager(object):
     def changePartitionTable(self, ptid, cell_list):
         """Change a part of a partition table. The list of cells is
         a tuple of tuples, each of which consists of an offset (row ID),
-        an UUID of a storage node, and a cell state. The Partition
+        the NID of a storage node, and a cell state. The Partition
         Table ID must be stored as well."""
         raise NotImplementedError
 
@@ -282,9 +248,9 @@ class DatabaseManager(object):
         thrown away."""
         raise NotImplementedError
 
-    def dropPartitions(self, num_partitions, offset_list):
-        """ Drop any data of non-assigned partitions for a given UUID """
-        raise NotImplementedError('this method must be overriden')
+    def dropPartitions(self, offset_list):
+        """Delete all data for specified partitions"""
+        raise NotImplementedError
 
     def dropUnfinishedData(self):
         """Drop any unfinished data from a database."""
@@ -294,19 +260,19 @@ class DatabaseManager(object):
         """Store a transaction temporarily, if temporary is true. Note
         that this transaction is not finished yet. The list of objects
         contains tuples, each of which consists of an object ID,
-        a checksum and object serial.
+        a data_id and object serial.
         The transaction is either None or a tuple of the list of OIDs,
         user information, a description, extension information and transaction
         pack state (True for packed)."""
         raise NotImplementedError
 
-    def _pruneData(self, checksum_list):
+    def _pruneData(self, data_id_list):
         """To be overriden by the backend to delete any unreferenced data
 
         'unreferenced' means:
         - not in self._uncommitted_data
         - and not referenced by a fully-committed object (storage should have
-          an index or a refcound of all data checksums of all objects)
+          an index or a refcount of all data ids of all objects)
         """
         raise NotImplementedError
 
@@ -318,41 +284,37 @@ class DatabaseManager(object):
         """
         raise NotImplementedError
 
-    def storeData(self, checksum, data=None, compression=None):
+    def storeData(self, checksum_or_id, data=None, compression=None):
         """Store object raw data
 
-        'checksum' must be the result of neo.lib.util.makeChecksum(data)
+        checksum must be the result of neo.lib.util.makeChecksum(data)
         'compression' indicates if 'data' is compressed.
         A volatile reference is set to this data until 'unlockData' is called
         with this checksum.
-        If called with only a checksum, it only increment the volatile
-        reference to the data matching the checksum.
+        If called with only an id, it only increment the volatile
+        reference to the data matching the id.
         """
         refcount = self._uncommitted_data
-        refcount[checksum] = 1 + refcount.get(checksum, 0)
         if data is not None:
-            self._storeData(checksum, data, compression)
+            checksum_or_id = self._storeData(checksum_or_id, data, compression)
+        refcount[checksum_or_id] = 1 + refcount.get(checksum_or_id, 0)
+        return checksum_or_id
 
-    def unlockData(self, checksum_list, prune=False):
+    def unlockData(self, data_id_list, prune=False):
         """Release 1 volatile reference to given list of checksums
 
         If 'prune' is true, any data that is not referenced anymore (either by
         a volatile reference or by a fully-committed object) is deleted.
         """
         refcount = self._uncommitted_data
-        for checksum in checksum_list:
-            count = refcount[checksum] - 1
+        for data_id in data_id_list:
+            count = refcount[data_id] - 1
             if count:
-                refcount[checksum] = count
+                refcount[data_id] = count
             else:
-                del refcount[checksum]
+                del refcount[data_id]
         if prune:
-            self.begin()
-            try:
-                self._pruneData(checksum_list)
-            except:
-                self.rollback()
-                raise
+            self._pruneData(data_id_list)
             self.commit()
 
     __getDataTID = set()
@@ -375,12 +337,12 @@ class DatabaseManager(object):
         """
         if self.__class__ not in self.__getDataTID:
             self.__getDataTID.add(self.__class__)
-            neo.lib.logging.warning("Fallback to generic/slow implementation"
+            logging.warning("Fallback to generic/slow implementation"
                 " of _getDataTID. It should be overriden by backend storage.")
         r = self._getObject(oid, tid, before_tid)
         if r:
-            serial, _, _, checksum, _, value_serial = r
-            if value_serial is None and checksum:
+            serial, _, _, data_id, _, value_serial = r
+            if value_serial is None and data_id:
                 return serial, serial
             return serial, value_serial
         return None, None
@@ -430,9 +392,9 @@ class DatabaseManager(object):
                                      " oid %d at tid %d: reference = %d"
                                      % (oid, value_serial, tid))
                 if value_serial != getDataTID(value_serial)[1]:
-                    neo.lib.logging.warning("Multiple levels of indirection"
+                    logging.warning("Multiple levels of indirection"
                         " when getting data serial for oid %d at tid %d."
-                        " This causes suboptimal performance." % (oid, tid))
+                        " This causes suboptimal performance.", oid, tid)
             return tid, value_serial
         if transaction_object:
             current_tid = current_data_tid = u64(transaction_object[2])
@@ -461,24 +423,24 @@ class DatabaseManager(object):
         an oid list"""
         raise NotImplementedError
 
-    def deleteTransactionsAbove(self, num_partitions, partition, tid, max_tid):
-        """Delete all transactions above given TID (inclued) in given
-        partition, but never above max_tid (in case transactions are committed
-        during replication)."""
-        raise NotImplementedError
-
     def deleteObject(self, oid, serial=None):
         """Delete given object. If serial is given, only delete that serial for
         given oid."""
         raise NotImplementedError
 
-    def deleteObjectsAbove(self, num_partitions, partition, oid, serial,
-            max_tid):
-        """Delete all objects above given OID and serial (inclued) in given
-        partition, but never above max_tid (in case objects are stored during
-        replication)"""
+    def _deleteRange(self, partition, min_tid=None, max_tid=None):
+        """Delete all objects and transactions between given min_tid (excluded)
+        and max_tid (included)"""
         raise NotImplementedError
 
+    def truncate(self, tid):
+        assert tid not in (None, ZERO_TID), tid
+        assert self.getBackupTID()
+        self.setBackupTID(None) # XXX
+        for partition in xrange(self.getNumPartitions()):
+            self._deleteRange(partition, tid)
+        self.commit()
+
     def getTransaction(self, tid, all = False):
         """Return a tuple of the list of OIDs, user information,
         a description, and extension information, for a given transaction
@@ -494,21 +456,20 @@ class DatabaseManager(object):
         If there is no such object ID in a database, return None."""
         raise NotImplementedError
 
-    def getObjectHistoryFrom(self, oid, min_serial, max_serial, length,
-            num_partitions, partition):
-        """Return a dict of length serials grouped by oid at (or above)
-        min_oid and min_serial and below max_serial, for given partition,
+    def getReplicationObjectList(self, min_tid, max_tid, length, partition,
+            min_oid):
+        """Return a dict of length oids grouped by serial at (or above)
+        min_tid and min_oid and below max_tid, for given partition,
         sorted in ascending order."""
         raise NotImplementedError
 
-    def getTIDList(self, offset, length, num_partitions, partition_list):
+    def getTIDList(self, offset, length, partition_list):
         """Return a list of TIDs in ascending order from an offset,
         at most the specified length. The list of partitions are passed
         to filter out non-applicable TIDs."""
         raise NotImplementedError
 
-    def getReplicationTIDList(self, min_tid, max_tid, length, num_partitions,
-        partition):
+    def getReplicationTIDList(self, min_tid, max_tid, length, partition):
         """Return a list of TIDs in ascending order from an initial tid value,
         at most the specified length up to max_tid. The partition number is
         passed to filter out non-applicable TIDs."""
@@ -526,20 +487,18 @@ class DatabaseManager(object):
             to it.
         - getObjectData function
             To call if value_serial is None and an object needs to be updated.
-            Takes no parameter, returns a 3-tuple: compression, checksum,
+            Takes no parameter, returns a 3-tuple: compression, data_id,
             value
         """
         raise NotImplementedError
 
-    def checkTIDRange(self, min_tid, max_tid, length, num_partitions, partition):
+    def checkTIDRange(self, partition, length, min_tid, max_tid):
         """
         Generate a diggest from transaction list.
         min_tid (packed)
             TID at which verification starts.
         length (int)
             Maximum number of records to include in result.
-        num_partitions, partition (int, int)
-            Specifies concerned partition.
 
         Returns a 3-tuple:
             - number of records actually found
@@ -550,18 +509,15 @@ class DatabaseManager(object):
         """
         raise NotImplementedError
 
-    def checkSerialRange(self, min_oid, min_serial, max_tid, length,
-            num_partitions, partition):
+    def checkSerialRange(self, partition, length, min_tid, max_tid, min_oid):
         """
         Generate a diggest from object list.
         min_oid (packed)
             OID at which verification starts.
-        min_serial (packed)
+        min_tid (packed)
             Serial of min_oid object at which search should start.
         length
             Maximum number of records to include in result.
-        num_partitions, partition (int, int)
-            Specifies concerned partition.
 
         Returns a 5-tuple:
             - number of records actually found
