@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2010  Nexedi SA
+# Copyright (C) 2006-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,26 +12,55 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import neo.lib
-from neo.lib import protocol
+from neo.lib import logging
+from neo.lib.handler import EventHandler
 from neo.lib.util import dump, makeChecksum
-from neo.lib.protocol import Packets, LockState, Errors, ZERO_HASH
-from neo.storage.handlers import BaseClientAndStorageOperationHandler
-from neo.storage.transactions import ConflictError, DelayedError
-from neo.storage.exception import AlreadyPendingError
+from neo.lib.protocol import Packets, LockState, Errors, ProtocolError, \
+    ZERO_HASH, INVALID_PARTITION
+from ..transactions import ConflictError, DelayedError
+from ..exception import AlreadyPendingError
 import time
 
 # Log stores taking (incl. lock delays) more than this many seconds.
 # Set to None to disable.
 SLOW_STORE = 2
 
-class ClientOperationHandler(BaseClientAndStorageOperationHandler):
+class ClientOperationHandler(EventHandler):
 
-    def _askObject(self, oid, serial, ttid):
-        return self.app.dm.getObject(oid, serial, ttid)
+    def askTransactionInformation(self, conn, tid):
+        t = self.app.dm.getTransaction(tid)
+        if t is None:
+            p = Errors.TidNotFound('%s does not exist' % dump(tid))
+        else:
+            p = Packets.AnswerTransactionInformation(tid, t[1], t[2], t[3],
+                    t[4], t[0])
+        conn.answer(p)
+
+    def askObject(self, conn, oid, serial, tid):
+        app = self.app
+        if app.tm.loadLocked(oid):
+            # Delay the response.
+            app.queueEvent(self.askObject, conn, (oid, serial, tid))
+            return
+        o = app.dm.getObject(oid, serial, tid)
+        if o is None:
+            logging.debug('oid = %s does not exist', dump(oid))
+            p = Errors.OidDoesNotExist(dump(oid))
+        elif o is False:
+            logging.debug('oid = %s not found', dump(oid))
+            p = Errors.OidNotFound(dump(oid))
+        else:
+            serial, next_serial, compression, checksum, data, data_serial = o
+            logging.debug('oid = %s, serial = %s, next_serial = %s',
+                          dump(oid), dump(serial), dump(next_serial))
+            if checksum is None:
+                checksum = ZERO_HASH
+                data = ''
+            p = Packets.AnswerObject(oid, serial, next_serial,
+                compression, checksum, data, data_serial)
+        conn.answer(p)
 
     def connectionLost(self, conn, new_state):
         uuid = conn.getUUID()
@@ -52,7 +81,7 @@ class ClientOperationHandler(BaseClientAndStorageOperationHandler):
             data_serial, ttid, unlock, request_time):
         if ttid not in self.app.tm:
             # transaction was aborted, cancel this event
-            neo.lib.logging.info('Forget store of %s:%s by %s delayed by %s',
+            logging.info('Forget store of %s:%s by %s delayed by %s',
                     dump(oid), dump(serial), dump(ttid),
                     dump(self.app.tm.getLockingTID(oid)))
             # send an answer as the client side is waiting for it
@@ -63,8 +92,7 @@ class ClientOperationHandler(BaseClientAndStorageOperationHandler):
                     checksum, data, data_serial, unlock)
         except ConflictError, err:
             # resolvable or not
-            ttid_or_serial = err.getTID()
-            conn.answer(Packets.AnswerStoreObject(1, oid, ttid_or_serial))
+            conn.answer(Packets.AnswerStoreObject(1, oid, err.getTID()))
         except DelayedError:
             # locked by a previous transaction, retry later
             # If we are unlocking, we want queueEvent to raise
@@ -81,7 +109,7 @@ class ClientOperationHandler(BaseClientAndStorageOperationHandler):
             if SLOW_STORE is not None:
                 duration = time.time() - request_time
                 if duration > SLOW_STORE:
-                    neo.lib.logging.info('StoreObject delay: %.02fs', duration)
+                    logging.info('StoreObject delay: %.02fs', duration)
             conn.answer(Packets.AnswerStoreObject(0, oid, serial))
 
     def askStoreObject(self, conn, oid, serial,
@@ -97,31 +125,23 @@ class ClientOperationHandler(BaseClientAndStorageOperationHandler):
         self._askStoreObject(conn, oid, serial, compression, checksum, data,
             data_serial, ttid, unlock, time.time())
 
-    def askTIDsFrom(self, conn, min_tid, max_tid, length, partition_list):
-        app = self.app
-        getReplicationTIDList = app.dm.getReplicationTIDList
-        partitions = app.pt.getPartitions()
-        tid_list = []
-        extend = tid_list.extend
-        for partition in partition_list:
-            extend(getReplicationTIDList(min_tid, max_tid, length,
-                partitions, partition))
-        conn.answer(Packets.AnswerTIDsFrom(tid_list))
+    def askTIDsFrom(self, conn, min_tid, max_tid, length, partition):
+        conn.answer(Packets.AnswerTIDsFrom(self.app.dm.getReplicationTIDList(
+            min_tid, max_tid, length, partition)))
 
     def askTIDs(self, conn, first, last, partition):
         # This method is complicated, because I must return TIDs only
         # about usable partitions assigned to me.
         if first >= last:
-            raise protocol.ProtocolError('invalid offsets')
+            raise ProtocolError('invalid offsets')
 
         app = self.app
-        if partition == protocol.INVALID_PARTITION:
+        if partition == INVALID_PARTITION:
             partition_list = app.pt.getAssignedPartitionList(app.uuid)
         else:
             partition_list = [partition]
 
-        tid_list = app.dm.getTIDList(first, last - first,
-                             app.pt.getPartitions(), partition_list)
+        tid_list = app.dm.getTIDList(first, last - first, partition_list)
         conn.answer(Packets.AnswerTIDs(tid_list))
 
     def askObjectUndoSerial(self, conn, ttid, ltid, undone_tid, oid_list):
@@ -142,8 +162,7 @@ class ClientOperationHandler(BaseClientAndStorageOperationHandler):
 
     def askHasLock(self, conn, ttid, oid):
         locking_tid = self.app.tm.getLockingTID(oid)
-        neo.lib.logging.info('%r check lock of %r:%r', conn,
-                        dump(ttid), dump(oid))
+        logging.info('%r check lock of %r:%r', conn, dump(ttid), dump(oid))
         if locking_tid is None:
             state = LockState.NOT_LOCKED
         elif locking_tid is ttid:
@@ -154,7 +173,7 @@ class ClientOperationHandler(BaseClientAndStorageOperationHandler):
 
     def askObjectHistory(self, conn, oid, first, last):
         if first >= last:
-            raise protocol.ProtocolError( 'invalid offsets')
+            raise ProtocolError('invalid offsets')
 
         app = self.app
         history_list = app.dm.getObjectHistory(oid, first, last - first)
@@ -170,9 +189,8 @@ class ClientOperationHandler(BaseClientAndStorageOperationHandler):
     def _askCheckCurrentSerial(self, conn, ttid, serial, oid, request_time):
         if ttid not in self.app.tm:
             # transaction was aborted, cancel this event
-            neo.lib.logging.info(
-                'Forget serial check of %s:%s by %s delayed by '
-                '%s', dump(oid), dump(serial), dump(ttid),
+            logging.info('Forget serial check of %s:%s by %s delayed by %s',
+                dump(oid), dump(serial), dump(ttid),
                 dump(self.app.tm.getLockingTID(oid)))
             # send an answer as the client side is waiting for it
             conn.answer(Packets.AnswerStoreObject(0, oid, serial))
@@ -194,7 +212,6 @@ class ClientOperationHandler(BaseClientAndStorageOperationHandler):
             if SLOW_STORE is not None:
                 duration = time.time() - request_time
                 if duration > SLOW_STORE:
-                    neo.lib.logging.info('CheckCurrentSerial delay: %.02fs',
-                        duration)
+                    logging.info('CheckCurrentSerial delay: %.02fs', duration)
             conn.answer(Packets.AnswerCheckCurrentSerial(0, oid, serial))
 
