@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2009-2010  Nexedi SA
+# Copyright (C) 2009-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,10 +12,10 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import __builtin__
+import errno
 import os
 import random
 import socket
@@ -23,15 +23,18 @@ import sys
 import tempfile
 import unittest
 import MySQLdb
-import neo
 import transaction
 
 from mock import Mock
-from neo.lib import debug, logger, protocol, setupLog
-from neo.lib.protocol import Packets
+from neo.lib import debug, logging, protocol
+from neo.lib.protocol import NodeTypes, Packets, UUID_NAMESPACES
 from neo.lib.util import getAddressType
-from time import time, gmtime
+from time import time
 from struct import pack, unpack
+try:
+    from ZODB.utils import newTid
+except ImportError:
+    pass
 
 DB_PREFIX = os.getenv('NEO_DB_PREFIX', 'test_neo')
 DB_ADMIN = os.getenv('NEO_DB_ADMIN', 'root')
@@ -45,10 +48,12 @@ IP_VERSION_FORMAT_DICT = {
 
 ADDRESS_TYPE = socket.AF_INET
 
-debug.ENABLED = True
+logging.default_root_handler.handle = lambda record: None
+logging.backlog(None)
+
 debug.register()
 # prevent "signal only works in main thread" errors in subprocesses
-debug.ENABLED = False
+debug.register = lambda on_log=None: None
 
 def mockDefaultValue(name, function):
     def method(self, *args, **kw):
@@ -110,26 +115,33 @@ def setupMySQLdb(db_list, user=DB_USER, password='', clear_databases=True):
     conn.close()
 
 class NeoTestBase(unittest.TestCase):
+
     def setUp(self):
-        logger.PACKET_LOGGER.enable(True)
         sys.stdout.write(' * %s ' % (self.id(), ))
         sys.stdout.flush()
-        self.setupLog()
+        logging.name = self.setupLog()
         unittest.TestCase.setUp(self)
 
     def setupLog(self):
-        test_case, test_method = self.id().rsplit('.', 1)
-        log_file = os.path.join(getTempDirectory(), test_case + '.log')
-        setupLog(test_method, log_file, True)
+        test_case, logging.name = self.id().rsplit('.', 1)
+        logging.setup(os.path.join(getTempDirectory(), test_case + '.log'))
 
-    def tearDown(self):
+    def tearDown(self,
+            success='ok' if sys.version_info < (2, 7) else 'success'):
+        assert self.tearDown.im_func is NeoTestBase.tearDown.im_func
+        self._tearDown(sys._getframe(1).f_locals[success])
+
+    def _tearDown(self, success):
         # Kill all unfinished transactions for next test.
         # Note we don't even abort them because it may require a valid
         # connection to a master node (see Storage.sync()).
         transaction.manager.__init__()
-        unittest.TestCase.tearDown(self)
-        sys.stdout.write('\n')
-        sys.stdout.flush()
+        print
+
+    class failureException(AssertionError):
+        def __init__(self, msg=None):
+            logging.error(msg)
+            AssertionError.__init__(self, msg)
 
     failIfEqual = failUnlessEqual = assertEquals = assertNotEquals = None
 
@@ -148,6 +160,10 @@ class NeoUnitTestBase(NeoTestBase):
 
     local_ip = IP_VERSION_FORMAT_DICT[ADDRESS_TYPE]
 
+    def setUp(self):
+        self.uuid_dict = {}
+        NeoTestBase.setUp(self)
+
     def prepareDatabase(self, number, prefix='test_neo'):
         """ create empties databases """
         setupMySQLdb(['%s%u' % (prefix, i) for i in xrange(number)])
@@ -173,66 +189,47 @@ class NeoUnitTestBase(NeoTestBase):
         assert index >= 0 and index <= 9
         masters = [(buildUrlFromString(self.local_ip),
                      10010 + i) for i in xrange(master_number)]
-        database = '%s@%s%s' % (DB_USER, prefix, index)
+        adapter = os.getenv('NEO_TESTS_ADAPTER', 'MySQL')
+        if adapter == 'MySQL':
+            db = '%s@%s%s' % (DB_USER, prefix, index)
+        elif adapter == 'SQLite':
+            db = os.path.join(getTempDirectory(), 'test_neo%s.sqlite' % index)
+        else:
+            assert False, adapter
         return Mock({
                 'getCluster': cluster,
-                'getName': 'storage',
                 'getBind': (masters[0], 10020 + index),
                 'getMasters': (masters, getAddressType((
                         self.local_ip, 0))),
-                'getDatabase': database,
+                'getDatabase': db,
                 'getUUID': uuid,
                 'getReset': False,
-                'getAdapter': 'MySQL',
+                'getAdapter': adapter,
         })
 
-    def _makeUUID(self, prefix):
+    def getNewUUID(self, node_type):
         """
             Retuns a 16-bytes UUID according to namespace 'prefix'
         """
-        assert len(prefix) == 1
-        uuid = protocol.INVALID_UUID
-        while uuid[1:] == protocol.INVALID_UUID[1:]:
-            uuid = prefix + os.urandom(15)
-        return uuid
-
-    def getNewUUID(self):
-        return self._makeUUID('\0')
+        if node_type is None:
+            node_type = random.choice(NodeTypes)
+        self.uuid_dict[node_type] = uuid = 1 + self.uuid_dict.get(node_type, 0)
+        return uuid + (UUID_NAMESPACES[node_type] << 24)
 
     def getClientUUID(self):
-        return self._makeUUID('C')
+        return self.getNewUUID(NodeTypes.CLIENT)
 
     def getMasterUUID(self):
-        return self._makeUUID('M')
+        return self.getNewUUID(NodeTypes.MASTER)
 
     def getStorageUUID(self):
-        return self._makeUUID('S')
+        return self.getNewUUID(NodeTypes.STORAGE)
 
     def getAdminUUID(self):
-        return self._makeUUID('A')
+        return self.getNewUUID(NodeTypes.ADMIN)
 
     def getNextTID(self, ltid=None):
-        tm = time()
-        gmt = gmtime(tm)
-        upper = ((((gmt.tm_year - 1900) * 12 + gmt.tm_mon - 1) * 31 \
-                  + gmt.tm_mday - 1) * 24 + gmt.tm_hour) * 60 + gmt.tm_min
-        lower = int((gmt.tm_sec % 60 + (tm - int(tm))) / (60.0 / 65536.0 / 65536.0))
-        tid = pack('!LL', upper, lower)
-        if ltid is not None and tid <= ltid:
-            upper, lower = unpack('!LL', self._last_tid)
-            if lower == 0xffffffff:
-                # This should not happen usually.
-                from datetime import timedelta, datetime
-                d = datetime(gmt.tm_year, gmt.tm_mon, gmt.tm_mday,
-                             gmt.tm_hour, gmt.tm_min) \
-                        + timedelta(0, 60)
-                upper = ((((d.year - 1900) * 12 + d.month - 1) * 31 \
-                          + d.day - 1) * 24 + d.hour) * 60 + d.minute
-                lower = 0
-            else:
-                lower += 1
-            tid = pack('!LL', upper, lower)
-        return tid
+        return newTid(ltid)
 
     def getPTID(self, i=None):
         """ Return an integer PTID """
@@ -246,12 +243,6 @@ class NeoUnitTestBase(NeoTestBase):
             return os.urandom(8)
         return pack('!Q', i)
 
-    def getTwoIDs(self):
-        """ Return a tuple of two sorted UUIDs """
-        # generate two ptid, first is lower
-        uuids = self.getNewUUID(), self.getNewUUID()
-        return min(uuids), max(uuids)
-
     def getFakeConnector(self, descriptor=None):
         return Mock({
             '__repr__': 'FakeConnector',
@@ -263,7 +254,7 @@ class NeoUnitTestBase(NeoTestBase):
             is_server=False, connector=None, peer_id=None):
         if connector is None:
             connector = self.getFakeConnector()
-        return Mock({
+        conn = Mock({
             'getUUID': uuid,
             'getAddress': address,
             'isServer': is_server,
@@ -272,6 +263,9 @@ class NeoUnitTestBase(NeoTestBase):
             'getConnector': connector,
             'getPeerId': peer_id,
         })
+        conn.mockAddReturnValues(__hash__ = id(conn))
+        conn.connecting = False
+        return conn
 
     def checkProtocolErrorRaised(self, method, *args, **kwargs):
         """ Check if the ProtocolError exception was raised """
@@ -327,13 +321,15 @@ class NeoUnitTestBase(NeoTestBase):
         """ ensure no UUID was set on the connection """
         self.assertEqual(len(conn.mockGetNamedCalls('setUUID')), 0)
 
-    def checkUUIDSet(self, conn, uuid=None):
-        """ ensure no UUID was set on the connection """
+    def checkUUIDSet(self, conn, uuid=None, check_intermediate=True):
+        """ ensure UUID was set on the connection """
         calls = conn.mockGetNamedCalls('setUUID')
-        self.assertEqual(len(calls), 1)
-        call = calls.pop()
+        found_uuid = calls.pop().getParam(0)
+        if check_intermediate:
+            for call in calls:
+                self.assertEqual(found_uuid, call.getParam(0))
         if uuid is not None:
-            self.assertEqual(call.getParam(0), uuid)
+            self.assertEqual(found_uuid, uuid)
 
     # in check(Ask|Answer|Notify)Packet we return the packet so it can be used
     # in tests if more accurates checks are required
@@ -347,7 +343,6 @@ class NeoUnitTestBase(NeoTestBase):
         self.assertEqual(type(packet), Packets.Error)
         if decode:
             return packet.decode()
-            return protocol.decode_table[type(packet)](packet._body)
         return packet
 
     def checkAskPacket(self, conn, packet_type, decode=False):
@@ -481,18 +476,12 @@ class NeoUnitTestBase(NeoTestBase):
     def checkAnswerObjectHistory(self, conn, **kw):
         return self.checkAnswerPacket(conn, Packets.AnswerObjectHistory, **kw)
 
-    def checkAnswerObjectHistoryFrom(self, conn, **kw):
-        return self.checkAnswerPacket(conn, Packets.AnswerObjectHistoryFrom, **kw)
-
     def checkAnswerStoreTransaction(self, conn, **kw):
         return self.checkAnswerPacket(conn, Packets.AnswerStoreTransaction, **kw)
 
     def checkAnswerStoreObject(self, conn, **kw):
         return self.checkAnswerPacket(conn, Packets.AnswerStoreObject, **kw)
 
-    def checkAnswerOids(self, conn, **kw):
-        return self.checkAnswerPacket(conn, Packets.AnswerOIDs, **kw)
-
     def checkAnswerPartitionTable(self, conn, **kw):
         return self.checkAnswerPacket(conn, Packets.AnswerPartitionTable, **kw)
 
@@ -503,7 +492,7 @@ connector_cpt = 0
 
 class DoNothingConnector(Mock):
     def __init__(self, s=None):
-        neo.lib.logging.info("initializing connector")
+        logging.info("initializing connector")
         global connector_cpt
         self.desc = connector_cpt
         connector_cpt += 1
@@ -525,3 +514,28 @@ class DoNothingConnector(Mock):
 
 __builtin__.pdb = lambda depth=0: \
     debug.getPdb().set_trace(sys._getframe(depth+1))
+
+def _fixMockForInspect():
+    """
+    inspect module change broke Mock, see http://bugs.python.org/issue1785
+    Monkey-patch Mock class if needed by replacing predicate parameter on 2nd
+    getmembers call with isroutine (was ismethod).
+    """
+    import inspect
+    class A(object):
+        def f(self):
+            pass
+    if not inspect.getmembers(A, inspect.ismethod):
+        from mock import MockCallable
+        # _setupSubclassMethodInterceptors is under the FreeBSD license.
+        # See pyMock module for the whole license.
+        def _setupSubclassMethodInterceptors(self):
+            methods = inspect.getmembers(self.__class__, inspect.isroutine)
+            baseMethods = dict(inspect.getmembers(Mock, inspect.isroutine))
+            for m in methods:
+                name = m[0]
+                # Don't record calls to methods of Mock base class.
+                if not name in baseMethods:
+                    self.__dict__[name] = MockCallable(name, self, handcrafted=True)
+        Mock._setupSubclassMethodInterceptors = _setupSubclassMethodInterceptors
+_fixMockForInspect()
