@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2009-2010  Nexedi SA
+# Copyright (C) 2009-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,31 +12,35 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import unittest
 from cPickle import dumps
 from mock import Mock, ReturnValues
 from ZODB.POSException import StorageTransactionError, UndoError, ConflictError
-from neo.tests import NeoUnitTestBase, buildUrlFromString, ADDRESS_TYPE
+from .. import NeoUnitTestBase, buildUrlFromString, ADDRESS_TYPE
 from neo.client.app import Application
+from neo.client.cache import test as testCache
 from neo.client.exception import NEOStorageError, NEOStorageNotFoundError
-from neo.client.exception import NEOStorageDoesNotExistError
-from neo.lib.protocol import Packet, Packets, Errors, INVALID_TID, \
-    INVALID_PARTITION
+from neo.lib.protocol import NodeTypes, Packets, Errors, \
+    INVALID_PARTITION, UUID_NAMESPACES
 from neo.lib.util import makeChecksum, SOCKET_CONNECTORS_DICT
 import time
 
+class Dispatcher(object):
+
+    def pending(self, queue):
+        return not queue.empty()
+
+    def forget_queue(self, queue, flush_queue=True):
+        pass
+
 def _getMasterConnection(self):
     if self.master_conn is None:
-        self.uuid = 'C' * 16
+        self.uuid = 1 + (UUID_NAMESPACES[NodeTypes.CLIENT] << 24)
         self.num_partitions = 10
         self.num_replicas = 1
-        self.pt = Mock({
-            'getCellListForOID': (),
-            'getCellListForTID': (),
-        })
+        self.pt = Mock({'getCellList': ()})
         self.master_conn = Mock()
     return self.master_conn
 
@@ -45,9 +49,9 @@ def getPartitionTable(self):
         self.master_conn = _getMasterConnection(self)
     return self.pt
 
-def _ask(self, conn, packet, handler=None):
+def _ask(self, conn, packet, handler=None, **kw):
     self.setHandlerData(None)
-    conn.ask(packet)
+    conn.ask(packet, **kw)
     if handler is None:
         raise NotImplementedError
     else:
@@ -73,7 +77,7 @@ class ClientApplicationTests(NeoUnitTestBase):
         Application.getPartitionTable = getPartitionTable
         self._to_stop_list = []
 
-    def tearDown(self):
+    def _tearDown(self, success):
         # stop threads
         for app in self._to_stop_list:
             app.close()
@@ -81,7 +85,7 @@ class ClientApplicationTests(NeoUnitTestBase):
         Application._getMasterConnection = self._getMasterConnection
         Application._ask = self._ask
         Application.getPartitionTable = self.getPartitionTable
-        NeoUnitTestBase.tearDown(self)
+        NeoUnitTestBase._tearDown(self, success)
 
     # some helpers
 
@@ -109,7 +113,7 @@ class ClientApplicationTests(NeoUnitTestBase):
     def makeOID(self, value=None):
         from random import randint
         if value is None:
-            value = randint(0, 255)
+            value = randint(1, 255)
         return '\00' * 7 + chr(value)
     makeTID = makeOID
 
@@ -156,6 +160,8 @@ class ClientApplicationTests(NeoUnitTestBase):
         #self.assertEqual(calls[0].getParam(0), conn)
         #self.assertTrue(isinstance(calls[0].getParam(2), Queue))
 
+    testCache = testCache
+
     def test_registerDB(self):
         app = self.getApp()
         dummy_db = []
@@ -285,13 +291,12 @@ class ClientApplicationTests(NeoUnitTestBase):
             None, txn)
         # check partition_id and an empty cell list -> NEOStorageError
         self._begin(app, txn, self.makeTID())
-        app.pt = Mock({ 'getCellListForOID': (), })
+        app.pt = Mock({'getCellList': ()})
         app.num_partitions = 2
         self.assertRaises(NEOStorageError, app.store, oid, tid, '',  None,
             txn)
-        calls = app.pt.mockGetNamedCalls('getCellListForOID')
+        calls = app.pt.mockGetNamedCalls('getCellList')
         self.assertEqual(len(calls), 1)
-        self.assertEqual(calls[0].getParam(0), oid) # oid=11
 
     def test_store2(self):
         app = self.getApp()
@@ -304,17 +309,14 @@ class ClientApplicationTests(NeoUnitTestBase):
         packet.setId(0)
         storage_address = ('127.0.0.1', 10020)
         node, cell, conn = self.getNodeCellConn(address=storage_address)
-        app.pt = Mock({ 'getCellListForOID': (cell, cell)})
+        app.pt = Mock()
         app.cp = self.getConnectionPool([(node, conn)])
-        class Dispatcher(object):
-            def pending(self, queue):
-                return not queue.empty()
         app.dispatcher = Dispatcher()
         app.nm.createStorage(address=storage_address)
         data_dict = txn_context['data_dict']
         data_dict[oid] = 'BEFORE'
         app.store(oid, tid, '', None, txn)
-        txn_context['queue'].put((conn, packet))
+        txn_context['queue'].put((conn, packet, {}))
         self.assertRaises(ConflictError, app.waitStoreResponses, txn_context,
             failing_tryToResolveConflict)
         self.assertTrue(oid not in data_dict)
@@ -323,7 +325,7 @@ class ClientApplicationTests(NeoUnitTestBase):
 
     def test_store3(self):
         app = self.getApp()
-        uuid = self.getNewUUID()
+        uuid = self.getStorageUUID()
         oid = self.makeOID(11)
         tid = self.makeTID()
         txn = self.makeTransactionObject()
@@ -335,15 +337,12 @@ class ClientApplicationTests(NeoUnitTestBase):
         node, cell, conn = self.getNodeCellConn(address=storage_address,
             uuid=uuid)
         app.cp = self.getConnectionPool([(node, conn)])
-        app.pt = Mock({ 'getCellListForOID': (cell, cell, ) })
-        class Dispatcher(object):
-            def pending(self, queue):
-                return not queue.empty()
+        app.pt = Mock()
         app.dispatcher = Dispatcher()
         app.nm.createStorage(address=storage_address)
         app.store(oid, tid, 'DATA', None, txn)
         self.checkAskStoreObject(conn)
-        txn_context['queue'].put((conn, packet))
+        txn_context['queue'].put((conn, packet, {}))
         app.waitStoreResponses(txn_context, resolving_tryToResolveConflict)
         self.assertEqual(txn_context['object_stored_counter_dict'][oid],
             {tid: set([uuid])})
@@ -389,7 +388,6 @@ class ClientApplicationTests(NeoUnitTestBase):
         app.master_conn = Mock()
         conn = Mock()
         cell = Mock()
-        app.pt = Mock({'getCellListForTID': (cell, cell)})
         app.cp = Mock({'getConnForCell': ReturnValues(None, cell)})
         app.tpc_abort(txn)
         # no packet sent
@@ -433,10 +431,9 @@ class ClientApplicationTests(NeoUnitTestBase):
         oid1 = self.makeOID(num_partitions + 1) # on partition 1, conflicting
         oid2 = self.makeOID(num_partitions + 2) # on partition 2
         # storage nodes
-        uuid1, uuid2, uuid3 = [self.getNewUUID() for _ in range(3)]
-        address1 = ('127.0.0.1', 10000)
-        address2 = ('127.0.0.1', 10001)
-        address3 = ('127.0.0.1', 10002)
+        address1 = ('127.0.0.1', 10000); uuid1 = self.getMasterUUID()
+        address2 = ('127.0.0.1', 10001); uuid2 = self.getStorageUUID()
+        address3 = ('127.0.0.1', 10002); uuid3 = self.getStorageUUID()
         app.nm.createMaster(address=address1, uuid=uuid1)
         app.nm.createStorage(address=address2, uuid=uuid2)
         app.nm.createStorage(address=address3, uuid=uuid3)
@@ -454,14 +451,7 @@ class ClientApplicationTests(NeoUnitTestBase):
         node1 = Mock({'__repr__': 'node1', '__hash__': 1, 'getConnection': conn1})
         node2 = Mock({'__repr__': 'node2', '__hash__': 2, 'getConnection': conn2})
         node3 = Mock({'__repr__': 'node3', '__hash__': 3, 'getConnection': conn3})
-        cell1 = Mock({ 'getNode': node1, '__hash__': 1, 'getConnection': conn1})
-        cell2 = Mock({ 'getNode': node2, '__hash__': 2, 'getConnection': conn2})
-        cell3 = Mock({ 'getNode': node3, '__hash__': 3, 'getConnection': conn3})
         # fake environment
-        app.pt = Mock({
-            'getCellListForTID': [cell1],
-            'getCellListForOID': ReturnValues([cell2], [cell3]),
-        })
         app.cp = Mock({'getConnForCell': ReturnValues(conn2, conn3, conn1)})
         app.cp = Mock({
             'getConnForNode': ReturnValues(conn2, conn3, conn1),
@@ -470,19 +460,13 @@ class ClientApplicationTests(NeoUnitTestBase):
         app.master_conn = Mock({'__hash__': 0})
         txn = self.makeTransactionObject()
         txn_context = self._begin(app, txn, tid)
-        class Dispatcher(object):
-            def pending(self, queue):
-                return not queue.empty()
-
-            def forget_queue(self, queue, flush_queue=True):
-                pass
         app.dispatcher = Dispatcher()
         # conflict occurs on storage 2
         app.store(oid1, tid, 'DATA', None, txn)
         app.store(oid2, tid, 'DATA', None, txn)
         queue = txn_context['queue']
-        queue.put((conn2, packet2))
-        queue.put((conn3, packet3))
+        queue.put((conn2, packet2, {}))
+        queue.put((conn3, packet3, {}))
         # vote fails as the conflict is not resolved, nothing is sent to storage 3
         self.assertRaises(ConflictError, app.tpc_vote, txn, failing_tryToResolveConflict)
         # abort must be sent to storage 1 and 2
@@ -508,9 +492,6 @@ class ClientApplicationTests(NeoUnitTestBase):
         txn_context = self._begin(app, txn, tid)
         self.f_called = False
         self.f_called_with_tid = None
-        def hook(tid):
-            self.f_called = True
-            self.f_called_with_tid = tid
         packet = Packets.AnswerTransactionFinished(ttid, tid)
         packet.setId(0)
         app.master_conn = Mock({
@@ -519,9 +500,7 @@ class ClientApplicationTests(NeoUnitTestBase):
             'fakeReceived': packet,
         })
         txn_context['txn_voted'] = True
-        app.tpc_finish(txn, None, hook)
-        self.assertTrue(self.f_called)
-        self.assertEqual(self.f_called_with_tid, tid)
+        app.tpc_finish(txn, None)
         self.checkAskFinishTransaction(app.master_conn)
         #self.checkDispatcherRegisterCalled(app, app.master_conn)
         self.assertEqual(app._txn_container.get(txn), None)
@@ -530,13 +509,12 @@ class ClientApplicationTests(NeoUnitTestBase):
         # invalid transaction
         app = self.getApp()
         tid = self.makeTID()
-        snapshot_tid = self.getNextTID()
         txn = self.makeTransactionObject()
         def tryToResolveConflict(oid, conflict_serial, serial, data):
             pass
         app.master_conn = Mock()
         conn = Mock()
-        self.assertRaises(StorageTransactionError, app.undo, snapshot_tid, tid,
+        self.assertRaises(StorageTransactionError, app.undo, tid,
             txn, tryToResolveConflict)
         # no packet sent
         self.checkNoPacketSent(conn)
@@ -548,27 +526,20 @@ class ClientApplicationTests(NeoUnitTestBase):
             'getAddress': 'FakeServer',
             'getState': 'FakeState',
         })
-        app.pt = Mock({
-            'getCellListForTID': [cell, ],
-            'getCellListForOID': [cell, ],
-            'getCellList': [cell, ],
-        })
+        app.pt = Mock({'getCellList': [cell]})
         transaction_info = Packets.AnswerTransactionInformation(tid1, '', '',
             '', False, (oid0, ))
         transaction_info.setId(1)
         conn = Mock({
             'getNextId': 1,
             'fakeReceived': transaction_info,
-            'getAddress': ('127.0.0.1', 10010),
+            'getAddress': ('127.0.0.1', 10020),
         })
         node = app.nm.createStorage(address=conn.getAddress())
         app.cp = Mock({
             'iterateForObject': [(node, conn)],
             'getConnForCell': conn,
         })
-        class Dispatcher(object):
-            def pending(self, queue):
-                return not queue.empty()
         app.dispatcher = Dispatcher()
         def load(oid, tid=None, before_tid=None):
             self.assertEqual(oid, oid0)
@@ -579,6 +550,7 @@ class ClientApplicationTests(NeoUnitTestBase):
                 unlock=False):
             store_marker.append((oid, serial, data, data_serial))
         app._store = _store
+        app.last_tid = self.getNextTID()
         return app, conn, store_marker
 
     def test_undoWithResolutionSuccess(self):
@@ -594,13 +566,14 @@ class ClientApplicationTests(NeoUnitTestBase):
         tid1 = self.getNextTID()
         tid2 = self.getNextTID()
         tid3 = self.getNextTID()
-        snapshot_tid = self.getNextTID()
         app, conn, store_marker = self._getAppForUndoTests(oid0, tid0, tid1,
             tid2)
         undo_serial = Packets.AnswerObjectUndoSerial({
             oid0: (tid2, tid0, False)})
+        conn.ask = lambda p, queue=None, **kw: \
+            isinstance(p, Packets.AskObjectUndoSerial) and \
+            queue.put((conn, undo_serial, kw))
         undo_serial.setId(2)
-        app._getThreadQueue().put((conn, undo_serial))
         marker = []
         def tryToResolveConflict(oid, conflict_serial, serial, data,
                 committedData=''):
@@ -608,7 +581,7 @@ class ClientApplicationTests(NeoUnitTestBase):
             return 'solved'
         # The undo
         txn = self.beginTransaction(app, tid=tid3)
-        app.undo(snapshot_tid, tid1, txn, tryToResolveConflict)
+        app.undo(tid1, txn, tryToResolveConflict)
         # Checking what happened
         moid, mconflict_serial, mserial, mdata, mcommittedData = marker[0]
         self.assertEqual(moid, oid0)
@@ -635,13 +608,14 @@ class ClientApplicationTests(NeoUnitTestBase):
         tid1 = self.getNextTID()
         tid2 = self.getNextTID()
         tid3 = self.getNextTID()
-        snapshot_tid = self.getNextTID()
         undo_serial = Packets.AnswerObjectUndoSerial({
             oid0: (tid2, tid0, False)})
         undo_serial.setId(2)
         app, conn, store_marker = self._getAppForUndoTests(oid0, tid0, tid1,
             tid2)
-        app._getThreadQueue().put((conn, undo_serial))
+        conn.ask = lambda p, queue=None, **kw: \
+            type(p) is Packets.AskObjectUndoSerial and \
+            queue.put((conn, undo_serial, kw))
         marker = []
         def tryToResolveConflict(oid, conflict_serial, serial, data,
                 committedData=''):
@@ -649,8 +623,7 @@ class ClientApplicationTests(NeoUnitTestBase):
             return None
         # The undo
         txn = self.beginTransaction(app, tid=tid3)
-        self.assertRaises(UndoError, app.undo, snapshot_tid, tid1, txn,
-            tryToResolveConflict)
+        self.assertRaises(UndoError, app.undo, tid1, txn, tryToResolveConflict)
         # Checking what happened
         moid, mconflict_serial, mserial, mdata, mcommittedData = marker[0]
         self.assertEqual(moid, oid0)
@@ -667,9 +640,7 @@ class ClientApplicationTests(NeoUnitTestBase):
             marker.append((oid, conflict_serial, serial, data, committedData))
             raise ConflictError
         # The undo
-        app._getThreadQueue().put((conn, undo_serial))
-        self.assertRaises(UndoError, app.undo, snapshot_tid, tid1, txn,
-            tryToResolveConflict)
+        self.assertRaises(UndoError, app.undo, tid1, txn, tryToResolveConflict)
         # Checking what happened
         moid, mconflict_serial, mserial, mdata, mcommittedData = marker[0]
         self.assertEqual(moid, oid0)
@@ -691,7 +662,6 @@ class ClientApplicationTests(NeoUnitTestBase):
         tid1 = self.getNextTID()
         tid2 = self.getNextTID()
         tid3 = self.getNextTID()
-        snapshot_tid = self.getNextTID()
         transaction_info = Packets.AnswerTransactionInformation(tid1, '', '',
             '', False, (oid0, ))
         transaction_info.setId(1)
@@ -700,14 +670,16 @@ class ClientApplicationTests(NeoUnitTestBase):
         undo_serial.setId(2)
         app, conn, store_marker = self._getAppForUndoTests(oid0, tid0, tid1,
             tid2)
-        app._getThreadQueue().put((conn, undo_serial))
+        conn.ask = lambda p, queue=None, **kw: \
+            type(p) is Packets.AskObjectUndoSerial and \
+            queue.put((conn, undo_serial, kw))
         def tryToResolveConflict(oid, conflict_serial, serial, data,
                 committedData=''):
             raise Exception, 'Test called conflict resolution, but there ' \
                 'is no conflict in this test !'
         # The undo
         txn = self.beginTransaction(app, tid=tid3)
-        app.undo(snapshot_tid, tid1, txn, tryToResolveConflict)
+        app.undo(tid1, txn, tryToResolveConflict)
         # Checking what happened
         moid, mserial, mdata, mdata_serial = store_marker[0]
         self.assertEqual(moid, oid0)
@@ -718,10 +690,8 @@ class ClientApplicationTests(NeoUnitTestBase):
     def test_undoLog(self):
         app = self.getApp()
         app.num_partitions = 2
-        uuid1, uuid2 = '\x00' * 15 + '\x01', '\x00' * 15 + '\x02'
+        uuid1, uuid2 = self.getStorageUUID(), self.getStorageUUID()
         # two nodes, two partition, two transaction, two objects :
-        node1, node2 = Mock({}), Mock({})
-        cell1, cell2 = Mock({}), Mock({})
         tid1, tid2 = self.makeTID(1), self.makeTID(2)
         oid1, oid2 = self.makeOID(1), self.makeOID(2)
         # TIDs packets supplied by _ask hook
@@ -742,40 +712,41 @@ class ClientApplicationTests(NeoUnitTestBase):
             'getUUID': ReturnValues(uuid1, uuid2),
             'fakeGetApp': app,
             'fakeReceived': ReturnValues(p3, p4),
-            'getAddress': ('127.0.0.1', 10010),
+            'getAddress': ('127.0.0.1', 10021),
         })
-        storage_1_conn = Mock()
-        storage_2_conn = Mock()
+        asked = []
+        def answerTIDs(packet):
+            conn = Mock({'getAddress': packet})
+            app.nm.createStorage(address=conn.getAddress())
+            def ask(p, queue, **kw):
+                asked.append(p)
+                queue.put((conn, packet, kw))
+            conn.ask = ask
+            return conn
+        app.dispatcher = Dispatcher()
         app.pt = Mock({
-            'getNodeList': (node1, node2, ),
-            'getCellListForTID': ReturnValues([cell1], [cell2]),
+            'getNodeSet': (Mock(), Mock()),
         })
         app.cp = Mock({
-            'getConnForNode': ReturnValues(storage_1_conn, storage_2_conn),
+            'getConnForNode': ReturnValues(answerTIDs(p1), answerTIDs(p2)),
             'iterateForObject': [(Mock(), conn)]
         })
-        def waitResponses(queue, handler_data):
-            app.setHandlerData(handler_data)
-            for p in (p1, p2):
-                app._handlePacket(Mock(), p, handler=app.storage_handler)
-        app.waitResponses = waitResponses
         def txn_filter(info):
             return info['id'] > '\x00' * 8
         first = 0
         last = 4
         result = app.undoLog(first, last, filter=txn_filter)
-        pfirst, plast, ppartition = self.checkAskPacket(storage_1_conn,
-            Packets.AskTIDs, decode=True)
+        pfirst, plast, ppartition = asked.pop().decode()
         self.assertEqual(pfirst, first)
         self.assertEqual(plast, last)
         self.assertEqual(ppartition, INVALID_PARTITION)
-        pfirst, plast, ppartition = self.checkAskPacket(storage_2_conn,
-            Packets.AskTIDs, decode=True)
+        pfirst, plast, ppartition = asked.pop().decode()
         self.assertEqual(pfirst, first)
         self.assertEqual(plast, last)
         self.assertEqual(ppartition, INVALID_PARTITION)
         self.assertEqual(result[0]['id'], tid1)
         self.assertEqual(result[1]['id'], tid2)
+        self.assertFalse(asked)
 
     def test_connectToPrimaryNode(self):
         # here we have three master nodes :
@@ -784,25 +755,26 @@ class ClientApplicationTests(NeoUnitTestBase):
         # the third will not be ready
         # after the third, the partition table will be operational
         # (as if it was connected to the primary master node)
-        from neo.tests import DoNothingConnector
+        from .. import DoNothingConnector
         # will raise IndexError at the third iteration
         app = self.getApp('127.0.0.1:10010 127.0.0.1:10011')
         # TODO: test more connection failure cases
-        # Seventh packet : askNodeInformation succeeded
         all_passed = []
-        def _ask8(_):
+        # askLastTransaction
+        def _ask9(_):
             all_passed.append(1)
+        # Seventh packet : askNodeInformation succeeded
+        def _ask8(_):
+            pass
         # Sixth packet : askPartitionTable succeeded
         def _ask7(_):
             app.pt = Mock({'operational': True})
         # fifth packet : request node identification succeeded
         def _ask6(conn):
-            conn.setUUID('D' * 16)
-            app.uuid = 'C' * 16
-        # fourth iteration : connection to primary master succeeded
-        def _ask5(_):
+            app.master_conn = conn
+            app.uuid = 1 + (UUID_NAMESPACES[NodeTypes.CLIENT] << 24)
             app.trying_master_node = app.primary_master_node = Mock({
-                'getAddress': ('192.168.1.1', 10000),
+                'getAddress': ('127.0.0.1', 10011),
                 '__str__': 'Fake master node',
             })
         # third iteration : node not ready
@@ -811,19 +783,23 @@ class ClientApplicationTests(NeoUnitTestBase):
         # second iteration : master node changed
         def _ask3(_):
             app.primary_master_node = Mock({
-                'getAddress': ('192.168.1.1', 10000),
+                'getAddress': ('127.0.0.1', 10010),
                 '__str__': 'Fake master node',
             })
         # first iteration : connection failed
         def _ask2(_):
             app.trying_master_node = None
         # do nothing for the first call
+        # Case of an unknown primary_uuid (XXX: handler should probably raise,
+        # it's not normal for a node to inform of a primary uuid without
+        # telling us what its address is.)
         def _ask1(_):
             pass
-        ask_func_list = [_ask1, _ask2, _ask3, _ask4, _ask5, _ask6, _ask7,
-            _ask8]
+        ask_func_list = [_ask1, _ask2, _ask3, _ask4, _ask6, _ask7,
+            _ask8, _ask9]
         def _ask_base(conn, _, handler=None):
             ask_func_list.pop(0)(conn)
+            app.nm.getByAddress(conn.getAddress())._connection = None
         app._ask = _ask_base
         # faked environnement
         app.connector_handler = DoNothingConnector
