@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2009-2010  Nexedi SA
+# Copyright (C) 2009-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,19 +12,18 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import unittest
 from mock import Mock
-from neo.tests import NeoUnitTestBase
-from neo.lib.protocol import NodeTypes, LockState
+from .. import NeoUnitTestBase
+from neo.lib.protocol import NodeTypes
 from neo.client.handlers.storage import StorageBootstrapHandler, \
        StorageAnswersHandler
 from neo.client.exception import NEOStorageError, NEOStorageNotFoundError
 from neo.client.exception import NEOStorageDoesNotExistError
-from ZODB.POSException import ConflictError
 from neo.lib.exception import NodeNotReady
+from neo.lib.node import NodeManager
 from ZODB.TimeStamp import TimeStamp
 
 MARKER = []
@@ -32,56 +31,61 @@ MARKER = []
 class StorageBootstrapHandlerTests(NeoUnitTestBase):
 
     def setUp(self):
-        NeoUnitTestBase.setUp(self)
+        super(StorageBootstrapHandlerTests, self).setUp()
         self.app = Mock()
+        self.app.nm = NodeManager()
         self.handler = StorageBootstrapHandler(self.app)
+        self.app.primary_master_node = node = Mock({
+            'getConnection': self.getFakeConnection(),
+            'getUUID': self.getMasterUUID(),
+            'getAddress': (self.local_ip, 2999)
+        })
+        self._next_port = 3000
 
-    def getConnection(self):
-        return self.getFakeConnection()
+    def getKnownStorage(self):
+        node = self.app.nm.createStorage(
+            uuid=self.getStorageUUID(),
+            address=(self.local_ip, self._next_port),
+        )
+        self._next_port += 1
+        conn = self.getFakeConnection(address=node.getAddress(),
+            uuid=node.getUUID())
+        node.setConnection(conn)
+        return node, conn
 
     def test_notReady(self):
-        conn = self.getConnection()
+        conn = self.getFakeConnection()
         self.assertRaises(NodeNotReady, self.handler.notReady, conn, 'message')
 
     def test_acceptIdentification1(self):
         """ Not a storage node """
-        uuid = self.getNewUUID()
-        conn = self.getConnection()
-        conn = self.getConnection()
-        node = Mock()
-        self.app.nm = Mock({'getByAddress': node})
-        self.handler.acceptIdentification(conn, NodeTypes.CLIENT, uuid,
-            10, 0, None)
+        node, conn = self.getKnownStorage()
+        self.handler.acceptIdentification(conn, NodeTypes.CLIENT,
+            node.getUUID(),
+            10, 0, None, self.app.primary_master_node.getAddress(), [])
         self.checkClosed(conn)
 
     def test_acceptIdentification2(self):
-        uuid = self.getNewUUID()
-        conn = self.getConnection()
-        node = Mock()
-        self.app.nm = Mock({'getByAddress': node})
-        self.handler.acceptIdentification(conn, NodeTypes.STORAGE, uuid,
-            10, 0, None)
-        self.checkUUIDSet(node, uuid)
-        self.checkUUIDSet(conn, uuid)
-
+        node, conn = self.getKnownStorage()
+        self.handler.acceptIdentification(conn, NodeTypes.STORAGE,
+            node.getUUID(),
+            10, 0, None, self.app.primary_master_node.getAddress(), [])
+        self.checkNotClosed(conn)
 
 class StorageAnswerHandlerTests(NeoUnitTestBase):
 
     def setUp(self):
-        NeoUnitTestBase.setUp(self)
+        super(StorageAnswerHandlerTests, self).setUp()
         self.app = Mock()
         self.handler = StorageAnswersHandler(self.app)
 
-    def getConnection(self):
-        return self.getFakeConnection()
-
     def _checkHandlerData(self, ref):
         calls = self.app.mockGetNamedCalls('setHandlerData')
         self.assertEqual(len(calls), 1)
         calls[0].checkArgs(ref)
 
     def test_answerObject(self):
-        conn = self.getConnection()
+        conn = self.getFakeConnection()
         oid = self.getOID(0)
         tid1 = self.getNextTID()
         tid2 = self.getNextTID(tid1)
@@ -101,7 +105,7 @@ class StorageAnswerHandlerTests(NeoUnitTestBase):
         return StorageAnswersHandler(app)
 
     def test_answerStoreObject_1(self):
-        conn = self.getConnection()
+        conn = self.getFakeConnection()
         oid = self.getOID(0)
         tid = self.getNextTID()
         # conflict
@@ -120,7 +124,7 @@ class StorageAnswerHandlerTests(NeoUnitTestBase):
             conn, 1, oid, tid)
 
     def test_answerStoreObject_2(self):
-        conn = self.getConnection()
+        conn = self.getFakeConnection()
         oid = self.getOID(0)
         tid = self.getNextTID()
         tid_2 = self.getNextTID()
@@ -144,7 +148,7 @@ class StorageAnswerHandlerTests(NeoUnitTestBase):
             ).answerStoreObject(conn, 1, oid, tid_2)
 
     def test_answerStoreObject_3(self):
-        conn = self.getConnection()
+        conn = self.getFakeConnection()
         oid = self.getOID(0)
         tid = self.getNextTID()
         tid_2 = self.getNextTID()
@@ -167,7 +171,7 @@ class StorageAnswerHandlerTests(NeoUnitTestBase):
             {oid: set([tid, ])}).answerStoreObject(conn, 1, oid, tid_2)
 
     def test_answerStoreObject_4(self):
-        uuid = self.getNewUUID()
+        uuid = self.getStorageUUID()
         conn = self.getFakeConnection(uuid=uuid)
         oid = self.getOID(0)
         tid = self.getNextTID()
@@ -184,7 +188,7 @@ class StorageAnswerHandlerTests(NeoUnitTestBase):
         self.assertEqual(object_stored_counter_dict[oid], {tid: set([uuid])})
 
     def test_answerTransactionInformation(self):
-        conn = self.getConnection()
+        conn = self.getFakeConnection()
         tid = self.getNextTID()
         user = 'USER'
         desc = 'DESC'
@@ -203,37 +207,32 @@ class StorageAnswerHandlerTests(NeoUnitTestBase):
         }, ext))
 
     def test_oidNotFound(self):
-        conn = self.getConnection()
+        conn = self.getFakeConnection()
         self.assertRaises(NEOStorageNotFoundError, self.handler.oidNotFound,
             conn, 'message')
 
     def test_oidDoesNotExist(self):
-        conn = self.getConnection()
+        conn = self.getFakeConnection()
         self.assertRaises(NEOStorageDoesNotExistError,
             self.handler.oidDoesNotExist, conn, 'message')
 
     def test_tidNotFound(self):
-        conn = self.getConnection()
+        conn = self.getFakeConnection()
         self.assertRaises(NEOStorageNotFoundError, self.handler.tidNotFound,
             conn, 'message')
 
     def test_answerTIDs(self):
-        uuid = self.getNewUUID()
+        uuid = self.getStorageUUID()
         tid1 = self.getNextTID()
         tid2 = self.getNextTID(tid1)
         tid_list = [tid1, tid2]
         conn = self.getFakeConnection(uuid=uuid)
         tid_set = set()
-        app = Mock({
-            'getHandlerData': tid_set,
-        })
-        handler = StorageAnswersHandler(app)
-
-        handler.answerTIDs(conn, tid_list)
+        StorageAnswersHandler(Mock()).answerTIDs(conn, tid_list, tid_set)
         self.assertEqual(tid_set, set(tid_list))
 
     def test_answerObjectUndoSerial(self):
-        uuid = self.getNewUUID()
+        uuid = self.getStorageUUID()
         conn = self.getFakeConnection(uuid=uuid)
         oid1 = self.getOID(1)
         oid2 = self.getOID(2)
@@ -242,13 +241,10 @@ class StorageAnswerHandlerTests(NeoUnitTestBase):
         tid2 = self.getNextTID()
         tid3 = self.getNextTID()
         undo_dict = {}
-        app = Mock({
-            'getHandlerData': undo_dict,
-        })
-        handler = StorageAnswersHandler(app)
-        handler.answerObjectUndoSerial(conn, {oid1: [tid0, tid1]})
+        handler = StorageAnswersHandler(Mock())
+        handler.answerObjectUndoSerial(conn, {oid1: [tid0, tid1]}, undo_dict)
         self.assertEqual(undo_dict, {oid1: [tid0, tid1]})
-        handler.answerObjectUndoSerial(conn, {oid2: [tid2, tid3]})
+        handler.answerObjectUndoSerial(conn, {oid2: [tid2, tid3]}, undo_dict)
         self.assertEqual(undo_dict, {
             oid1: [tid0, tid1],
             oid2: [tid2, tid3],
