@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2009-2010  Nexedi SA
+# Copyright (C) 2009-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import errno
 import os
@@ -25,6 +24,7 @@ import signal
 import random
 import weakref
 import MySQLdb
+import sqlite3
 import unittest
 import tempfile
 import traceback
@@ -33,17 +33,21 @@ import psutil
 
 import neo.scripts
 from neo.neoctl.neoctl import NeoCTL, NotReadyException
-from neo.lib import setupLog
-from neo.lib.protocol import ClusterStates, NodeTypes, CellStates, NodeStates
+from neo.lib import logging
+from neo.lib.protocol import ClusterStates, NodeTypes, CellStates, NodeStates, \
+    UUID_NAMESPACES
 from neo.lib.util import dump
-from neo.tests import DB_USER, setupMySQLdb, NeoTestBase, buildUrlFromString, \
+from .. import DB_USER, setupMySQLdb, NeoTestBase, buildUrlFromString, \
         ADDRESS_TYPE, IP_VERSION_FORMAT_DICT, getTempDirectory
-from neo.tests.cluster import SocketLock
+from ..cluster import SocketLock
 from neo.client.Storage import Storage
+from neo.storage.database import buildDatabaseManager
 
-NEO_MASTER = 'neomaster'
-NEO_STORAGE = 'neostorage'
-NEO_ADMIN = 'neoadmin'
+command_dict = {
+    NodeTypes.MASTER: 'neomaster',
+    NodeTypes.STORAGE: 'neostorage',
+    NodeTypes.ADMIN: 'neoadmin',
+}
 
 DELAY_SAFETY_MARGIN = 10
 MAX_START_TIME = 30
@@ -67,6 +71,10 @@ class PortAllocator(object):
 
     def __init__(self):
         self.socket_list = []
+        # WKRD: this is a workaround for a weird bug allowing the same port to
+        # be bound more than once, causing later failure in tests, when
+        # different processes try to bind to the same port.
+        self.sock_port_set = set()
 
     def allocate(self, address_type, local_ip):
         s = socket.socket(address_type, socket.SOCK_STREAM)
@@ -75,62 +83,40 @@ class PortAllocator(object):
             self.lock.acquire()
         self.allocator_set[self] = None
         self.socket_list.append(s)
+        sock_port_set = self.sock_port_set
         while True:
             # Do not let the system choose the port to avoid conflicts
             # with other software. IOW, use a range different than:
             # - /proc/sys/net/ipv4/ip_local_port_range on Linux
             # - what IANA recommends (49152 to 65535)
+            port = random.randint(16384, 32767)
             try:
-                s.bind((local_ip, random.randint(16384, 32767)))
-                return s.getsockname()[1]
+                s.bind((local_ip, port))
             except socket.error, e:
-              if e.errno != errno.EADDRINUSE:
-                raise
+                if e.errno != errno.EADDRINUSE:
+                    raise
+            else:
+                if port not in sock_port_set:
+                    sock_port_set.add(port)
+                    return port
+                logging.warning('Same port allocated twice: %s in %s',
+                    port, sock_port_set)
 
     def release(self):
         for s in self.socket_list:
             s.close()
-        self.socket_list = None
+        self.__init__()
 
     def reset(self):
         if self.lock.locked():
             self.allocator_set.pop(self, None)
             if not self.allocator_set:
                 self.lock.release()
-            if self.socket_list:
-                for s in self.socket_list:
-                    s.close()
-            self.__init__()
+            self.release()
 
     __del__ = reset
 
 
-class ChildException(KeyboardInterrupt):
-    """Wrap any exception into an exception that is not catched by TestCase.run
-
-    The exception is not wrapped and re-raised immediately if there is no need
-    to wrap.
-    """
-
-    def __init__(self, type, value, tb):
-        code = unittest.TestCase.run.im_func.func_code
-        f = tb.tb_frame
-        while f is not None:
-            if f.f_code is code:
-                break
-            f = f.f_back
-        else:
-            raise type, value, tb
-        KeyboardInterrupt.__init__(self, type, value, tb)
-
-    def __call__(self):
-        """Re-raise wrapped exception"""
-        type, value, tb = self.args
-        if type is KeyboardInterrupt:
-          sys.exit(1)
-        raise type, value, tb
-
-
 class NEOProcess(object):
     pid = 0
 
@@ -140,7 +126,7 @@ class NEOProcess(object):
         except ImportError:
             raise NotFound, '%s not found' % (command)
         self.command = command
-        self.arg_dict = arg_dict
+        self.arg_dict = dict(('--' + k, v) for k, v in arg_dict.iteritems())
         self.with_uuid = True
         self.setUUID(uuid)
 
@@ -160,23 +146,39 @@ class NEOProcess(object):
         self.pid = os.fork()
         if self.pid == 0:
             # Child
-            # prevent child from killing anything
-            del self.__class__.__del__
             try:
+                # release SQLite debug log
+                logging.setup()
                 # release system-wide lock
                 for allocator in PortAllocator.allocator_set.copy():
                     allocator.reset()
                 sys.argv = [command] + args
                 getattr(neo.scripts,  command).main()
-                sys.exit()
+                status = 0
+            except SystemExit, e:
+                status = e.code
+                if status is None:
+                    status = 0
+            except KeyboardInterrupt:
+                status = 1
             except:
-                raise ChildException(*sys.exc_info())
-        neo.lib.logging.info('pid %u: %s %s',
-                             self.pid, command, ' '.join(map(repr, args)))
+                status = -1
+                traceback.print_exc()
+            finally:
+                # prevent child from killing anything (cf __del__), or
+                # running any other cleanup code normally done by the parent
+                try:
+                    os._exit(status)
+                except:
+                    print >>sys.stderr, status
+                finally:
+                    os._exit(1)
+        logging.info('pid %u: %s %s',
+            self.pid, command, ' '.join(map(repr, args)))
 
     def kill(self, sig=signal.SIGTERM):
         if self.pid:
-            neo.lib.logging.info('kill pid %u', self.pid)
+            logging.info('kill pid %u', self.pid)
             try:
                 pdb.kill(self.pid, sig)
             except OSError:
@@ -221,7 +223,7 @@ class NEOProcess(object):
           Note: for this change to take effect, the node must be restarted.
         """
         self.uuid = uuid
-        self.arg_dict['--uuid'] = dump(uuid)
+        self.arg_dict['--uuid'] = str(uuid)
 
     def isAlive(self):
         try:
@@ -232,93 +234,88 @@ class NEOProcess(object):
 class NEOCluster(object):
 
     def __init__(self, db_list, master_count=1, partitions=1, replicas=0,
-                 db_user=DB_USER, db_password='',
+                 db_user=DB_USER, db_password='', name=None,
                  cleanup_on_delete=False, temp_dir=None, clear_databases=True,
                  adapter=os.getenv('NEO_TESTS_ADAPTER'),
-                 verbose=True,
-                 address_type=ADDRESS_TYPE,
+                 address_type=ADDRESS_TYPE, bind_ip=None, logger=True,
         ):
         if not adapter:
             adapter = 'MySQL'
         self.adapter = adapter
         self.zodb_storage_list = []
         self.cleanup_on_delete = cleanup_on_delete
-        self.verbose = verbose
-        self.uuid_set = set()
-        self.db_user = db_user
-        self.db_password = db_password
+        self.uuid_dict = {}
         self.db_list = db_list
-        self.address_type = address_type
-        self.local_ip = local_ip = IP_VERSION_FORMAT_DICT[self.address_type]
-        self.setupDB(clear_databases)
-        self.process_dict = {}
         if temp_dir is None:
             temp_dir = tempfile.mkdtemp(prefix='neo_')
-            print 'Using temp directory %r.' % (temp_dir, )
+            print 'Using temp directory ' + temp_dir
+        if adapter == 'MySQL':
+            self.db_user = db_user
+            self.db_password = db_password
+            self.db_template = ('%s:%s@%%s' % (db_user, db_password)).__mod__
+        elif adapter == 'SQLite':
+            self.db_template = (lambda t: lambda db:
+                ':memory:' if db is None else db if os.sep in db else t % db
+                )(os.path.join(temp_dir, '%s.sqlite'))
+        else:
+            assert False, adapter
+        self.address_type = address_type
+        self.local_ip = local_ip = bind_ip or \
+            IP_VERSION_FORMAT_DICT[self.address_type]
+        self.setupDB(clear_databases)
+        self.process_dict = {}
         self.temp_dir = temp_dir
         self.port_allocator = PortAllocator()
         admin_port = self.port_allocator.allocate(address_type, local_ip)
-        self.cluster_name = 'neo_%s' % (random.randint(0, 100), )
+        self.cluster_name = name or 'neo_%s' % random.randint(0, 100)
         master_node_list = [self.port_allocator.allocate(address_type, local_ip)
                             for i in xrange(master_count)]
-        self.master_nodes = '/'.join('%s:%s' % (
+        self.master_nodes = ' '.join('%s:%s' % (
                 buildUrlFromString(self.local_ip), x, )
                 for x in master_node_list)
-
         # create admin node
-        self.__newProcess(NEO_ADMIN, {
-            '--cluster': self.cluster_name,
-            '--name': 'admin',
-            '--bind': '%s:%d' % (buildUrlFromString(
-                      self.local_ip), admin_port, ),
-            '--masters': self.master_nodes,
-        })
+        self._newProcess(NodeTypes.ADMIN, logger and 'admin', admin_port)
         # create master nodes
-        for index, port in enumerate(master_node_list):
-            self.__newProcess(NEO_MASTER, {
-                '--cluster': self.cluster_name,
-                '--name': 'master_%d' % index,
-                '--bind': '%s:%d' % (buildUrlFromString(
-                          self.local_ip), port, ),
-                '--masters': self.master_nodes,
-                '--replicas': replicas,
-                '--partitions': partitions,
-            })
+        for i, port in enumerate(master_node_list):
+            self._newProcess(NodeTypes.MASTER, logger and 'master_%u' % i,
+                             port, partitions=partitions, replicas=replicas)
         # create storage nodes
-        for index, db in enumerate(db_list):
-            self.__newProcess(NEO_STORAGE, {
-                '--cluster': self.cluster_name,
-                '--name': 'storage_%d' % index,
-                '--bind': '%s:%d' % (buildUrlFromString(
-                                        self.local_ip),
-                                        0 ),
-                '--masters': self.master_nodes,
-                '--database': '%s:%s@%s' % (db_user, db_password, db),
-                '--adapter': adapter,
-            })
+        for i, db in enumerate(db_list):
+            self._newProcess(NodeTypes.STORAGE, logger and 'storage_%u' % i,
+                             0, adapter=adapter, database=self.db_template(db))
         # create neoctl
         self.neoctl = NeoCTL((self.local_ip, admin_port))
 
-    def __newProcess(self, command, arguments):
-        uuid = self.__allocateUUID()
-        arguments['--uuid'] = uuid
-        if self.verbose:
-            arguments['--verbose'] = True
-        logfile = arguments['--name']
-        arguments['--logfile'] = os.path.join(self.temp_dir, '%s.log' % (logfile, ))
-
-        self.process_dict.setdefault(command, []).append(
-            NEOProcess(command, uuid, arguments))
-
-    def __allocateUUID(self):
-        uuid = ('%032x' % random.getrandbits(128)).decode('hex')
-        self.uuid_set.add(uuid)
-        return uuid
+    def _newProcess(self, node_type, logfile=None, port=None, **kw):
+        self.uuid_dict[node_type] = uuid = 1 + self.uuid_dict.get(node_type, 0)
+        uuid += UUID_NAMESPACES[node_type] << 24
+        kw['uuid'] = uuid
+        kw['cluster'] = self.cluster_name
+        kw['masters'] = self.master_nodes
+        if logfile:
+            kw['logfile'] = os.path.join(self.temp_dir, logfile + '.log')
+        if port is not None:
+            kw['bind'] = '%s:%u' % (buildUrlFromString(self.local_ip), port)
+        self.process_dict.setdefault(node_type, []).append(
+            NEOProcess(command_dict[node_type], uuid, kw))
 
     def setupDB(self, clear_databases=True):
         if self.adapter == 'MySQL':
             setupMySQLdb(self.db_list, self.db_user, self.db_password,
                          clear_databases)
+        elif self.adapter == 'SQLite':
+            if clear_databases:
+                for db in self.db_list:
+                    if db is None:
+                        continue
+                    db = self.db_template(db)
+                    try:
+                        os.remove(db)
+                    except OSError, e:
+                        if e.errno != errno.ENOENT:
+                            raise
+                    else:
+                        logging.debug('%r deleted', db)
 
     def run(self, except_storages=()):
         """ Start cluster processes except some storage nodes """
@@ -343,33 +340,46 @@ class NEOCluster(object):
         """ Do a complete start of a cluster """
         self.run(except_storages=except_storages)
         neoctl = self.neoctl
-        neoctl.startCluster()
-        target_count = len(self.db_list) - len(except_storages)
-        storage_node_list = []
+        target = [len(self.db_list) - len(except_storages)]
         def test():
-            storage_node_list[:] = neoctl.getNodeList(
-                node_type=NodeTypes.STORAGE)
-            # wait at least number of started storages, admin node can know
-            # more nodes when the cluster restart with an existing partition
-            # table referencing non-running nodes
-            return len(storage_node_list) >= target_count
+            try:
+                state = neoctl.getClusterState()
+                if state == ClusterStates.RUNNING:
+                    return True
+                if state == ClusterStates.RECOVERING and target[0]:
+                    pending_count = 0
+                    for x in neoctl.getNodeList(node_type=NodeTypes.STORAGE):
+                        if x[3] != NodeStates.PENDING:
+                            target[0] = None # cluster must start automatically
+                            break
+                        pending_count += 1
+                    if pending_count == target[0]:
+                        neoctl.startCluster()
+            except (NotReadyException, RuntimeError):
+                pass
         if not pdb.wait(test, MAX_START_TIME):
             raise AssertionError('Timeout when starting cluster')
-        if storage_node_list:
-            self.expectClusterRunning()
-            neoctl.enableStorageList([x[2] for x in storage_node_list])
 
     def stop(self, clients=True):
-        error_list = []
+        # Suspend all processes to kill before actually killing them, so that
+        # nodes don't log errors because they get disconnected from other nodes:
+        # otherwise, storage nodes would often flush MB of logs just because we
+        # killed the master first, and waste much file system space.
+        stopped_list = []
         for process_list in self.process_dict.itervalues():
             for process in process_list:
                 try:
-                    process.kill(signal.SIGKILL)
-                    process.wait()
+                    process.kill(signal.SIGSTOP)
+                    stopped_list.append(process)
                 except AlreadyStopped:
                     pass
-                except NodeProcessError, e:
-                    error_list += e.args
+        error_list = []
+        for process in stopped_list:
+            try:
+                process.kill(signal.SIGKILL)
+                process.wait()
+            except NodeProcessError, e:
+                error_list += e.args
         if clients:
             for zodb_storage in self.zodb_storage_list:
                 zodb_storage.close()
@@ -378,6 +388,14 @@ class NEOCluster(object):
         if error_list:
             raise NodeProcessError('\n'.join(error_list))
 
+    def waitAll(self):
+        for process_list in self.process_dict.itervalues():
+            for process in process_list:
+                try:
+                    process.wait()
+                except (AlreadyStopped, NodeProcessError):
+                    pass
+
     def getNEOCTL(self):
         return self.neoctl
 
@@ -386,8 +404,6 @@ class NEOCluster(object):
         result = Storage(
             master_nodes=master_nodes,
             name=self.cluster_name,
-            logfile=os.path.join(self.temp_dir, 'client.log'),
-            verbose=self.verbose,
             **kw)
         self.zodb_storage_list.append(result)
         return result
@@ -397,24 +413,18 @@ class NEOCluster(object):
         db = ZODB.DB(storage=self.getZODBStorage(**kw))
         return (db, db.open())
 
-    def getSQLConnection(self, db, autocommit=False):
-        assert db in self.db_list
-        conn = MySQLdb.Connect(user=self.db_user, passwd=self.db_password,
-                               db=db)
-        conn.autocommit(autocommit)
-        return conn
-
-    def _getProcessList(self, type):
-        return self.process_dict.get(type)
+    def getSQLConnection(self, db):
+        assert db is not None and db in self.db_list
+        return buildDatabaseManager(self.adapter, (self.db_template(db),))
 
     def getMasterProcessList(self):
-        return self._getProcessList(NEO_MASTER)
+        return self.process_dict.get(NodeTypes.MASTER)
 
     def getStorageProcessList(self):
-        return self._getProcessList(NEO_STORAGE)
+        return self.process_dict.get(NodeTypes.STORAGE)
 
     def getAdminProcessList(self):
-        return self._getProcessList(NEO_ADMIN)
+        return self.process_dict.get(NodeTypes.ADMIN)
 
     def _killMaster(self, primary=False, all=False):
         killed_uuid_list = []
@@ -500,7 +510,10 @@ class NEOCluster(object):
 
     def expectAllMasters(self, node_count, state=None, *args, **kw):
         def callback(last_try):
-            current_try = len(self.getMasterList(state=state))
+            try:
+                current_try = len(self.getMasterList(state=state))
+            except NotReadyException:
+                current_try = 0
             if last_try is not None and current_try < last_try:
                 raise AssertionError, 'Regression: %s became %s' % \
                     (last_try, current_try)
@@ -511,7 +524,10 @@ class NEOCluster(object):
         if not isinstance(state, (tuple, list)):
             state = (state, )
         def callback(last_try):
-            current_try = self.__getNodeState(node_type, uuid)
+            try:
+                current_try = self.__getNodeState(node_type, uuid)
+            except NotReadyException:
+                current_try = None
             return current_try in state, current_try
         self.expectCondition(callback, *args, **kw)
 
@@ -570,7 +586,10 @@ class NEOCluster(object):
 
     def expectClusterState(self, state, *args, **kw):
         def callback(last_try):
-            current_try = self.neoctl.getClusterState()
+            try:
+                current_try = self.neoctl.getClusterState()
+            except NotReadyException:
+                current_try = None
             return current_try == state, current_try
         self.expectCondition(callback, *args, **kw)
 
@@ -589,6 +608,12 @@ class NEOCluster(object):
             return current_try, current_try
         self.expectCondition(callback, *args, **kw)
 
+    def expectDead(self, process, *args, **kw):
+        def callback(last_try):
+            current_try = not process.isAlive()
+            return current_try, current_try
+        self.expectCondition(callback, *args, **kw)
+
     def expectStorageNotKnown(self, process, *args, **kw):
         # /!\ Not Known != Unknown
         process_uuid = process.getUUID()
@@ -607,8 +632,7 @@ class NEOCluster(object):
 class NEOFunctionalTest(NeoTestBase):
 
     def setupLog(self):
-        log_file = os.path.join(self.getTempDirectory(), 'test.log')
-        setupLog('TEST', log_file, True)
+        logging.setup(os.path.join(self.getTempDirectory(), 'test.log'))
 
     def getTempDirectory(self):
         # build the full path based on test case and current test method
@@ -618,12 +642,6 @@ class NEOFunctionalTest(NeoTestBase):
             os.makedirs(temp_dir)
         return temp_dir
 
-    def run(self, *args, **kw):
-        try:
-            return super(NEOFunctionalTest, self).run(*args, **kw)
-        except ChildException, e:
-            e()
-
     def runWithTimeout(self, timeout, method, args=(), kwargs=None):
         if kwargs is None:
             kwargs = {}
@@ -634,7 +652,7 @@ class NEOFunctionalTest(NeoTestBase):
             except:
                 exc_list.append(sys.exc_info())
         thread = threading.Thread(None, excWrapper, args=args, kwargs=kwargs)
-        thread.setDaemon(True)
+        thread.daemon = True
         thread.start()
         thread.join(timeout)
         self.assertFalse(thread.isAlive(), 'Run timeout')
