@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2009-2010  Nexedi SA
+# Copyright (C) 2009-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import os
 import unittest
@@ -21,15 +20,13 @@ import transaction
 import ZODB
 import socket
 
-from struct import pack, unpack
+from struct import pack
 from neo.neoctl.neoctl import NeoCTL
 from ZODB.FileStorage import FileStorage
 from ZODB.POSException import ConflictError
 from ZODB.tests.StorageTestBase import zodb_pickle
 from persistent import Persistent
-from neo.lib.util import SOCKET_CONNECTORS_DICT
-from neo.tests.functional import NEOCluster, NEOFunctionalTest
-from neo.tests import IP_VERSION_FORMAT_DICT
+from . import NEOCluster, NEOFunctionalTest
 
 TREE_SIZE = 6
 
@@ -77,10 +74,10 @@ class ClientTests(NEOFunctionalTest):
             temp_dir=self.getTempDirectory()
         )
 
-    def tearDown(self):
+    def _tearDown(self, success):
         if self.neo is not None:
             self.neo.stop()
-        NEOFunctionalTest.tearDown(self)
+        NEOFunctionalTest._tearDown(self, success)
 
     def __setup(self):
         # start cluster
@@ -234,6 +231,9 @@ class ClientTests(NEOFunctionalTest):
                 temp_dir=self.getTempDirectory())
             neoctl = self.neo.getNEOCTL()
             self.neo.start()
+            # BUG: The following 2 lines creates 2 app, i.e. 2 TCP connections
+            #      to the storage, so there may be a race condition at network
+            #      level and 'st2.store' may be effective before 'st1.store'.
             db1, conn1 = self.neo.getZODBConnection()
             db2, conn2 = self.neo.getZODBConnection()
             st1, st2 = conn1._storage, conn2._storage
