@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2009-2010  Nexedi SA
+# Copyright (C) 2009-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,19 +12,16 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import time
 import unittest
 import transaction
 from persistent import Persistent
 
-from neo.tests.functional import NEOCluster, NEOFunctionalTest
+from . import NEOCluster, NEOFunctionalTest
 from neo.lib.protocol import ClusterStates, NodeStates
 from ZODB.tests.StorageTestBase import zodb_pickle
-from MySQLdb import ProgrammingError
-from MySQLdb.constants.ER import NO_SUCH_TABLE
 
 class PObject(Persistent):
 
@@ -40,15 +37,10 @@ class StorageTests(NEOFunctionalTest):
         NEOFunctionalTest.setUp(self)
         self.neo = None
 
-    def tearDown(self):
+    def _tearDown(self, success):
         if self.neo is not None:
             self.neo.stop()
-        NEOFunctionalTest.tearDown(self)
-
-    def queryCount(self, db, query):
-        db.query(query)
-        result = db.store_result().fetch_row()[0][0]
-        return result
+        NEOFunctionalTest._tearDown(self, success)
 
     def __setup(self, storage_number=2, pending_number=0, replicas=1,
             partitions=10, master_count=2):
@@ -58,7 +50,6 @@ class StorageTests(NEOFunctionalTest):
             partitions=partitions, replicas=replicas,
             temp_dir=self.getTempDirectory(),
             clear_databases=True,
-            adapter='MySQL',
         )
         # too many pending storage nodes requested
         assert pending_number <= storage_number
@@ -80,21 +71,19 @@ class StorageTests(NEOFunctionalTest):
         db.close()
 
     def __checkDatabase(self, db_name):
-        db = self.neo.getSQLConnection(db_name, autocommit=True)
+        db = self.neo.getSQLConnection(db_name)
         # wait for the sql transaction to be commited
         def callback(last_try):
-            object_number = self.queryCount(db, 'select count(*) from obj')
+            # One revision per object and two for the root, before and after
+            (object_number,), = db.query('select count(*) from obj')
             return object_number == OBJECT_NUMBER + 2, object_number
         self.neo.expectCondition(callback)
         # no more temporarily objects
-        t_objects = self.queryCount(db, 'select count(*) from tobj')
+        (t_objects,), = db.query('select count(*) from tobj')
         self.assertEqual(t_objects, 0)
-        # One revision per object and two for the root, before and after
-        revisions = self.queryCount(db, 'select count(*) from obj')
-        self.assertEqual(revisions, OBJECT_NUMBER + 2)
         # One object more for the root
         query = 'select count(*) from (select * from obj group by oid) as t'
-        objects = self.queryCount(db, query)
+        (objects,), = db.query(query)
         self.assertEqual(objects, OBJECT_NUMBER + 1)
         # Check object content
         db, conn = self.neo.getZODBConnection()
@@ -121,22 +110,6 @@ class StorageTests(NEOFunctionalTest):
         storage_list = self.neo.getStorageList(NodeStates.RUNNING)
         self.assertEqual(len(storage_list), 2)
 
-    def __checkReplicateCount(self, db_name, target_count, timeout=0, delay=1):
-        db = self.neo.getSQLConnection(db_name, autocommit=True)
-        def callback(last_try):
-            try:
-                replicate_count = self.queryCount(db,
-                    'select count(distinct uuid) from pt')
-            except ProgrammingError, exc:
-                if exc[0] != NO_SUCH_TABLE:
-                    raise
-                replicate_count = 0
-            if last_try is not None and last_try < replicate_count:
-                raise AssertionError, 'Regression: %s became %s' % \
-                    (last_try, replicate_count)
-            return replicate_count == target_count, replicate_count
-        self.neo.expectCondition(callback, timeout, delay)
-
     def testNewNodesInPendingState(self):
         """ Check that new storage nodes are set as pending, the cluster remains
         running """
@@ -177,6 +150,7 @@ class StorageTests(NEOFunctionalTest):
         # add it to the partition table
         self.neo.neoctl.enableStorageList([stopped[0].getUUID()])
         self.neo.expectRunning(stopped[0])
+        self.neo.neoctl.tweakPartitionTable()
         self.neo.expectAssignedCells(stopped[0], number=10)
         self.neo.expectClusterRunning()
 
@@ -189,18 +163,27 @@ class StorageTests(NEOFunctionalTest):
         down, the cluster remains up since there is a replica """
 
         # populate the two storages
-        (started, _) = self.__setup(storage_number=2, replicas=1)
+        started, _ = self.__setup(partitions=3, replicas=1, storage_number=3)
         self.neo.expectRunning(started[0])
         self.neo.expectRunning(started[1])
+        self.neo.expectRunning(started[2])
         self.neo.expectOudatedCells(number=0)
-        self.__populate()
-        self.__checkReplicationDone()
+
+        self.neo.neoctl.killNode(started[0].getUUID())
+        # Cluster still operational. All cells of first storage should be
+        # outdated.
+        self.neo.expectUnavailable(started[0])
+        self.neo.expectOudatedCells(2)
         self.neo.expectClusterRunning()
 
-        # stop one storage and check outdated cells
-        started[0].stop()
-        self.neo.expectOudatedCells(number=10)
-        self.neo.expectClusterRunning()
+        self.assertRaises(RuntimeError, self.neo.neoctl.killNode,
+            started[1].getUUID())
+        started[1].stop()
+        # Cluster not operational anymore. Only cells of second storage that
+        # were shared with the third one should become outdated.
+        self.neo.expectUnavailable(started[1])
+        self.neo.expectClusterVerifying()
+        self.neo.expectOudatedCells(3)
 
     def testVerificationTriggered(self):
         """ Check that the verification stage is executed when a storage node
@@ -308,6 +291,7 @@ class StorageTests(NEOFunctionalTest):
         stopped[0].start()
         self.neo.expectPending(stopped[0])
         self.neo.neoctl.enableStorageList([stopped[0].getUUID()])
+        self.neo.neoctl.tweakPartitionTable()
         self.neo.expectRunning(stopped[0])
         self.neo.expectClusterRunning()
         self.neo.expectOudatedCells(number=0)
@@ -343,6 +327,9 @@ class StorageTests(NEOFunctionalTest):
         self.neo.expectStorageNotKnown(started[0])
         self.neo.expectAssignedCells(started[0], 0)
         self.neo.expectAssignedCells(started[1], 10)
+        self.assertRaises(RuntimeError, self.neo.neoctl.dropNode,
+                          started[1].getUUID())
+        self.neo.expectClusterRunning()
 
     def testReplicationThenRunningWithReplicas(self):
         """ Add a replicas to a cluster, wait for the replication to finish,
@@ -366,6 +353,7 @@ class StorageTests(NEOFunctionalTest):
         stopped[0].start()
         self.neo.expectPending(stopped[0])
         self.neo.neoctl.enableStorageList([stopped[0].getUUID()])
+        self.neo.neoctl.tweakPartitionTable()
         self.neo.expectRunning(stopped[0])
         self.neo.expectClusterRunning()
         self.neo.expectAssignedCells(started[0], 10)
@@ -400,68 +388,6 @@ class StorageTests(NEOFunctionalTest):
         self.__setup(storage_number=2, partitions=5000, master_count=1)
         self.neo.expectClusterState(ClusterStates.RUNNING)
 
-    def testDropNodeThenRestartCluster(self):
-        """ Start a cluster with more than one storage, down one, shutdown the
-        cluster then restart it. The partition table recovered must not include
-        the dropped node """
-
-        # start with two storage / one replica
-        (started, stopped) = self.__setup(storage_number=2, replicas=1,
-                master_count=1, partitions=10)
-        self.neo.expectRunning(started[0])
-        self.neo.expectRunning(started[1])
-        self.neo.expectOudatedCells(number=0)
-
-        # drop one
-        self.neo.neoctl.dropNode(started[0].getUUID())
-        self.neo.expectStorageNotKnown(started[0])
-        self.neo.expectRunning(started[1])
-
-        # wait for running storage to store new partition table
-        self.__checkReplicateCount(self.neo.db_list[1], 1)
-
-        # restart all nodes except the dropped, it must not be known
-        self.neo.stop()
-        self.neo.start(except_storages=[started[0]])
-        self.neo.expectStorageNotKnown(started[0])
-        self.neo.expectRunning(started[1])
-
-        # then restart it, it must be in pending state
-        started[0].start()
-        self.neo.expectPending(started[0])
-        self.neo.expectRunning(started[1])
-
-    def testAcceptFirstEmptyStorageAfterStartupAllowed(self):
-        """ Create a new cluster with no storage node, allow it to starts
-        then run the first empty storage, it must be accepted """
-        (started, stopped) = self.__setup(storage_number=1, replicas=0,
-                pending_number=1, partitions=10)
-        # start without storage
-        self.neo.expectClusterRecovering()
-        self.neo.expectStorageNotKnown(stopped[0])
-        # start the empty storage, it must be accepted
-        stopped[0].start(with_uuid=False)
-        self.neo.expectClusterRunning()
-        self.assertEqual(len(self.neo.getStorageList()), 1)
-        self.neo.expectOudatedCells(number=0)
-
-    def testDropNodeWithOtherPending(self):
-        """ Ensure we can drop a node """
-        # start with one storage
-        (started, stopped) = self.__setup(storage_number=2, replicas=1,
-                pending_number=1, partitions=10)
-        self.neo.expectRunning(started[0])
-        self.neo.expectStorageNotKnown(stopped[0])
-        self.neo.expectOudatedCells(number=0)
-        self.neo.expectClusterRunning()
-
-        # set the second storage in pending state and drop the first
-        stopped[0].start()
-        self.neo.expectPending(stopped[0])
-        self.neo.neoctl.dropNode(started[0].getUUID())
-        self.neo.expectStorageNotKnown(started[0])
-        self.neo.expectPending(stopped[0])
-
     def testRecoveryWithMultiplePT(self):
         # start a cluster with 2 storages and a replica
         (started, stopped) = self.__setup(storage_number=2, replicas=1,
@@ -486,15 +412,16 @@ class StorageTests(NEOFunctionalTest):
 
         # restart the cluster with the first storage killed
         self.neo.run(except_storages=[started[1]])
-        self.neo.expectRunning(started[0])
+        self.neo.expectPending(started[0])
         self.neo.expectUnknown(started[1])
         self.neo.expectClusterRecovering()
+        # Cluster doesn't know there are outdated cells
         self.neo.expectOudatedCells(number=0)
         started[1].start()
         self.neo.expectRunning(started[0])
         self.neo.expectRunning(started[1])
-        self.neo.expectClusterRecovering()
-        self.neo.expectOudatedCells(number=10)
+        self.neo.expectClusterRunning()
+        self.neo.expectOudatedCells(number=0)
 
     def testReplicationBlockedByUnfinished(self):
         # start a cluster with 1 of 2 storages and a replica
@@ -523,6 +450,7 @@ class StorageTests(NEOFunctionalTest):
         stopped[0].start()
         self.neo.expectPending(stopped[0])
         self.neo.neoctl.enableStorageList([stopped[0].getUUID()])
+        self.neo.neoctl.tweakPartitionTable()
         self.neo.expectRunning(stopped[0])
         self.neo.expectClusterRunning()
         self.neo.expectAssignedCells(started[0], 10)
