@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2009-2010  Nexedi SA
+# Copyright (C) 2009-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,16 +12,15 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import unittest
 from mock import Mock, ReturnValues
 from collections import deque
 from neo.lib.util import makeChecksum
-from neo.tests import NeoUnitTestBase
+from .. import NeoUnitTestBase
 from neo.storage.app import Application
-from neo.storage.transactions import ConflictError, DelayedError
+from neo.storage.transactions import ConflictError
 from neo.storage.handlers.client import ClientOperationHandler
 from neo.lib.protocol import INVALID_PARTITION, INVALID_TID, INVALID_OID
 from neo.lib.protocol import Packets, LockState, ZERO_HASH
@@ -50,16 +49,16 @@ class StorageClientHandlerTests(NeoUnitTestBase):
         # handler
         self.operation = ClientOperationHandler(self.app)
         # set pmn
-        self.master_uuid = self.getNewUUID()
+        self.master_uuid = self.getMasterUUID()
         pmn = self.app.nm.getMasterList()[0]
         pmn.setUUID(self.master_uuid)
         self.app.primary_master_node = pmn
         self.master_port = 10010
 
-    def tearDown(self):
+    def _tearDown(self, success):
         self.app.close()
         del self.app
-        super(StorageClientHandlerTests, self).tearDown()
+        super(StorageClientHandlerTests, self)._tearDown(success)
 
     def _getConnection(self, uuid=None):
         return self.getFakeConnection(uuid=uuid, address=('127.0.0.1', 1000))
@@ -70,7 +69,7 @@ class StorageClientHandlerTests(NeoUnitTestBase):
         calls[0].checkArgs(uuid)
 
     def test_connectionLost(self):
-        uuid = self.getNewUUID()
+        uuid = self.getClientUUID()
         self.app.nm.createClient(uuid=uuid)
         conn = self._getConnection(uuid=uuid)
         self.operation.connectionClosed(conn)
@@ -149,7 +148,7 @@ class StorageClientHandlerTests(NeoUnitTestBase):
         self.operation.askTIDs(conn, 1, 2, 1)
         calls = self.app.dm.mockGetNamedCalls('getTIDList')
         self.assertEqual(len(calls), 1)
-        calls[0].checkArgs(1, 1, 1, [1, ])
+        calls[0].checkArgs(1, 1, [1, ])
         self.checkAnswerTids(conn)
 
     def test_25_askTIDs3(self):
@@ -166,7 +165,7 @@ class StorageClientHandlerTests(NeoUnitTestBase):
         self.assertEqual(len(self.app.pt.mockGetNamedCalls('getAssignedPartitionList')), 1)
         calls = self.app.dm.mockGetNamedCalls('getTIDList')
         self.assertEqual(len(calls), 1)
-        calls[0].checkArgs(1, 1, 1, [0])
+        calls[0].checkArgs(1, 1, [0])
         self.checkAnswerTids(conn)
 
     def test_26_askObjectHistory1(self):
@@ -193,8 +192,7 @@ class StorageClientHandlerTests(NeoUnitTestBase):
         self.checkAnswerObjectHistory(conn)
 
     def test_askStoreTransaction(self):
-        uuid = self.getNewUUID()
-        conn = self._getConnection(uuid=uuid)
+        conn = self._getConnection(uuid=self.getClientUUID())
         tid = self.getNextTID()
         user = 'USER'
         desc = 'DESC'
@@ -218,8 +216,7 @@ class StorageClientHandlerTests(NeoUnitTestBase):
 
     def test_askStoreObject1(self):
         # no conflict => answer
-        uuid = self.getNewUUID()
-        conn = self._getConnection(uuid=uuid)
+        conn = self._getConnection(uuid=self.getClientUUID())
         tid = self.getNextTID()
         oid, serial, comp, checksum, data = self._getObject()
         self.operation.askStoreObject(conn, oid, serial, comp, checksum,
@@ -234,8 +231,7 @@ class StorageClientHandlerTests(NeoUnitTestBase):
 
     def test_askStoreObjectWithDataTID(self):
         # same as test_askStoreObject1, but with a non-None data_tid value
-        uuid = self.getNewUUID()
-        conn = self._getConnection(uuid=uuid)
+        conn = self._getConnection(uuid=self.getClientUUID())
         tid = self.getNextTID()
         oid, serial, comp, checksum, data = self._getObject()
         data_tid = self.getNextTID()
@@ -251,8 +247,7 @@ class StorageClientHandlerTests(NeoUnitTestBase):
 
     def test_askStoreObject2(self):
         # conflict error
-        uuid = self.getNewUUID()
-        conn = self._getConnection(uuid=uuid)
+        conn = self._getConnection(uuid=self.getClientUUID())
         tid = self.getNextTID()
         locking_tid = self.getNextTID(tid)
         def fakeStoreObject(*args):
@@ -276,8 +271,7 @@ class StorageClientHandlerTests(NeoUnitTestBase):
         calls[0].checkArgs(tid)
 
     def test_askObjectUndoSerial(self):
-        uuid = self.getNewUUID()
-        conn = self._getConnection(uuid=uuid)
+        conn = self._getConnection(uuid=self.getClientUUID())
         tid = self.getNextTID()
         ltid = self.getNextTID()
         undone_tid = self.getNextTID()
