@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2009-2010  Nexedi SA
+# Copyright (C) 2009-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,21 +12,19 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from binascii import a2b_hex
 import unittest
-from mock import Mock
-from neo.lib.util import dump, p64, u64
-from neo.lib.protocol import CellStates, ZERO_OID, ZERO_TID
-from neo.tests import NeoUnitTestBase
-from neo.lib.exception import DatabaseFailure
-from neo.storage.database.mysqldb import MySQLDatabaseManager
+from neo.lib.util import add64, p64, u64
+from neo.lib.protocol import CellStates, ZERO_HASH, ZERO_OID, ZERO_TID, MAX_TID
+from .. import NeoUnitTestBase
 
-MAX_TID = '\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFE' # != INVALID_TID
 
 class StorageDBTests(NeoUnitTestBase):
 
+    _last_ttid = ZERO_TID
+
     def setUp(self):
         NeoUnitTestBase.setUp(self)
 
@@ -38,14 +36,14 @@ class StorageDBTests(NeoUnitTestBase):
             self.setNumPartitions(1)
             return self._db
 
-    def tearDown(self):
+    def _tearDown(self, success):
         try:
             self.__dict__.pop('_db', None).close()
         except AttributeError:
             pass
-        NeoUnitTestBase.tearDown(self)
+        NeoUnitTestBase._tearDown(self, success)
 
-    def getDB(self):
+    def getDB(self, reset=0):
         raise NotImplementedError
 
     def setNumPartitions(self, num_partitions, reset=0):
@@ -64,10 +62,10 @@ class StorageDBTests(NeoUnitTestBase):
                 if num_partitions == n:
                     return
                 if num_partitions < n:
-                    db.dropPartitions(n, range(num_partitions, n))
+                    db.dropPartitions(n)
         db.setNumPartitions(num_partitions)
         self.assertEqual(num_partitions, db.getNumPartitions())
-        uuid = self.getNewUUID()
+        uuid = self.getStorageUUID()
         db.setUUID(uuid)
         self.assertEqual(uuid, db.getUUID())
         db.setPartitionTable(1,
@@ -75,7 +73,7 @@ class StorageDBTests(NeoUnitTestBase):
 
     def checkConfigEntry(self, get_call, set_call, value):
         # generic test for all configuration entries accessors
-        self.assertRaises(KeyError, get_call)
+        self.assertEqual(get_call(), None)
         set_call(value)
         self.assertEqual(get_call(), value)
         set_call(value * 2)
@@ -83,7 +81,7 @@ class StorageDBTests(NeoUnitTestBase):
 
     def test_UUID(self):
         db = self.getDB()
-        self.checkConfigEntry(db.getUUID, db.setUUID, 'TEST_VALUE')
+        self.checkConfigEntry(db.getUUID, db.setUUID, 123)
 
     def test_Name(self):
         db = self.getDB()
@@ -96,20 +94,13 @@ class StorageDBTests(NeoUnitTestBase):
     def test_getPartitionTable(self):
         db = self.getDB()
         ptid = self.getPTID(1)
-        uuid1, uuid2 = self.getNewUUID(), self.getNewUUID()
+        uuid1, uuid2 = self.getStorageUUID(), self.getStorageUUID()
         cell1 = (0, uuid1, CellStates.OUT_OF_DATE)
         cell2 = (1, uuid1, CellStates.UP_TO_DATE)
         db.setPartitionTable(ptid, [cell1, cell2])
         result = db.getPartitionTable()
         self.assertEqual(set(result), set([cell1, cell2]))
 
-    def test_getLastOID(self):
-        db = self.getDB()
-        oid1 = self.getOID(1)
-        db.setLastOID(oid1)
-        result1 = db.getLastOID()
-        self.assertEqual(result1, oid1)
-
     def getOIDs(self, count):
         return map(self.getOID, xrange(count))
 
@@ -120,31 +111,33 @@ class StorageDBTests(NeoUnitTestBase):
         return tid_list
 
     def getTransaction(self, oid_list):
-        transaction = (oid_list, 'user', 'desc', 'ext', False)
+        self._last_ttid = ttid = add64(self._last_ttid, 1)
+        transaction = oid_list, 'user', 'desc', 'ext', False, ttid
         H = "0" * 20
-        for _ in oid_list:
-            self.db.storeData(H, '', 1)
-        object_list = [(oid, H, None) for oid in oid_list]
+        object_list = [(oid, self.db.storeData(H, '', 1), None)
+                       for oid in oid_list]
         return (transaction, object_list)
 
     def checkSet(self, list1, list2):
         self.assertEqual(set(list1), set(list2))
 
-    def test_getLastTID(self):
+    def test_getLastIDs(self):
         tid1, tid2, tid3, tid4 = self.getTIDs(4)
         oid1, oid2 = self.getOIDs(2)
         txn, objs = self.getTransaction([oid1, oid2])
-        # max TID is in obj table
         self.db.storeTransaction(tid1, objs, txn, False)
         self.db.storeTransaction(tid2, objs, txn, False)
-        self.assertEqual(self.db.getLastTID(), tid2)
-        # max tid is in ttrans table
+        self.assertEqual(self.db.getLastIDs(),
+            (tid2, {0: tid2}, {0: tid2}, oid2))
         self.db.storeTransaction(tid3, objs, txn)
-        result = self.db.getLastTID()
-        self.assertEqual(self.db.getLastTID(), tid3)
-        # max tid is in tobj (serial)
+        tids = {0: tid2, None: tid3}
+        self.assertEqual(self.db.getLastIDs(), (tid3, tids, tids, oid2))
         self.db.storeTransaction(tid4, objs, None)
-        self.assertEqual(self.db.getLastTID(), tid4)
+        self.assertEqual(self.db.getLastIDs(),
+            (tid4, tids, {0: tid2, None: tid4}, oid2))
+        self.db.finishTransaction(tid3)
+        self.assertEqual(self.db.getLastIDs(),
+            (tid4, {0: tid3}, {0: tid3, None: tid4}, oid2))
 
     def test_getUnfinishedTIDList(self):
         tid1, tid2, tid3, tid4 = self.getTIDs(4)
@@ -225,38 +218,38 @@ class StorageDBTests(NeoUnitTestBase):
     def test_setPartitionTable(self):
         db = self.getDB()
         ptid = self.getPTID(1)
-        uuid1, uuid2 = self.getNewUUID(), self.getNewUUID()
-        cell1 = (0, uuid1, CellStates.OUT_OF_DATE)
-        cell2 = (1, uuid1, CellStates.UP_TO_DATE)
-        cell3 = (1, uuid1, CellStates.DISCARDED)
+        uuid = self.getStorageUUID()
+        cell1 = 0, uuid, CellStates.OUT_OF_DATE
+        cell2 = 1, uuid, CellStates.UP_TO_DATE
+        cell3 = 1, uuid, CellStates.DISCARDED
         # no partition table
-        self.assertEqual(db.getPartitionTable(), [])
+        self.assertEqual(list(db.getPartitionTable()), [])
         # set one
         db.setPartitionTable(ptid, [cell1])
         result = db.getPartitionTable()
-        self.assertEqual(result, [cell1])
+        self.assertEqual(list(result), [cell1])
         # then another
         db.setPartitionTable(ptid, [cell2])
         result = db.getPartitionTable()
-        self.assertEqual(result, [cell2])
+        self.assertEqual(list(result), [cell2])
         # drop discarded cells
         db.setPartitionTable(ptid, [cell2, cell3])
         result = db.getPartitionTable()
-        self.assertEqual(result, [])
+        self.assertEqual(list(result), [])
 
     def test_changePartitionTable(self):
         db = self.getDB()
         ptid = self.getPTID(1)
-        uuid1, uuid2 = self.getNewUUID(), self.getNewUUID()
-        cell1 = (0, uuid1, CellStates.OUT_OF_DATE)
-        cell2 = (1, uuid1, CellStates.UP_TO_DATE)
-        cell3 = (1, uuid1, CellStates.DISCARDED)
+        uuid = self.getStorageUUID()
+        cell1 = 0, uuid, CellStates.OUT_OF_DATE
+        cell2 = 1, uuid, CellStates.UP_TO_DATE
+        cell3 = 1, uuid, CellStates.DISCARDED
         # no partition table
-        self.assertEqual(db.getPartitionTable(), [])
+        self.assertEqual(list(db.getPartitionTable()), [])
         # set one
         db.changePartitionTable(ptid, [cell1])
         result = db.getPartitionTable()
-        self.assertEqual(result, [cell1])
+        self.assertEqual(list(result), [cell1])
         # add more entries
         db.changePartitionTable(ptid, [cell2])
         result = db.getPartitionTable()
@@ -264,7 +257,7 @@ class StorageDBTests(NeoUnitTestBase):
         # drop discarded cells
         db.changePartitionTable(ptid, [cell2, cell3])
         result = db.getPartitionTable()
-        self.assertEqual(result, [cell1])
+        self.assertEqual(list(result), [cell1])
 
     def test_dropUnfinishedData(self):
         oid1, oid2 = self.getOIDs(2)
@@ -296,7 +289,7 @@ class StorageDBTests(NeoUnitTestBase):
         txn1, objs1 = self.getTransaction([oid1])
         txn2, objs2 = self.getTransaction([oid2])
         # nothing in database
-        self.assertEqual(self.db.getLastTID(), None)
+        self.assertEqual(self.db.getLastIDs(), (None, {}, {}, None))
         self.assertEqual(self.db.getUnfinishedTIDList(), [])
         self.assertEqual(self.db.getObject(oid1), None)
         self.assertEqual(self.db.getObject(oid2), None)
@@ -308,22 +301,22 @@ class StorageDBTests(NeoUnitTestBase):
         self.db.storeTransaction(tid1, objs1, txn1)
         self.db.storeTransaction(tid2, objs2, txn2)
         result = self.db.getTransaction(tid1, True)
-        self.assertEqual(result, ([oid1], 'user', 'desc', 'ext', False))
+        self.assertEqual(result, ([oid1], 'user', 'desc', 'ext', False, p64(1)))
         result = self.db.getTransaction(tid2, True)
-        self.assertEqual(result, ([oid2], 'user', 'desc', 'ext', False))
+        self.assertEqual(result, ([oid2], 'user', 'desc', 'ext', False, p64(2)))
         self.assertEqual(self.db.getTransaction(tid1, False), None)
         self.assertEqual(self.db.getTransaction(tid2, False), None)
         # commit pending transaction
         self.db.finishTransaction(tid1)
         self.db.finishTransaction(tid2)
         result = self.db.getTransaction(tid1, True)
-        self.assertEqual(result, ([oid1], 'user', 'desc', 'ext', False))
+        self.assertEqual(result, ([oid1], 'user', 'desc', 'ext', False, p64(1)))
         result = self.db.getTransaction(tid2, True)
-        self.assertEqual(result, ([oid2], 'user', 'desc', 'ext', False))
+        self.assertEqual(result, ([oid2], 'user', 'desc', 'ext', False, p64(2)))
         result = self.db.getTransaction(tid1, False)
-        self.assertEqual(result, ([oid1], 'user', 'desc', 'ext', False))
+        self.assertEqual(result, ([oid1], 'user', 'desc', 'ext', False, p64(1)))
         result = self.db.getTransaction(tid2, False)
-        self.assertEqual(result, ([oid2], 'user', 'desc', 'ext', False))
+        self.assertEqual(result, ([oid2], 'user', 'desc', 'ext', False, p64(2)))
 
     def test_askFinishTransaction(self):
         oid1, oid2 = self.getOIDs(2)
@@ -334,22 +327,22 @@ class StorageDBTests(NeoUnitTestBase):
         self.db.storeTransaction(tid1, objs1, txn1)
         self.db.storeTransaction(tid2, objs2, txn2)
         result = self.db.getTransaction(tid1, True)
-        self.assertEqual(result, ([oid1], 'user', 'desc', 'ext', False))
+        self.assertEqual(result, ([oid1], 'user', 'desc', 'ext', False, p64(1)))
         result = self.db.getTransaction(tid2, True)
-        self.assertEqual(result, ([oid2], 'user', 'desc', 'ext', False))
+        self.assertEqual(result, ([oid2], 'user', 'desc', 'ext', False, p64(2)))
         self.assertEqual(self.db.getTransaction(tid1, False), None)
         self.assertEqual(self.db.getTransaction(tid2, False), None)
         # stored and finished
         self.db.finishTransaction(tid1)
         self.db.finishTransaction(tid2)
         result = self.db.getTransaction(tid1, True)
-        self.assertEqual(result, ([oid1], 'user', 'desc', 'ext', False))
+        self.assertEqual(result, ([oid1], 'user', 'desc', 'ext', False, p64(1)))
         result = self.db.getTransaction(tid2, True)
-        self.assertEqual(result, ([oid2], 'user', 'desc', 'ext', False))
+        self.assertEqual(result, ([oid2], 'user', 'desc', 'ext', False, p64(2)))
         result = self.db.getTransaction(tid1, False)
-        self.assertEqual(result, ([oid1], 'user', 'desc', 'ext', False))
+        self.assertEqual(result, ([oid1], 'user', 'desc', 'ext', False, p64(1)))
         result = self.db.getTransaction(tid2, False)
-        self.assertEqual(result, ([oid2], 'user', 'desc', 'ext', False))
+        self.assertEqual(result, ([oid2], 'user', 'desc', 'ext', False, p64(2)))
 
     def test_deleteTransaction(self):
         oid1, oid2 = self.getOIDs(2)
@@ -364,24 +357,6 @@ class StorageDBTests(NeoUnitTestBase):
         self.assertEqual(self.db.getTransaction(tid1, True), None)
         self.assertEqual(self.db.getTransaction(tid2, True), None)
 
-    def test_deleteTransactionsAbove(self):
-        self.setNumPartitions(2)
-        tid1 = self.getOID(0)
-        tid2 = self.getOID(1)
-        tid3 = self.getOID(2)
-        oid1 = self.getOID(1)
-        for tid in (tid1, tid2, tid3):
-            txn, objs = self.getTransaction([oid1])
-            self.db.storeTransaction(tid, objs, txn)
-            self.db.finishTransaction(tid)
-        self.db.deleteTransactionsAbove(2, 0, tid2, tid3)
-        # Right partition, below cutoff
-        self.assertNotEqual(self.db.getTransaction(tid1, True), None)
-        # Wrong partition, above cutoff
-        self.assertNotEqual(self.db.getTransaction(tid2, True), None)
-        # Right partition, above cutoff
-        self.assertEqual(self.db.getTransaction(tid3, True), None)
-
     def test_deleteObject(self):
         oid1, oid2 = self.getOIDs(2)
         tid1, tid2 = self.getTIDs(2)
@@ -399,34 +374,28 @@ class StorageDBTests(NeoUnitTestBase):
         self.assertEqual(self.db.getObject(oid2, tid=tid2),
             (tid2, None, 1, "0" * 20, '', None))
 
-    def test_deleteObjectsAbove(self):
-        self.setNumPartitions(2)
-        tid1 = self.getOID(1)
-        tid2 = self.getOID(2)
-        tid3 = self.getOID(3)
-        oid1 = self.getOID(0)
-        oid2 = self.getOID(1)
-        oid3 = self.getOID(2)
-        for tid in (tid1, tid2, tid3):
-            txn, objs = self.getTransaction([oid1, oid2, oid3])
+    def test_deleteRange(self):
+        np = 4
+        self.setNumPartitions(np)
+        t1, t2, t3 = map(self.getOID, (1, 2, 3))
+        oid_list = self.getOIDs(np * 2)
+        for tid in t1, t2, t3:
+            txn, objs = self.getTransaction(oid_list)
             self.db.storeTransaction(tid, objs, txn)
             self.db.finishTransaction(tid)
-        self.db.deleteObjectsAbove(2, 0, oid1, tid2, tid3)
-        # Check getObjectHistoryFrom because MySQL adapter use two tables
-        # that must be synchronized
-        self.assertEqual(self.db.getObjectHistoryFrom(ZERO_OID, ZERO_TID,
-            MAX_TID, 10, 2, 0), {oid1: [tid1]})
-        # Right partition, below cutoff
-        self.assertNotEqual(self.db.getObject(oid1, tid=tid1), None)
-        # Right partition, above tid cutoff
-        self.assertFalse(self.db.getObject(oid1, tid=tid2))
-        self.assertFalse(self.db.getObject(oid1, tid=tid3))
-        # Wrong partition, above cutoff
-        self.assertNotEqual(self.db.getObject(oid2, tid=tid1), None)
-        self.assertNotEqual(self.db.getObject(oid2, tid=tid2), None)
-        self.assertNotEqual(self.db.getObject(oid2, tid=tid3), None)
-        # Right partition, above cutoff
-        self.assertEqual(self.db.getObject(oid3), None)
+        def check(offset, tid_list, *tids):
+            self.assertEqual(self.db.getReplicationTIDList(ZERO_TID,
+                MAX_TID, len(tid_list) + 1, offset), tid_list)
+            expected = [(t, oid_list[offset+i]) for t in tids for i in (0, np)]
+            self.assertEqual(self.db.getReplicationObjectList(ZERO_TID,
+                MAX_TID, len(expected) + 1, offset, ZERO_OID), expected)
+        self.db._deleteRange(0, MAX_TID)
+        self.db._deleteRange(0, max_tid=ZERO_TID)
+        check(0, [], t1, t2, t3)
+        self.db._deleteRange(0);             check(0, [])
+        self.db._deleteRange(1, t2);         check(1, [t1], t1, t2)
+        self.db._deleteRange(2, max_tid=t2); check(2, [], t3)
+        self.db._deleteRange(3, t1, t2);     check(3, [t3], t1, t3)
 
     def test_getTransaction(self):
         oid1, oid2 = self.getOIDs(2)
@@ -438,12 +407,12 @@ class StorageDBTests(NeoUnitTestBase):
         self.db.storeTransaction(tid2, objs2, txn2)
         self.db.finishTransaction(tid1)
         result = self.db.getTransaction(tid1, True)
-        self.assertEqual(result, ([oid1], 'user', 'desc', 'ext', False))
+        self.assertEqual(result, ([oid1], 'user', 'desc', 'ext', False, p64(1)))
         result = self.db.getTransaction(tid2, True)
-        self.assertEqual(result, ([oid2], 'user', 'desc', 'ext', False))
+        self.assertEqual(result, ([oid2], 'user', 'desc', 'ext', False, p64(2)))
         # get from non-temporary only
         result = self.db.getTransaction(tid1, False)
-        self.assertEqual(result, ([oid1], 'user', 'desc', 'ext', False))
+        self.assertEqual(result, ([oid1], 'user', 'desc', 'ext', False, p64(1)))
         self.assertEqual(self.db.getTransaction(tid2, False), None)
 
     def test_getObjectHistory(self):
@@ -469,61 +438,6 @@ class StorageDBTests(NeoUnitTestBase):
         result = self.db.getObjectHistory(oid, 2, 3)
         self.assertEqual(result, None)
 
-    def test_getObjectHistoryFrom(self):
-        self.setNumPartitions(2)
-        oid1 = self.getOID(0)
-        oid2 = self.getOID(2)
-        oid3 = self.getOID(1)
-        tid1, tid2, tid3, tid4, tid5 = self.getTIDs(5)
-        txn1, objs1 = self.getTransaction([oid1])
-        txn2, objs2 = self.getTransaction([oid2])
-        txn3, objs3 = self.getTransaction([oid1])
-        txn4, objs4 = self.getTransaction([oid2])
-        txn5, objs5 = self.getTransaction([oid3])
-        self.db.storeTransaction(tid1, objs1, txn1)
-        self.db.storeTransaction(tid2, objs2, txn2)
-        self.db.storeTransaction(tid3, objs3, txn3)
-        self.db.storeTransaction(tid4, objs4, txn4)
-        self.db.storeTransaction(tid5, objs5, txn5)
-        self.db.finishTransaction(tid1)
-        self.db.finishTransaction(tid2)
-        self.db.finishTransaction(tid3)
-        self.db.finishTransaction(tid4)
-        self.db.finishTransaction(tid5)
-        # Check full result
-        result = self.db.getObjectHistoryFrom(ZERO_OID, ZERO_TID, MAX_TID, 10,
-            2, 0)
-        self.assertEqual(result, {
-            oid1: [tid1, tid3],
-            oid2: [tid2, tid4],
-        })
-        # Lower bound is inclusive
-        result = self.db.getObjectHistoryFrom(oid1, tid1, MAX_TID, 10, 2, 0)
-        self.assertEqual(result, {
-            oid1: [tid1, tid3],
-            oid2: [tid2, tid4],
-        })
-        # Upper bound is inclusive
-        result = self.db.getObjectHistoryFrom(ZERO_OID, ZERO_TID, tid3, 10,
-            2, 0)
-        self.assertEqual(result, {
-            oid1: [tid1, tid3],
-            oid2: [tid2],
-        })
-        # Length is total number of serials
-        result = self.db.getObjectHistoryFrom(ZERO_OID, ZERO_TID, MAX_TID, 3,
-            2, 0)
-        self.assertEqual(result, {
-            oid1: [tid1, tid3],
-            oid2: [tid2],
-        })
-        # Partition constraints are honored
-        result = self.db.getObjectHistoryFrom(ZERO_OID, ZERO_TID, MAX_TID, 10,
-            2, 1)
-        self.assertEqual(result, {
-            oid3: [tid5],
-        })
-
     def _storeTransactions(self, count):
         # use OID generator to know result of tid % N
         tid_list = self.getOIDs(count)
@@ -539,44 +453,71 @@ class StorageDBTests(NeoUnitTestBase):
         tid1, tid2, tid3, tid4 = self._storeTransactions(4)
         # get tids
         # - all partitions
-        result = self.db.getTIDList(0, 4, 2, [0, 1])
+        result = self.db.getTIDList(0, 4, [0, 1])
         self.checkSet(result, [tid1, tid2, tid3, tid4])
         # - one partition
-        result = self.db.getTIDList(0, 4, 2, [0])
+        result = self.db.getTIDList(0, 4, [0])
         self.checkSet(result, [tid1, tid3])
-        result = self.db.getTIDList(0, 4, 2, [1])
+        result = self.db.getTIDList(0, 4, [1])
         self.checkSet(result, [tid2, tid4])
         # get a subset of tids
-        result = self.db.getTIDList(0, 1, 2, [0])
+        result = self.db.getTIDList(0, 1, [0])
         self.checkSet(result, [tid3]) # desc order
-        result = self.db.getTIDList(1, 1, 2, [1])
+        result = self.db.getTIDList(1, 1, [1])
         self.checkSet(result, [tid2])
-        result = self.db.getTIDList(2, 2, 2, [0])
+        result = self.db.getTIDList(2, 2, [0])
         self.checkSet(result, [])
 
     def test_getReplicationTIDList(self):
         self.setNumPartitions(2, True)
         tid1, tid2, tid3, tid4 = self._storeTransactions(4)
-        # get tids
-        # - all
-        result = self.db.getReplicationTIDList(ZERO_TID, MAX_TID, 10, 2, 0)
-        self.checkSet(result, [tid1, tid3])
         # - one partition
-        result = self.db.getReplicationTIDList(ZERO_TID, MAX_TID, 10, 2, 0)
+        result = self.db.getReplicationTIDList(ZERO_TID, MAX_TID, 10, 0)
         self.checkSet(result, [tid1, tid3])
         # - another partition
-        result = self.db.getReplicationTIDList(ZERO_TID, MAX_TID, 10, 2, 1)
+        result = self.db.getReplicationTIDList(ZERO_TID, MAX_TID, 10, 1)
         self.checkSet(result, [tid2, tid4])
         # - min_tid is inclusive
-        result = self.db.getReplicationTIDList(tid3, MAX_TID, 10, 2, 0)
+        result = self.db.getReplicationTIDList(tid3, MAX_TID, 10, 0)
         self.checkSet(result, [tid3])
         # - max tid is inclusive
-        result = self.db.getReplicationTIDList(ZERO_TID, tid2, 10, 2, 0)
+        result = self.db.getReplicationTIDList(ZERO_TID, tid2, 10, 0)
         self.checkSet(result, [tid1])
         # - limit
-        result = self.db.getReplicationTIDList(ZERO_TID, MAX_TID, 1, 2, 0)
+        result = self.db.getReplicationTIDList(ZERO_TID, MAX_TID, 1, 0)
         self.checkSet(result, [tid1])
 
+    def test_checkRange(self):
+        def check(trans, obj, *args):
+            self.assertEqual(trans, self.db.checkTIDRange(*args))
+            self.assertEqual(obj, self.db.checkSerialRange(*(args+(ZERO_OID,))))
+        self.setNumPartitions(2, True)
+        tid1, tid2, tid3, tid4 = self._storeTransactions(4)
+        z = 0, ZERO_HASH, ZERO_TID, ZERO_HASH, ZERO_OID
+        # - one partition
+        check((2, a2b_hex('84320eb8dbbe583f67055c15155ab6794f11654d'), tid3),
+            z,
+            0, 10, ZERO_TID, MAX_TID)
+        # - another partition
+        check((2, a2b_hex('1f02f98cf775a9e0ce9252ff5972dce728c4ddb0'), tid4),
+            (4, a2b_hex('e5b47bddeae2096220298df686737d939a27d736'), tid4,
+                a2b_hex('1e9093698424b5370e19acd2d5fc20dcd56a32cd'), p64(1)),
+            1, 10, ZERO_TID, MAX_TID)
+        self.assertEqual(
+            (3, a2b_hex('b85e2d4914e22b5ad3b82b312b3dc405dc17dcb8'), tid4,
+                a2b_hex('1b6d73ecdc064595fe915a5c26da06b195caccaa'), p64(1)),
+            self.db.checkSerialRange(1, 10, ZERO_TID, MAX_TID, p64(2)))
+        # - min_tid is inclusive
+        check((1, a2b_hex('da4b9237bacccdf19c0760cab7aec4a8359010b0'), tid3),
+            z,
+            0, 10, tid3, MAX_TID)
+        # - max tid is inclusive
+        x = 1, a2b_hex('b6589fc6ab0dc82cf12099d1c2d40ab994e8410c'), tid1
+        check(x, z, 0, 10, ZERO_TID, tid2)
+        # - limit
+        y = 1, a2b_hex('356a192b7913b04c54574d18c28d46e6395428ab'), tid2
+        check(y, x + y[1:], 1, 1, ZERO_TID, MAX_TID)
+
     def test_findUndoTID(self):
         self.setNumPartitions(4, True)
         db = self.db
@@ -586,10 +527,8 @@ class StorageDBTests(NeoUnitTestBase):
         tid4 = self.getNextTID()
         tid5 = self.getNextTID()
         oid1 = self.getOID(1)
-        foo = "3" * 20
-        bar = "4" * 20
-        db.storeData(foo, 'foo', 0)
-        db.storeData(bar, 'bar', 0)
+        foo = db.storeData("3" * 20, 'foo', 0)
+        bar = db.storeData("4" * 20, 'bar', 0)
         db.unlockData((foo, bar))
         db.storeTransaction(
             tid1, (
