@@ -12,13 +12,12 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
-
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import random
 import unittest
-from mock import Mock
-from neo.tests import NeoUnitTestBase
+from mock import Mock, ReturnValues
+from .. import NeoUnitTestBase
 from neo.storage.transactions import Transaction, TransactionManager
 from neo.storage.transactions import ConflictError, DelayedError
 
@@ -26,7 +25,7 @@ from neo.storage.transactions import ConflictError, DelayedError
 class TransactionTests(NeoUnitTestBase):
 
     def testInit(self):
-        uuid = self.getNewUUID()
+        uuid = self.getClientUUID()
         ttid = self.getNextTID()
         tid = self.getNextTID()
         txn = Transaction(uuid, ttid)
@@ -38,15 +37,8 @@ class TransactionTests(NeoUnitTestBase):
         self.assertEqual(txn.getObjectList(), [])
         self.assertEqual(txn.getOIDList(), [])
 
-    def testRepr(self):
-        """ Just check if the __repr__ implementation will not raise """
-        uuid = self.getNewUUID()
-        tid = self.getNextTID()
-        txn = Transaction(uuid, tid)
-        repr(txn)
-
     def testLock(self):
-        txn = Transaction(self.getNewUUID(), self.getNextTID())
+        txn = Transaction(self.getClientUUID(), self.getNextTID())
         self.assertFalse(txn.isLocked())
         txn.lock()
         self.assertTrue(txn.isLocked())
@@ -54,14 +46,16 @@ class TransactionTests(NeoUnitTestBase):
         self.assertRaises(AssertionError, txn.lock)
 
     def testTransaction(self):
-        txn = Transaction(self.getNewUUID(), self.getNextTID())
+        txn = Transaction(self.getClientUUID(), self.getNextTID())
+        repr(txn) # check __repr__ does not raise
         oid_list = [self.getOID(1), self.getOID(2)]
         txn_info = (oid_list, 'USER', 'DESC', 'EXT', False)
         txn.prepare(*txn_info)
-        self.assertEqual(txn.getTransactionInformations(), txn_info)
+        self.assertEqual(txn.getTransactionInformations(),
+                         txn_info + (txn.getTTID(),))
 
     def testObjects(self):
-        txn = Transaction(self.getNewUUID(), self.getNextTID())
+        txn = Transaction(self.getClientUUID(), self.getNextTID())
         oid1, oid2 = self.getOID(1), self.getOID(2)
         object1 = oid1, "0" * 20, None
         object2 = oid2, "1" * 20, None
@@ -77,7 +71,7 @@ class TransactionTests(NeoUnitTestBase):
     def test_getObject(self):
         oid_1 = self.getOID(1)
         oid_2 = self.getOID(2)
-        txn = Transaction(self.getNewUUID(), self.getNextTID())
+        txn = Transaction(self.getClientUUID(), self.getNextTID())
         object_info = oid_1, None, None
         txn.addObject(*object_info)
         self.assertRaises(KeyError, txn.getObject, oid_2)
@@ -125,7 +119,9 @@ class TransactionManagerTests(NeoUnitTestBase):
 
     def testSimpleCase(self):
         """ One node, one transaction, not abort """
-        uuid = self.getNewUUID()
+        data_id_list = random.random(), random.random()
+        self.app.dm.mockAddReturnValues(storeData=ReturnValues(*data_id_list))
+        uuid = self.getClientUUID()
         ttid = self.getNextTID()
         tid, txn = self._getTransaction()
         serial1, object1 = self._getObject(1)
@@ -137,16 +133,16 @@ class TransactionManagerTests(NeoUnitTestBase):
         self.assertTrue(ttid in self.manager)
         self.manager.lock(ttid, tid, txn[0])
         self._checkTransactionStored(tid, [
-            (object1[0], object1[2], object1[4]),
-            (object2[0], object2[2], object2[4]),
-            ], txn)
+            (object1[0], data_id_list[0], object1[4]),
+            (object2[0], data_id_list[1], object2[4]),
+            ], txn + (ttid,))
         self.manager.unlock(ttid)
         self.assertFalse(ttid in self.manager)
         self._checkTransactionFinished(tid)
 
     def testDelayed(self):
         """ Two transactions, the first cause the second to be delayed """
-        uuid = self.getNewUUID()
+        uuid = self.getClientUUID()
         ttid1 = self.getNextTID()
         ttid2 = self.getNextTID()
         tid1, txn1 = self._getTransaction()
@@ -167,7 +163,7 @@ class TransactionManagerTests(NeoUnitTestBase):
 
     def testUnresolvableConflict(self):
         """ A newer transaction has already modified an object """
-        uuid = self.getNewUUID()
+        uuid = self.getClientUUID()
         ttid1 = self.getNextTID()
         ttid2 = self.getNextTID()
         tid1, txn1 = self._getTransaction()
@@ -188,7 +184,7 @@ class TransactionManagerTests(NeoUnitTestBase):
 
     def testResolvableConflict(self):
         """ Try to store an object with the lastest revision """
-        uuid = self.getNewUUID()
+        uuid = self.getClientUUID()
         tid, txn = self._getTransaction()
         serial, obj = self._getObject(1)
         next_serial = self.getNextTID(serial)
@@ -201,8 +197,8 @@ class TransactionManagerTests(NeoUnitTestBase):
 
     def testLockDelayed(self):
         """ Check lock delay """
-        uuid1 = self.getNewUUID()
-        uuid2 = self.getNewUUID()
+        uuid1 = self.getClientUUID()
+        uuid2 = self.getClientUUID()
         self.assertNotEqual(uuid1, uuid2)
         ttid1 = self.getNextTID()
         ttid2 = self.getNextTID()
@@ -228,8 +224,8 @@ class TransactionManagerTests(NeoUnitTestBase):
 
     def testLockConflict(self):
         """ Check lock conflict """
-        uuid1 = self.getNewUUID()
-        uuid2 = self.getNewUUID()
+        uuid1 = self.getClientUUID()
+        uuid2 = self.getClientUUID()
         self.assertNotEqual(uuid1, uuid2)
         ttid1 = self.getNextTID()
         ttid2 = self.getNextTID()
@@ -255,7 +251,7 @@ class TransactionManagerTests(NeoUnitTestBase):
 
     def testAbortUnlocked(self):
         """ Abort a non-locked transaction """
-        uuid = self.getNewUUID()
+        uuid = self.getClientUUID()
         tid, txn = self._getTransaction()
         serial, obj = self._getObject(1)
         self.manager.register(uuid, tid)
@@ -270,7 +266,7 @@ class TransactionManagerTests(NeoUnitTestBase):
 
     def testAbortLockedDoNothing(self):
         """ Try to abort a locked transaction """
-        uuid = self.getNewUUID()
+        uuid = self.getClientUUID()
         ttid = self.getNextTID()
         tid, txn = self._getTransaction()
         self.manager.register(uuid, ttid)
@@ -287,8 +283,8 @@ class TransactionManagerTests(NeoUnitTestBase):
 
     def testAbortForNode(self):
         """ Abort transaction for a node """
-        uuid1 = self.getNewUUID()
-        uuid2 = self.getNewUUID()
+        uuid1 = self.getClientUUID()
+        uuid2 = self.getClientUUID()
         self.assertNotEqual(uuid1, uuid2)
         ttid1 = self.getNextTID()
         ttid2 = self.getNextTID()
@@ -317,7 +313,7 @@ class TransactionManagerTests(NeoUnitTestBase):
 
     def testReset(self):
         """ Reset the manager """
-        uuid = self.getNewUUID()
+        uuid = self.getClientUUID()
         tid, txn = self._getTransaction()
         ttid = self.getNextTID()
         self.manager.register(uuid, ttid)
@@ -331,7 +327,9 @@ class TransactionManagerTests(NeoUnitTestBase):
             self.assertFalse(self.manager.loadLocked(oid))
 
     def test_getObjectFromTransaction(self):
-        uuid = self.getNewUUID()
+        data_id = random.random()
+        self.app.dm.mockAddReturnValues(storeData=ReturnValues(data_id))
+        uuid = self.getClientUUID()
         tid1, txn1 = self._getTransaction()
         tid2, txn2 = self._getTransaction()
         serial1, obj1 = self._getObject(1)
@@ -343,10 +341,10 @@ class TransactionManagerTests(NeoUnitTestBase):
         self.assertEqual(self.manager.getObjectFromTransaction(tid1, obj2[0]),
             None)
         self.assertEqual(self.manager.getObjectFromTransaction(tid1, obj1[0]),
-            (obj1[0], obj1[2], obj1[4]))
+            (obj1[0], data_id, obj1[4]))
 
     def test_getLockingTID(self):
-        uuid = self.getNewUUID()
+        uuid = self.getClientUUID()
         serial1, obj1 = self._getObject(1)
         oid1 = obj1[0]
         tid1, txn1 = self._getTransaction()
@@ -359,7 +357,7 @@ class TransactionManagerTests(NeoUnitTestBase):
         ram_serial = self.getNextTID()
         oid = self.getOID(1)
         orig_serial = self.getNextTID()
-        uuid = self.getNewUUID()
+        uuid = self.getClientUUID()
         locking_serial = self.getNextTID()
         other_serial = self.getNextTID()
         new_serial = self.getNextTID()
