@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2009-2010  Nexedi SA
+# Copyright (C) 2009-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -13,8 +13,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 import unittest
 from time import time
 from mock import Mock
@@ -22,13 +21,12 @@ from neo.lib.connection import ListeningConnection, Connection, \
      ClientConnection, ServerConnection, MTClientConnection, \
      HandlerSwitcher, CRITICAL_TIMEOUT
 from neo.lib.connector import getConnectorHandler, registerConnectorHandler
-from neo.tests import DoNothingConnector
+from . import DoNothingConnector
 from neo.lib.connector import ConnectorException, ConnectorTryAgainException, \
      ConnectorInProgressException, ConnectorConnectionRefusedException
 from neo.lib.handler import EventHandler
-from neo.lib.protocol import Packets, ParserState
-from neo.tests import NeoUnitTestBase
-from neo.lib.util import ReadBuffer
+from neo.lib.protocol import Packets, PACKET_HEADER_FORMAT
+from . import NeoUnitTestBase
 from neo.lib.locking import Queue
 
 class ConnectionTests(NeoUnitTestBase):
@@ -39,6 +37,7 @@ class ConnectionTests(NeoUnitTestBase):
         self.em = Mock({'__repr__': 'Fake Em'})
         self.handler = Mock({'__repr__': 'Fake Handler'})
         self.address = ("127.0.0.7", 93413)
+        self.node = Mock({'getAddress': self.address})
 
     def _makeListeningConnection(self, addr):
         # create instance after monkey patches
@@ -54,7 +53,7 @@ class ConnectionTests(NeoUnitTestBase):
     def _makeClientConnection(self):
         self.connector = DoNothingConnector()
         return ClientConnection(event_manager=self.em, handler=self.handler,
-                connector=self.connector, addr=self.address)
+                connector=self.connector, node=self.node)
 
     def _makeServerConnection(self):
         self.connector = DoNothingConnector()
@@ -164,15 +163,18 @@ class ConnectionTests(NeoUnitTestBase):
             return self, ('', 0)
         DoNothingConnector.getNewConnection = getNewConnection
         addr = ("127.0.0.7", 93413)
-        bc = self._makeListeningConnection(addr=addr)
-        self.assertEqual(bc.getAddress(), addr)
-        self._checkRegistered()
-        self._checkReaderAdded()
-        self._checkMakeListeningConnection()
-        # test readable
-        bc.readable()
-        self._checkGetNewConnection()
-        self._checkConnectionAccepted()
+        try:
+            bc = self._makeListeningConnection(addr=addr)
+            self.assertEqual(bc.getAddress(), addr)
+            self._checkRegistered()
+            self._checkReaderAdded()
+            self._checkMakeListeningConnection()
+            # test readable
+            bc.readable()
+            self._checkGetNewConnection()
+            self._checkConnectionAccepted()
+        finally:
+            del DoNothingConnector.getNewConnection
 
     def test_02_ListeningConnection2(self):
         # test with exception raise when getting new connection
@@ -180,15 +182,18 @@ class ConnectionTests(NeoUnitTestBase):
             raise ConnectorTryAgainException
         DoNothingConnector.getNewConnection = getNewConnection
         addr = ("127.0.0.7", 93413)
-        bc = self._makeListeningConnection(addr=addr)
-        self.assertEqual(bc.getAddress(), addr)
-        self._checkRegistered()
-        self._checkReaderAdded()
-        self._checkMakeListeningConnection()
-        # test readable
-        bc.readable()
-        self._checkGetNewConnection(1)
-        self._checkConnectionAccepted(0)
+        try:
+            bc = self._makeListeningConnection(addr=addr)
+            self.assertEqual(bc.getAddress(), addr)
+            self._checkRegistered()
+            self._checkReaderAdded()
+            self._checkMakeListeningConnection()
+            # test readable
+            bc.readable()
+            self._checkGetNewConnection(1)
+            self._checkConnectionAccepted(0)
+        finally:
+            del DoNothingConnector.getNewConnection
 
     def test_03_Connection(self):
         bc = self._makeConnection()
@@ -201,7 +206,7 @@ class ConnectionTests(NeoUnitTestBase):
         # test uuid
         self.assertEqual(bc.uuid, None)
         self.assertEqual(bc.getUUID(), None)
-        uuid = self.getNewUUID()
+        uuid = self.getNewUUID(None)
         bc.setUUID(uuid)
         self.assertEqual(bc.getUUID(), uuid)
         # test next id
@@ -229,48 +234,60 @@ class ConnectionTests(NeoUnitTestBase):
         def receive(self):
             return "testdata"
         DoNothingConnector.receive = receive
-        bc = self._makeConnection()
-        self._checkReadBuf(bc, '')
-        bc._recv()
-        self._checkReadBuf(bc, 'testdata')
+        try:
+            bc = self._makeConnection()
+            self._checkReadBuf(bc, '')
+            bc._recv()
+            self._checkReadBuf(bc, 'testdata')
+        finally:
+            del DoNothingConnector.receive
 
     def test_Connection_recv2(self):
         # patch receive method to raise try again
         def receive(self):
             raise ConnectorTryAgainException
         DoNothingConnector.receive = receive
-        bc = self._makeConnection()
-        self._checkReadBuf(bc, '')
-        bc._recv()
-        self._checkReadBuf(bc, '')
-        self._checkConnectionClosed(0)
-        self._checkUnregistered(0)
+        try:
+            bc = self._makeConnection()
+            self._checkReadBuf(bc, '')
+            bc._recv()
+            self._checkReadBuf(bc, '')
+            self._checkConnectionClosed(0)
+            self._checkUnregistered(0)
+        finally:
+            del DoNothingConnector.receive
 
     def test_Connection_recv3(self):
         # patch receive method to raise ConnectorConnectionRefusedException
         def receive(self):
             raise ConnectorConnectionRefusedException
         DoNothingConnector.receive = receive
-        bc = self._makeConnection()
-        self._checkReadBuf(bc, '')
-        # fake client connection instance with connecting attribute
-        bc.connecting = True
-        bc._recv()
-        self._checkReadBuf(bc, '')
-        self._checkConnectionFailed(1)
-        self._checkUnregistered(1)
+        try:
+            bc = self._makeConnection()
+            self._checkReadBuf(bc, '')
+            # fake client connection instance with connecting attribute
+            bc.connecting = True
+            bc._recv()
+            self._checkReadBuf(bc, '')
+            self._checkConnectionFailed(1)
+            self._checkUnregistered(1)
+        finally:
+            del DoNothingConnector.receive
 
     def test_Connection_recv4(self):
         # patch receive method to raise any other connector error
         def receive(self):
             raise ConnectorException
         DoNothingConnector.receive = receive
-        bc = self._makeConnection()
-        self._checkReadBuf(bc, '')
-        self.assertRaises(ConnectorException, bc._recv)
-        self._checkReadBuf(bc, '')
-        self._checkConnectionClosed(1)
-        self._checkUnregistered(1)
+        try:
+            bc = self._makeConnection()
+            self._checkReadBuf(bc, '')
+            self.assertRaises(ConnectorException, bc._recv)
+            self._checkReadBuf(bc, '')
+            self._checkConnectionClosed(1)
+            self._checkUnregistered(1)
+        finally:
+            del DoNothingConnector.receive
 
     def test_Connection_send1(self):
         # no data, nothing done
@@ -287,96 +304,113 @@ class ConnectionTests(NeoUnitTestBase):
         def send(self, data):
             return len(data)
         DoNothingConnector.send = send
-        bc = self._makeConnection()
-        self._checkWriteBuf(bc, '')
-        bc.write_buf = ["testdata"]
-        bc._send()
-        self._checkSend(1, "testdata")
-        self._checkWriteBuf(bc, '')
-        self._checkConnectionClosed(0)
-        self._checkUnregistered(0)
+        try:
+            bc = self._makeConnection()
+            self._checkWriteBuf(bc, '')
+            bc.write_buf = ["testdata"]
+            bc._send()
+            self._checkSend(1, "testdata")
+            self._checkWriteBuf(bc, '')
+            self._checkConnectionClosed(0)
+            self._checkUnregistered(0)
+        finally:
+            del DoNothingConnector.send
 
     def test_Connection_send3(self):
         # send part of the data
         def send(self, data):
             return len(data)/2
         DoNothingConnector.send = send
-        bc = self._makeConnection()
-        self._checkWriteBuf(bc, '')
-        bc.write_buf = ["testdata"]
-        bc._send()
-        self._checkSend(1, "testdata")
-        self._checkWriteBuf(bc, 'data')
-        self._checkConnectionClosed(0)
-        self._checkUnregistered(0)
+        try:
+            bc = self._makeConnection()
+            self._checkWriteBuf(bc, '')
+            bc.write_buf = ["testdata"]
+            bc._send()
+            self._checkSend(1, "testdata")
+            self._checkWriteBuf(bc, 'data')
+            self._checkConnectionClosed(0)
+            self._checkUnregistered(0)
+        finally:
+            del DoNothingConnector.send
 
     def test_Connection_send4(self):
         # send multiple packet
         def send(self, data):
             return len(data)
         DoNothingConnector.send = send
-        bc = self._makeConnection()
-        self._checkWriteBuf(bc, '')
-        bc.write_buf = ["testdata", "second", "third"]
-        bc._send()
-        self._checkSend(1, "testdatasecondthird")
-        self._checkWriteBuf(bc, '')
-        self._checkConnectionClosed(0)
-        self._checkUnregistered(0)
+        try:
+            bc = self._makeConnection()
+            self._checkWriteBuf(bc, '')
+            bc.write_buf = ["testdata", "second", "third"]
+            bc._send()
+            self._checkSend(1, "testdatasecondthird")
+            self._checkWriteBuf(bc, '')
+            self._checkConnectionClosed(0)
+            self._checkUnregistered(0)
+        finally:
+            del DoNothingConnector.send
 
     def test_Connection_send5(self):
         # send part of multiple packet
         def send(self, data):
             return len(data)/2
         DoNothingConnector.send = send
-        bc = self._makeConnection()
-        self._checkWriteBuf(bc, '')
-        bc.write_buf = ["testdata", "second", "third"]
-        bc._send()
-        self._checkSend(1, "testdatasecondthird")
-        self._checkWriteBuf(bc, 'econdthird')
-        self._checkConnectionClosed(0)
-        self._checkUnregistered(0)
+        try:
+            bc = self._makeConnection()
+            self._checkWriteBuf(bc, '')
+            bc.write_buf = ["testdata", "second", "third"]
+            bc._send()
+            self._checkSend(1, "testdatasecondthird")
+            self._checkWriteBuf(bc, 'econdthird')
+            self._checkConnectionClosed(0)
+            self._checkUnregistered(0)
+        finally:
+            del DoNothingConnector.send
 
     def test_Connection_send6(self):
         # raise try again
         def send(self, data):
             raise ConnectorTryAgainException
         DoNothingConnector.send = send
-        bc = self._makeConnection()
-        self._checkWriteBuf(bc, '')
-        bc.write_buf = ["testdata", "second", "third"]
-        bc._send()
-        self._checkSend(1, "testdatasecondthird")
-        self._checkWriteBuf(bc, 'testdatasecondthird')
-        self._checkConnectionClosed(0)
-        self._checkUnregistered(0)
+        try:
+            bc = self._makeConnection()
+            self._checkWriteBuf(bc, '')
+            bc.write_buf = ["testdata", "second", "third"]
+            bc._send()
+            self._checkSend(1, "testdatasecondthird")
+            self._checkWriteBuf(bc, 'testdatasecondthird')
+            self._checkConnectionClosed(0)
+            self._checkUnregistered(0)
+        finally:
+            del DoNothingConnector.send
 
     def test_Connection_send7(self):
         # raise other error
         def send(self, data):
             raise ConnectorException
         DoNothingConnector.send = send
-        bc = self._makeConnection()
-        self._checkWriteBuf(bc, '')
-        bc.write_buf = ["testdata", "second", "third"]
-        self.assertRaises(ConnectorException, bc._send)
-        self._checkSend(1, "testdatasecondthird")
-        # connection closed -> buffers flushed
-        self._checkWriteBuf(bc, '')
-        self._checkReaderRemoved(1)
-        self._checkConnectionClosed(1)
-        self._checkUnregistered(1)
+        try:
+            bc = self._makeConnection()
+            self._checkWriteBuf(bc, '')
+            bc.write_buf = ["testdata", "second", "third"]
+            self.assertRaises(ConnectorException, bc._send)
+            self._checkSend(1, "testdatasecondthird")
+            # connection closed -> buffers flushed
+            self._checkWriteBuf(bc, '')
+            self._checkReaderRemoved(1)
+            self._checkConnectionClosed(1)
+            self._checkUnregistered(1)
+        finally:
+            del DoNothingConnector.send
 
     def test_07_Connection_addPacket(self):
         # new packet
-        p = Mock({"encode" : "testdata", "getId": 0})
-        p._body = ''
-        p.handler_method_name = 'testmethod'
+        p = Packets.Ping()
+        p._id = 0
         bc = self._makeConnection()
         self._checkWriteBuf(bc, '')
         bc._addPacket(p)
-        self._checkWriteBuf(bc, 'testdata')
+        self._checkWriteBuf(bc, PACKET_HEADER_FORMAT.pack(0, p._code, 10))
         self._checkWriterAdded(1)
 
     def test_Connection_analyse1(self):
@@ -388,17 +422,7 @@ class ConnectionTests(NeoUnitTestBase):
         self._checkPacketReceived(0)
         self._checkReadBuf(bc, '')
 
-        # give some data to analyse
-        master_list = (
-                (("127.0.0.1", 2135), self.getNewUUID()),
-                (("127.0.0.1", 2135), self.getNewUUID()),
-                (("127.0.0.1", 2235), self.getNewUUID()),
-                (("127.0.0.1", 2134), self.getNewUUID()),
-                (("127.0.0.1", 2335), self.getNewUUID()),
-                (("127.0.0.1", 2133), self.getNewUUID()),
-                (("127.0.0.1", 2435), self.getNewUUID()),
-                (("127.0.0.1", 2132), self.getNewUUID()))
-        p = Packets.AnswerPrimary(self.getNewUUID(), master_list)
+        p = Packets.AnswerPrimary(self.getNewUUID(None))
         p.setId(1)
         p_data = ''.join(p.encode())
         data_edge = len(p_data) - 1
@@ -425,30 +449,10 @@ class ConnectionTests(NeoUnitTestBase):
         # give multiple packet
         bc = self._makeConnection()
         bc._queue = Mock()
-        # packet 1
-        master_list = (
-                (("127.0.0.1", 2135), self.getNewUUID()),
-                (("127.0.0.1", 2135), self.getNewUUID()),
-                (("127.0.0.1", 2235), self.getNewUUID()),
-                (("127.0.0.1", 2134), self.getNewUUID()),
-                (("127.0.0.1", 2335), self.getNewUUID()),
-                (("127.0.0.1", 2133), self.getNewUUID()),
-                (("127.0.0.1", 2435), self.getNewUUID()),
-                (("127.0.0.1", 2132), self.getNewUUID()))
-        p1 = Packets.AnswerPrimary(self.getNewUUID(), master_list)
+        p1 = Packets.AnswerPrimary(self.getNewUUID(None))
         p1.setId(1)
         self._appendPacketToReadBuf(bc, p1)
-        # packet 2
-        master_list = (
-                (("127.0.0.1", 2135), self.getNewUUID()),
-                (("127.0.0.1", 2135), self.getNewUUID()),
-                (("127.0.0.1", 2235), self.getNewUUID()),
-                (("127.0.0.1", 2134), self.getNewUUID()),
-                (("127.0.0.1", 2335), self.getNewUUID()),
-                (("127.0.0.1", 2133), self.getNewUUID()),
-                (("127.0.0.1", 2435), self.getNewUUID()),
-                (("127.0.0.1", 2132), self.getNewUUID()))
-        p2 = Packets.AnswerPrimary( self.getNewUUID(), master_list)
+        p2 = Packets.AnswerPrimary( self.getNewUUID(None))
         p2.setId(2)
         self._appendPacketToReadBuf(bc, p2)
         self.assertEqual(len(bc.read_buf), len(p1) + len(p2))
@@ -472,27 +476,18 @@ class ConnectionTests(NeoUnitTestBase):
     def test_Connection_analyse3(self):
         # give a bad packet, won't be decoded
         bc = self._makeConnection()
-        bc._queue = Mock()
-        self._appendToReadBuf(bc, 'datadatadatadata')
+        p = Packets.Ping()
+        p.setId(1)
+        self._appendToReadBuf(bc, '%s%sdatadatadatadata' % p.encode())
         bc.analyse()
-        self.assertEqual(len(bc._queue.mockGetNamedCalls("append")), 0)
-        self.assertEqual(
-            len(self.handler.mockGetNamedCalls("_packetMalformed")), 1)
+        self._checkPacketReceived(1) # ping packet
+        self._checkClose(1) # malformed packet
 
     def test_Connection_analyse4(self):
         # give an expected packet
         bc = self._makeConnection()
         bc._queue = Mock()
-        master_list = (
-                (("127.0.0.1", 2135), self.getNewUUID()),
-                (("127.0.0.1", 2135), self.getNewUUID()),
-                (("127.0.0.1", 2235), self.getNewUUID()),
-                (("127.0.0.1", 2134), self.getNewUUID()),
-                (("127.0.0.1", 2335), self.getNewUUID()),
-                (("127.0.0.1", 2133), self.getNewUUID()),
-                (("127.0.0.1", 2435), self.getNewUUID()),
-                (("127.0.0.1", 2132), self.getNewUUID()))
-        p = Packets.AnswerPrimary(self.getNewUUID(), master_list)
+        p = Packets.AnswerPrimary(self.getNewUUID(None))
         p.setId(1)
         self._appendPacketToReadBuf(bc, p)
         bc.analyse()
@@ -506,115 +501,118 @@ class ConnectionTests(NeoUnitTestBase):
         self._checkReadBuf(bc, '')
 
     def test_Connection_writable1(self):
-        # with  pending operation after send
+        # with pending operation after send
         def send(self, data):
             return len(data)/2
         DoNothingConnector.send = send
-        bc = self._makeConnection()
-        self._checkWriteBuf(bc, '')
-        bc.write_buf = ["testdata"]
-        self.assertTrue(bc.pending())
-        self.assertFalse(bc.aborted)
-        bc.writable()
-        # test send was called
-        self._checkSend(1, "testdata")
-        self.assertEqual(''.join(bc.write_buf), "data")
-        self._checkConnectionClosed(0)
-        self._checkUnregistered(0)
-        # pending, so nothing called
-        self.assertTrue(bc.pending())
-        self.assertFalse(bc.aborted)
-        self._checkWriterRemoved(0)
-        self._checkReaderRemoved(0)
-        self._checkShutdown(0)
-        self._checkClose(0)
+        try:
+            bc = self._makeConnection()
+            self._checkWriteBuf(bc, '')
+            bc.write_buf = ["testdata"]
+            self.assertTrue(bc.pending())
+            self.assertFalse(bc.aborted)
+            bc.writable()
+            # test send was called
+            self._checkSend(1, "testdata")
+            self._checkWriteBuf(bc, "data")
+            self._checkConnectionClosed(0)
+            self._checkClose(0)
+            self._checkUnregistered(0)
+            # pending, so nothing called
+            self.assertTrue(bc.pending())
+            self._checkWriterRemoved(0)
+            self._checkReaderRemoved(0)
+            self._checkShutdown(0)
+            self._checkClose(0)
+        finally:
+            del DoNothingConnector.send
 
     def test_Connection_writable2(self):
-        # with no longer pending operation after send
+        # without pending operation after send
         def send(self, data):
             return len(data)
         DoNothingConnector.send = send
-        bc = self._makeConnection()
-        self._checkWriteBuf(bc, '')
-        bc.write_buf = ["testdata"]
-        self.assertTrue(bc.pending())
-        self.assertFalse(bc.aborted)
-        bc.writable()
-        # test send was called
-        self._checkSend(1, "testdata")
-        self._checkWriteBuf(bc, '')
-        self._checkClose(0)
-        self._checkUnregistered(0)
-        # nothing else pending, and aborted is false, so writer has been removed
-        self.assertFalse(bc.pending())
-        self.assertFalse(bc.aborted)
-        self._checkWriterRemoved(1)
-        self._checkReaderRemoved(0)
-        self._checkShutdown(0)
-        self._checkClose(0)
+        try:
+            bc = self._makeConnection()
+            self._checkWriteBuf(bc, '')
+            bc.write_buf = ["testdata"]
+            self.assertTrue(bc.pending())
+            self.assertFalse(bc.aborted)
+            bc.writable()
+            # test send was called
+            self._checkSend(1, "testdata")
+            self._checkWriteBuf(bc, '')
+            self._checkConnectionClosed(0)
+            self._checkClose(0)
+            self._checkUnregistered(0)
+            # nothing else pending, so writer has been removed
+            self.assertFalse(bc.pending())
+            self._checkWriterRemoved(1)
+            self._checkReaderRemoved(0)
+            self._checkShutdown(0)
+            self._checkClose(0)
+        finally:
+            del DoNothingConnector.send
 
     def test_Connection_writable3(self):
-        # with no longer pending operation after send and aborted set to true
+        # without pending operation after send and aborted set to true
         def send(self, data):
             return len(data)
         DoNothingConnector.send = send
-        bc = self._makeConnection()
-        self._checkWriteBuf(bc, '')
-        bc.write_buf = ["testdata"]
-        self.assertTrue(bc.pending())
-        bc.abort()
-        self.assertTrue(bc.aborted)
-        bc.writable()
-        # test send was called
-        self._checkSend(1, "testdata")
-        self._checkWriteBuf(bc, '')
-        self._checkConnectionClosed(1)
-        self._checkUnregistered(1)
-        # nothing else pending, and aborted is false, so writer has been removed
-        self.assertFalse(bc.pending())
-        self.assertTrue(bc.aborted)
-        self._checkWriterRemoved(1)
-        self._checkReaderRemoved(1)
-        self._checkShutdown(1)
-        self._checkClose(1)
+        try:
+            bc = self._makeConnection()
+            self._checkWriteBuf(bc, '')
+            bc.write_buf = ["testdata"]
+            self.assertTrue(bc.pending())
+            bc.abort()
+            self.assertTrue(bc.aborted)
+            bc.writable()
+            # test send was called
+            self._checkSend(1, "testdata")
+            self._checkWriteBuf(bc, '')
+            self._checkConnectionClosed(1)
+            self._checkClose(1)
+            self._checkUnregistered(1)
+            # nothing else pending, so writer has been removed
+            self.assertFalse(bc.pending())
+            self._checkWriterRemoved(1)
+            self._checkReaderRemoved(1)
+            self._checkShutdown(1)
+            self._checkClose(1)
+        finally:
+            del DoNothingConnector.send
 
     def test_Connection_readable(self):
         # With aborted set to false
         # patch receive method to return data
         def receive(self):
-            master_list = ((("127.0.0.1", 2135), self.getNewUUID()),
-               (("127.0.0.1", 2136), self.getNewUUID()),
-               (("127.0.0.1", 2235), self.getNewUUID()),
-               (("127.0.0.1", 2134), self.getNewUUID()),
-               (("127.0.0.1", 2335), self.getNewUUID()),
-               (("127.0.0.1", 2133), self.getNewUUID()),
-               (("127.0.0.1", 2435), self.getNewUUID()),
-               (("127.0.0.1", 2132), self.getNewUUID()))
-            uuid = self.getNewUUID()
-            p = Packets.AnswerPrimary(uuid, master_list)
+            p = Packets.AnswerPrimary(self.getNewUUID(None))
             p.setId(1)
             return ''.join(p.encode())
         DoNothingConnector.receive = receive
-        bc = self._makeConnection()
-        bc._queue = Mock()
-        self._checkReadBuf(bc, '')
-        self.assertFalse(bc.aborted)
-        bc.readable()
-        # check packet decoded
-        self._checkReadBuf(bc, '')
-        self.assertEqual(len(bc._queue.mockGetNamedCalls("append")), 1)
-        call = bc._queue.mockGetNamedCalls("append")[0]
-        data = call.getParam(0)
-        self.assertEqual(type(data), Packets.AnswerPrimary)
-        self.assertEqual(data.getId(), 1)
-        self._checkReadBuf(bc, '')
-        # check not aborted
-        self.assertFalse(bc.aborted)
-        self._checkUnregistered(0)
-        self._checkWriterRemoved(0)
-        self._checkReaderRemoved(0)
-        self._checkShutdown(0)
-        self._checkClose(0)
+        try:
+            bc = self._makeConnection()
+            bc._queue = Mock()
+            self._checkReadBuf(bc, '')
+            self.assertFalse(bc.aborted)
+            bc.readable()
+            # check packet decoded
+            self._checkReadBuf(bc, '')
+            self.assertEqual(len(bc._queue.mockGetNamedCalls("append")), 1)
+            call = bc._queue.mockGetNamedCalls("append")[0]
+            data = call.getParam(0)
+            self.assertEqual(type(data), Packets.AnswerPrimary)
+            self.assertEqual(data.getId(), 1)
+            self._checkReadBuf(bc, '')
+            # check not aborted
+            self.assertFalse(bc.aborted)
+            self._checkUnregistered(0)
+            self._checkWriterRemoved(0)
+            self._checkReaderRemoved(0)
+            self._checkShutdown(0)
+            self._checkClose(0)
+        finally:
+            del DoNothingConnector.receive
 
     def test_ClientConnection_init1(self):
         # create a good client connection
@@ -686,32 +684,35 @@ class ConnectionTests(NeoUnitTestBase):
         DoNothingConnector.send = send
         DoNothingConnector.makeClientConnection = makeClientConnection
         try:
-            bc = self._makeClientConnection()
+            try:
+                bc = self._makeClientConnection()
+            finally:
+                DoNothingConnector.makeClientConnection = makeClientConnection_org
+            # check connector created and connection initialize
+            self.assertFalse(bc.connecting)
+            self._checkWriteBuf(bc, '')
+            bc.write_buf = ["testdata"]
+            self.assertTrue(bc.pending())
+            self.assertFalse(bc.aborted)
+            # call
+            self._checkConnectionCompleted(1)
+            self._checkReaderAdded(1)
+            bc.writable()
+            self.assertFalse(bc.pending())
+            self.assertFalse(bc.aborted)
+            self.assertFalse(bc.connecting)
+            self._checkSend(1, "testdata")
+            self._checkConnectionClosed(0)
+            self._checkConnectionCompleted(1)
+            self._checkConnectionFailed(0)
+            self._checkUnregistered(0)
+            self._checkReaderAdded(1)
+            self._checkWriterRemoved(1)
+            self._checkReaderRemoved(0)
+            self._checkShutdown(0)
+            self._checkClose(0)
         finally:
-            DoNothingConnector.makeClientConnection = makeClientConnection_org
-        # check connector created and connection initialize
-        self.assertFalse(bc.connecting)
-        self._checkWriteBuf(bc, '')
-        bc.write_buf = ["testdata"]
-        self.assertTrue(bc.pending())
-        self.assertFalse(bc.aborted)
-        # call
-        self._checkConnectionCompleted(1)
-        self._checkReaderAdded(1)
-        bc.writable()
-        self.assertFalse(bc.pending())
-        self.assertFalse(bc.aborted)
-        self.assertFalse(bc.connecting)
-        self._checkSend(1, "testdata")
-        self._checkConnectionClosed(0)
-        self._checkConnectionCompleted(1)
-        self._checkConnectionFailed(0)
-        self._checkUnregistered(0)
-        self._checkReaderAdded(1)
-        self._checkWriterRemoved(1)
-        self._checkReaderRemoved(0)
-        self._checkShutdown(0)
-        self._checkClose(0)
+            del DoNothingConnector.send
 
     def test_ClientConnection_writable2(self):
         # with a connecting connection, must not call parent's method
@@ -719,9 +720,11 @@ class ConnectionTests(NeoUnitTestBase):
         def getError(self):
             return True
         DoNothingConnector.getError = getError
-        bc = self._makeClientConnection()
+        try:
+            bc = self._makeClientConnection()
+        finally:
+            del DoNothingConnector.getError
         # check connector created and connection initialize
-        bc.connecting = True
         self._checkWriteBuf(bc, '')
         bc.write_buf = ["testdata"]
         self.assertTrue(bc.pending())
@@ -730,13 +733,13 @@ class ConnectionTests(NeoUnitTestBase):
         self._checkConnectionCompleted(1)
         self._checkReaderAdded(1)
         bc.writable()
-        self.assertTrue(bc.connecting)
+        self.assertFalse(bc.connecting)
         self.assertFalse(bc.pending())
         self.assertFalse(bc.aborted)
         self._checkWriteBuf(bc, '')
-        self._checkConnectionClosed(0)
+        self._checkConnectionClosed(1)
         self._checkConnectionCompleted(1)
-        self._checkConnectionFailed(1)
+        self._checkConnectionFailed(0)
         self._checkUnregistered(1)
         self._checkReaderAdded(1)
         self._checkWriterRemoved(1)
@@ -753,7 +756,7 @@ class ConnectionTests(NeoUnitTestBase):
         # test uuid
         self.assertEqual(bc.uuid, None)
         self.assertEqual(bc.getUUID(), None)
-        uuid = self.getNewUUID()
+        uuid = self.getNewUUID(None)
         bc.setUUID(uuid)
         self.assertEqual(bc.getUUID(), uuid)
         # test next id
@@ -843,7 +846,7 @@ class MTConnectionTests(ConnectionTests):
     def _makeClientConnection(self):
         self.connector = DoNothingConnector()
         return MTClientConnection(event_manager=self.em, handler=self.handler,
-                connector=self.connector, addr=self.address,
+                connector=self.connector, node=self.node,
                 dispatcher=self.dispatcher)
 
     def test_MTClientConnectionQueueParameter(self):
@@ -911,7 +914,7 @@ class HandlerSwitcherTests(NeoUnitTestBase):
         self.assertFalse(applied)
         self._checkCurrentHandler(self._handler)
         call_tracker = []
-        def packetReceived(conn, packet):
+        def packetReceived(conn, packet, kw):
             self._handlers.emit(self._makeRequest(2), 0, None)
             call_tracker.append(True)
         self._handler.packetReceived = packetReceived
@@ -999,7 +1002,7 @@ class HandlerSwitcherTests(NeoUnitTestBase):
         self._checkCurrentHandler(h3)
 
     def testHandleAnswer4(self):
-        # process in disorder
+        # process out of order
         r1 = self._makeRequest(1)
         r2 = self._makeRequest(2)
         r3 = self._makeRequest(3)
@@ -1022,7 +1025,7 @@ class HandlerSwitcherTests(NeoUnitTestBase):
         self._checkCurrentHandler(h)
 
     def testHandleUnexpected(self):
-        # process in disorder
+        # process out of order
         r1 = self._makeRequest(1)
         r2 = self._makeRequest(2)
         a2 = self._makeAnswer(2)
