@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2009-2010  Nexedi SA
+# Copyright (C) 2009-2012  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,20 +12,19 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import unittest
 from mock import Mock
 from neo.lib.protocol import NodeStates, CellStates
 from neo.lib.pt import Cell, PartitionTable, PartitionTableException
 from neo.lib.node import StorageNode
-from neo.tests import NeoUnitTestBase
+from . import NeoUnitTestBase
 
 class PartitionTableTests(NeoUnitTestBase):
 
     def test_01_Cell(self):
-        uuid = self.getNewUUID()
+        uuid = self.getStorageUUID()
         server = ("127.0.0.1", 19001)
         sn = StorageNode(Mock(), server, uuid)
         cell = Cell(sn)
@@ -49,7 +48,7 @@ class PartitionTableTests(NeoUnitTestBase):
         num_partitions = 5
         num_replicas = 2
         pt = PartitionTable(num_partitions, num_replicas)
-        uuid1 = self.getNewUUID()
+        uuid1 = self.getStorageUUID()
         server1 = ("127.0.0.1", 19001)
         sn1 = StorageNode(Mock(), server1, uuid1)
         for x in xrange(num_partitions):
@@ -130,7 +129,7 @@ class PartitionTableTests(NeoUnitTestBase):
         num_partitions = 5
         num_replicas = 2
         pt = PartitionTable(num_partitions, num_replicas)
-        uuid1 = self.getNewUUID()
+        uuid1 = self.getStorageUUID()
         server1 = ("127.0.0.1", 19001)
         sn1 = StorageNode(Mock(), server1, uuid1)
         for x in xrange(num_partitions):
@@ -170,19 +169,19 @@ class PartitionTableTests(NeoUnitTestBase):
         num_replicas = 2
         pt = PartitionTable(num_partitions, num_replicas)
         # add two kind of node, usable and unsable
-        uuid1 = self.getNewUUID()
+        uuid1 = self.getStorageUUID()
         server1 = ("127.0.0.1", 19001)
         sn1 = StorageNode(Mock(), server1, uuid1)
         pt.setCell(0, sn1, CellStates.UP_TO_DATE)
-        uuid2 = self.getNewUUID()
+        uuid2 = self.getStorageUUID()
         server2 = ("127.0.0.2", 19001)
         sn2 = StorageNode(Mock(), server2, uuid2)
         pt.setCell(0, sn2, CellStates.OUT_OF_DATE)
-        uuid3 = self.getNewUUID()
+        uuid3 = self.getStorageUUID()
         server3 = ("127.0.0.3", 19001)
         sn3 = StorageNode(Mock(), server3, uuid3)
         pt.setCell(0, sn3, CellStates.FEEDING)
-        uuid4 = self.getNewUUID()
+        uuid4 = self.getStorageUUID()
         server4 = ("127.0.0.4", 19001)
         sn4 = StorageNode(Mock(), server4, uuid4)
         pt.setCell(0, sn4, CellStates.DISCARDED) # won't be added
@@ -198,14 +197,6 @@ class PartitionTableTests(NeoUnitTestBase):
                 self.assertTrue(sn2 in all_nodes)
                 self.assertTrue(sn3 in all_nodes)
                 self.assertTrue(sn4 not in all_nodes)
-                # writable nodes
-                all_cell = pt.getCellList(0, writable=True)
-                all_nodes = [x.getNode() for x in all_cell]
-                self.assertEqual(len(all_cell), 3)
-                self.assertTrue(sn1 in all_nodes)
-                self.assertTrue(sn2 in all_nodes)
-                self.assertTrue(sn3 in all_nodes)
-                self.assertTrue(sn4 not in all_nodes)
                 # readable nodes
                 all_cell = pt.getCellList(0, readable=True)
                 all_nodes = [x.getNode() for x in all_cell]
@@ -214,14 +205,6 @@ class PartitionTableTests(NeoUnitTestBase):
                 self.assertTrue(sn2 not in all_nodes)
                 self.assertTrue(sn3 in all_nodes)
                 self.assertTrue(sn4 not in all_nodes)
-                # writable & readable nodes
-                all_cell = pt.getCellList(0, readable=True, writable=True)
-                all_nodes = [x.getNode() for x in all_cell]
-                self.assertEqual(len(all_cell), 2)
-                self.assertTrue(sn1 in all_nodes)
-                self.assertTrue(sn2 not in all_nodes)
-                self.assertTrue(sn3 in all_nodes)
-                self.assertTrue(sn4 not in all_nodes)
             else:
                 self.assertEqual(len(pt.getCellList(1, False)), 0)
                 self.assertEqual(len(pt.getCellList(1, True)), 0)
@@ -232,15 +215,15 @@ class PartitionTableTests(NeoUnitTestBase):
         num_replicas = 2
         pt = PartitionTable(num_partitions, num_replicas)
         # add two kind of node, usable and unsable
-        uuid1 = self.getNewUUID()
+        uuid1 = self.getStorageUUID()
         server1 = ("127.0.0.1", 19001)
         sn1 = StorageNode(Mock(), server1, uuid1)
         pt.setCell(0, sn1, CellStates.UP_TO_DATE)
-        uuid2 = self.getNewUUID()
+        uuid2 = self.getStorageUUID()
         server2 = ("127.0.0.2", 19001)
         sn2 = StorageNode(Mock(), server2, uuid2)
         pt.setCell(1, sn2, CellStates.OUT_OF_DATE)
-        uuid3 = self.getNewUUID()
+        uuid3 = self.getStorageUUID()
         server3 = ("127.0.0.3", 19001)
         sn3 = StorageNode(Mock(), server3, uuid3)
         pt.setCell(2, sn3, CellStates.FEEDING)
@@ -257,35 +240,31 @@ class PartitionTableTests(NeoUnitTestBase):
             self.assertEqual(len(part), 0)
         self.assertEqual(len(pt.count_dict), 0)
 
-    def test_07_getNodeList(self):
+    def test_07_getNodeSet(self):
         num_partitions = 5
         num_replicas = 2
         pt = PartitionTable(num_partitions, num_replicas)
         # add two kind of node, usable and unsable
-        uuid1 = self.getNewUUID()
+        uuid1 = self.getStorageUUID()
         server1 = ("127.0.0.1", 19001)
         sn1 = StorageNode(Mock(), server1, uuid1)
         pt.setCell(0, sn1, CellStates.UP_TO_DATE)
-        uuid2 = self.getNewUUID()
+        uuid2 = self.getStorageUUID()
         server2 = ("127.0.0.2", 19001)
         sn2 = StorageNode(Mock(), server2, uuid2)
         pt.setCell(0, sn2, CellStates.OUT_OF_DATE)
-        uuid3 = self.getNewUUID()
+        uuid3 = self.getStorageUUID()
         server3 = ("127.0.0.3", 19001)
         sn3 = StorageNode(Mock(), server3, uuid3)
         pt.setCell(0, sn3, CellStates.FEEDING)
-        uuid4 = self.getNewUUID()
+        uuid4 = self.getStorageUUID()
         server4 = ("127.0.0.4", 19001)
         sn4 = StorageNode(Mock(), server4, uuid4)
         pt.setCell(0, sn4, CellStates.DISCARDED) # won't be added
         # must get only two node as feeding and discarded not taken
         # into account
-        self.assertEqual(len(pt.getNodeList()), 2)
-        nodes = pt.getNodeList()
-        self.assertTrue(sn1 in nodes)
-        self.assertTrue(sn2 in nodes)
-        self.assertTrue(sn3 not in nodes)
-        self.assertTrue(sn4 not in nodes)
+        self.assertEqual(pt.getNodeSet(True), set((sn1, sn3)))
+        self.assertEqual(len(pt.getNodeSet()), 3)
 
     def test_08_filled(self):
         num_partitions = 5
@@ -295,7 +274,7 @@ class PartitionTableTests(NeoUnitTestBase):
         self.assertEqual(pt.num_filled_rows, 0)
         self.assertFalse(pt.filled())
         # adding a node in all partition
-        uuid1 = self.getNewUUID()
+        uuid1 = self.getStorageUUID()
         server1 = ("127.0.0.1", 19001)
         sn1 = StorageNode(Mock(), server1, uuid1)
         for x in xrange(num_partitions):
@@ -308,7 +287,7 @@ class PartitionTableTests(NeoUnitTestBase):
         num_replicas = 2
         pt = PartitionTable(num_partitions, num_replicas)
         # add two kind of node, usable and unsable
-        uuid1 = self.getNewUUID()
+        uuid1 = self.getStorageUUID()
         server1 = ("127.0.0.1", 19001)
         sn1 = StorageNode(Mock(), server1, uuid1)
         pt.setCell(0, sn1, CellStates.UP_TO_DATE)
@@ -325,7 +304,7 @@ class PartitionTableTests(NeoUnitTestBase):
         self.assertFalse(pt.filled())
         self.assertFalse(pt.operational())
         # adding a node in all partition
-        uuid1 = self.getNewUUID()
+        uuid1 = self.getStorageUUID()
         server1 = ("127.0.0.1", 19001)
         sn1 = StorageNode(Mock(), server1, uuid1)
         for x in xrange(num_partitions):
@@ -339,7 +318,7 @@ class PartitionTableTests(NeoUnitTestBase):
         self.assertFalse(pt.filled())
         self.assertFalse(pt.operational())
         # adding a node in all partition
-        uuid1 = self.getNewUUID()
+        uuid1 = self.getStorageUUID()
         server1 = ("127.0.0.1", 19001)
         sn1 = StorageNode(Mock(), server1, uuid1)
         for x in xrange(num_partitions):
@@ -354,7 +333,7 @@ class PartitionTableTests(NeoUnitTestBase):
         self.assertFalse(pt.filled())
         self.assertFalse(pt.operational())
         # adding a node in all partition
-        uuid1 = self.getNewUUID()
+        uuid1 = self.getStorageUUID()
         server1 = ("127.0.0.1", 19001)
         sn1 = StorageNode(Mock(), server1, uuid1)
         sn1.setState(NodeStates.TEMPORARILY_DOWN)
@@ -369,7 +348,7 @@ class PartitionTableTests(NeoUnitTestBase):
         self.assertFalse(pt.filled())
         self.assertFalse(pt.operational())
         # adding a node in all partition
-        uuid1 = self.getNewUUID()
+        uuid1 = self.getStorageUUID()
         server1 = ("127.0.0.1", 19001)
         sn1 = StorageNode(Mock(), server1, uuid1)
         for x in xrange(num_partitions):
@@ -383,18 +362,18 @@ class PartitionTableTests(NeoUnitTestBase):
         num_replicas = 2
         pt = PartitionTable(num_partitions, num_replicas)
         # add nodes
-        uuid1 = self.getNewUUID()
+        uuid1 = self.getStorageUUID()
         server1 = ("127.0.0.1", 19001)
         sn1 = StorageNode(Mock(), server1, uuid1)
         pt.setCell(0, sn1, CellStates.UP_TO_DATE)
         pt.setCell(1, sn1, CellStates.UP_TO_DATE)
         pt.setCell(2, sn1, CellStates.UP_TO_DATE)
-        uuid2 = self.getNewUUID()
+        uuid2 = self.getStorageUUID()
         server2 = ("127.0.0.2", 19001)
         sn2 = StorageNode(Mock(), server2, uuid2)
         pt.setCell(0, sn2, CellStates.UP_TO_DATE)
         pt.setCell(1, sn2, CellStates.UP_TO_DATE)
-        uuid3 = self.getNewUUID()
+        uuid3 = self.getStorageUUID()
         server3 = ("127.0.0.3", 19001)
         sn3 = StorageNode(Mock(), server3, uuid3)
         pt.setCell(0, sn3, CellStates.UP_TO_DATE)
@@ -421,35 +400,6 @@ class PartitionTableTests(NeoUnitTestBase):
         # unknwon row
         self.assertRaises(IndexError,  pt.getRow, 5)
 
-    def test_getNodeMap(self):
-        num_partitions = 5
-        num_replicas = 2
-        pt = PartitionTable(num_partitions, num_replicas)
-        uuid1 = self.getNewUUID()
-        uuid2 = self.getNewUUID()
-        uuid3 = self.getNewUUID()
-        sn1 = StorageNode(Mock(),("127.0.0.1", 19001) , uuid1)
-        pt.setCell(0, sn1, CellStates.UP_TO_DATE)
-        pt.setCell(1, sn1, CellStates.UP_TO_DATE)
-        pt.setCell(2, sn1, CellStates.UP_TO_DATE)
-        self.assertEqual(pt.getNodeMap(), {
-            sn1: [0, 1, 2],
-        })
-        sn2 = StorageNode(Mock(), ("127.0.0.2", 19001), uuid2)
-        pt.setCell(0, sn2, CellStates.UP_TO_DATE)
-        pt.setCell(1, sn2, CellStates.UP_TO_DATE)
-        self.assertEqual(pt.getNodeMap(), {
-            sn1: [0, 1, 2],
-            sn2: [0, 1],
-        })
-        sn3 = StorageNode(Mock(), ("127.0.0.3", 19001), uuid3)
-        pt.setCell(0, sn3, CellStates.UP_TO_DATE)
-        self.assertEqual(pt.getNodeMap(), {
-            sn1: [0, 1, 2],
-            sn2: [0, 1],
-            sn3: [0],
-        })
-
 if __name__ == '__main__':
     unittest.main()
 
