@@ -1,6 +1,6 @@
 
 #
-# Copyright (C) 2009-2010  Nexedi SA
+# Copyright (C) 2009-2011  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -13,8 +13,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import unittest
 try:
@@ -25,7 +24,7 @@ except ImportError:
 from ZODB.tests.PackableStorage import PackableUndoStorage
 from ZODB.tests.StorageTestBase import StorageTestBase
 
-from neo.tests.zodb import ZODBTestCase
+from . import ZODBTestCase
 
 class PackableTests(ZODBTestCase, StorageTestBase,
         PackableStorageWithOptionalGC, PackableUndoStorage):
