@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2009-2010  Nexedi SA
+# Copyright (C) 2009-2011  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,14 +12,13 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import unittest
 from ZODB.tests.ReadOnlyStorage import ReadOnlyStorage
 from ZODB.tests.StorageTestBase import StorageTestBase
 
-from neo.tests.zodb import ZODBTestCase
+from . import ZODBTestCase
 
 class ReadOnlyTests(ZODBTestCase, StorageTestBase, ReadOnlyStorage):
     pass
