@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2009-2010  Nexedi SA
+# Copyright (C) 2009-2011  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import unittest
 from ZODB.tests.VersionStorage import VersionStorage
@@ -21,7 +20,7 @@ from ZODB.tests.TransactionalUndoVersionStorage import \
          TransactionalUndoVersionStorage
 from ZODB.tests.StorageTestBase import StorageTestBase
 
-from neo.tests.zodb import ZODBTestCase
+from . import ZODBTestCase
 
 class VersionTests(ZODBTestCase, StorageTestBase, VersionStorage,
         TransactionalUndoVersionStorage):
